/* Copyright (c) 2012, 
 * LPC1850/4350 Demo Software
 * All rights reserved.
 * no warranties
 */
#include <stdio.h>

#include "lpclib.h"

#include "bsp-hitex4300.h"



#if defined(__ICCARM__)
    #define __stdout __iar_Stdout
    #define __stdin __iar_Stdin
#endif

#if defined(__GNUC__)
    #define __stdout (*stdout)
    #define __stdin (*stdin)
    #define _Handle _file
#endif

#if !defined(__ICCARM__)
struct __FILE {
    uint8_t _Handle;                                    /**< Dummy! */
};
#endif

volatile int32_t ITM_RxBuffer;                          /**< Declared in core_cm4.h */


#if LPCLIB_I2C
I2C_Handle i2c_bus;

const I2C_Config i2c_bitrate400k_config = {
    .opcode = I2C_OPCODE_SET_BITRATE,
    { .bitrate = 400000ul, }
};


#if BOARD_TYPE == BOARD_REVA2
PCA9502_Handle portExpander;

const PCA9502_Config portExpanderInit[] = {
    {.opcode = PCA9502_OPCODE_RESET, .continued = ENABLE, },
    {.opcode = PCA9502_OPCODE_SET_PIN_DIRECTION, .continued = ENABLE,
        {.pinFunc = {.pin = PCA9502_PIN0, .func = PCA9502_PINFUNC_OUTPUT}}},
    {.opcode = PCA9502_OPCODE_SET_PIN_DIRECTION, .continued = ENABLE,
        {.pinFunc = {.pin = PCA9502_PIN1, .func = PCA9502_PINFUNC_OUTPUT}}},
    {.opcode = PCA9502_OPCODE_SET_PIN_DIRECTION, .continued = ENABLE,
        {.pinFunc = {.pin = PCA9502_PIN2, .func = PCA9502_PINFUNC_OUTPUT}}},
    {.opcode = PCA9502_OPCODE_SET_PIN_DIRECTION, .continued = ENABLE,
        {.pinFunc = {.pin = PCA9502_PIN3, .func = PCA9502_PINFUNC_OUTPUT}}},
    {.opcode = PCA9502_OPCODE_SET_PIN_DIRECTION, .continued = ENABLE,
        {.pinFunc = {.pin = PCA9502_PIN4, .func = PCA9502_PINFUNC_INPUT, .enableInterrupt = ENABLE}}},
    {.opcode = PCA9502_OPCODE_SET_PIN_DIRECTION, .continued = ENABLE,
        {.pinFunc = {.pin = PCA9502_PIN5, .func = PCA9502_PINFUNC_INPUT, .enableInterrupt = ENABLE}}},
    {.opcode = PCA9502_OPCODE_SET_PIN_DIRECTION, .continued = ENABLE,
        {.pinFunc = {.pin = PCA9502_PIN6, .func = PCA9502_PINFUNC_INPUT, .enableInterrupt = ENABLE}}},
    {.opcode = PCA9502_OPCODE_SET_PIN_DIRECTION, .continued = DISABLE,
        {.pinFunc = {.pin = PCA9502_PIN7, .func = PCA9502_PINFUNC_INPUT, .enableInterrupt = ENABLE}}},
};
#endif
#if (BOARD_TYPE == BOARD_REVA3) || (BOARD_TYPE == BOARD_REVA4)
PCA9673_Handle portExpander;

const PCA9673_Config portExpanderInit[] = {
    {.opcode = PCA9673_OPCODE_SET_PIN_DIRECTION, .continued = ENABLE,
        {.pinFunc = {.pin = PCA9673_PIN00, .func = PCA9673_PINFUNC_OUTPUT}}},
    {.opcode = PCA9673_OPCODE_SET_PIN_DIRECTION, .continued = ENABLE,
        {.pinFunc = {.pin = PCA9673_PIN01, .func = PCA9673_PINFUNC_OUTPUT}}},
    {.opcode = PCA9673_OPCODE_SET_PIN_DIRECTION, .continued = ENABLE,
        {.pinFunc = {.pin = PCA9673_PIN02, .func = PCA9673_PINFUNC_OUTPUT}}},
    {.opcode = PCA9673_OPCODE_SET_PIN_DIRECTION, .continued = ENABLE,
        {.pinFunc = {.pin = PCA9673_PIN03, .func = PCA9673_PINFUNC_OUTPUT}}},
    {.opcode = PCA9673_OPCODE_SET_PIN_DIRECTION, .continued = ENABLE,
        {.pinFunc = {.pin = PCA9673_PIN10, .func = PCA9673_PINFUNC_INPUT}}},
    {.opcode = PCA9673_OPCODE_SET_PIN_DIRECTION, .continued = ENABLE,
        {.pinFunc = {.pin = PCA9673_PIN11, .func = PCA9673_PINFUNC_INPUT}}},
    {.opcode = PCA9673_OPCODE_SET_PIN_DIRECTION, .continued = ENABLE,
        {.pinFunc = {.pin = PCA9673_PIN12, .func = PCA9673_PINFUNC_INPUT}}},
    {.opcode = PCA9673_OPCODE_SET_PIN_DIRECTION, .continued = ENABLE,
        {.pinFunc = {.pin = PCA9673_PIN13, .func = PCA9673_PINFUNC_INPUT}}},
    {.opcode = PCA9673_OPCODE_SET_PIN_DIRECTION, .continued = ENABLE,
        {.pinFunc = {.pin = PCA9673_PIN14, .func = PCA9673_PINFUNC_INPUT}}},
    {.opcode = PCA9673_OPCODE_SET_PIN_DIRECTION, .continued = ENABLE,
        {.pinFunc = {.pin = PCA9673_PIN15, .func = PCA9673_PINFUNC_INPUT}}},
    {.opcode = PCA9673_OPCODE_SET_PIN_DIRECTION, .continued = ENABLE,
        {.pinFunc = {.pin = PCA9673_PIN16, .func = PCA9673_PINFUNC_INPUT}}},
    {.opcode = PCA9673_OPCODE_SET_PIN_DIRECTION, .continued = DISABLE,
        {.pinFunc = {.pin = PCA9673_PIN17, .func = PCA9673_PINFUNC_INPUT}}},
};
#endif
#endif



#if LPCLIB_SSP
SSP_Handle ssp0_bus;
ST7565R_Handle lcd;
GENERICSPI_Handle tftController;
TSC2046_Handle touchController;

static const SSP_Config ssp0Config[] = {
    {.opcode = SSP_OPCODE_SET_MODE, .continued = ENABLE,
        {.mode = SSP_MODE_MASTER, }},

    {.opcode = SSP_OPCODE_SET_FORMAT, .continued = ENABLE,
        {.format = {
            .bits = SSP_BITS_8,
            .clockFormat = SSP_CLOCK_CPHA0_CPOL0,
            .frameFormat = SSP_FRAMEFORMAT_SPI, }}},

    {.opcode = SSP_OPCODE_SET_BITRATE, .continued = DISABLE,
        {.bitrate = 200000ul }},
};


/** Arbitrary identifiers to be used in SSP slave select mechanism. */
enum {
    BSP_SSEL_LCD,
    BSP_SSEL_TFT,
    BSP_SSEL_TOUCH,
};


/** Event handler for SSP0 slave select demuxing. */
static void BSP_handleSsp0Select (LPCLIB_Event event)
{
    switch (event.opcode) {
    case SSP_EVENT_ASSERT_CHIPSELECT:
        switch (event.channel) {
#if BOARD_TYPE == BOARD_REVA2
        case BSP_SSEL_LCD:
            GPIO_writeBit(SPI_CS1, 0);
            /* Here the parameter is the value of the command/data select line */
            GPIO_writeBit(LCD_A0, (int)event.parameter);
            break;
#endif
#if (BOARD_TYPE == BOARD_REVA3) || (BOARD_TYPE == BOARD_REVA4)
        case BSP_SSEL_LCD:
            GPIO_writeBit(SSEL_MUX_A, 0);
            GPIO_writeBit(SSEL_MUX_B, 0);
            /* Here the parameter is the value of the command/data select line */
            GPIO_writeBit(LCD_A0, (int)event.parameter);
            break;
        case BSP_SSEL_TFT:
            GPIO_writeBit(SSEL_MUX_A, 1);
            GPIO_writeBit(SSEL_MUX_B, 0);
            break;
        case BSP_SSEL_TOUCH:
            GPIO_writeBit(SSEL_MUX_A, 0);
            GPIO_writeBit(SSEL_MUX_B, 1);
            break;
#endif
        }
        break;

    case SSP_EVENT_DEASSERT_CHIPSELECT:
#if BOARD_TYPE == BOARD_REVA2
        GPIO_writeBit(SPI_CS1, 1);
#endif
        break;

    default:
        break;
    }
}


static const SSP_DeviceSelect lcdSelect = {
    .callback = BSP_handleSsp0Select,
    .channel = BSP_SSEL_LCD,
};

static const SSP_DeviceSelect tftSelect = {
    .callback = BSP_handleSsp0Select,
    .channel = BSP_SSEL_TFT,
};

static const SSP_DeviceSelect touchSelect = {
    .callback = BSP_handleSsp0Select,
    .channel = BSP_SSEL_TOUCH,
};


const ST7565R_Config lcdInit[] = {
    {.opcode = ST7565R_OPCODE_INIT, .continued = DISABLE, },
};
#endif


#if LPCLIB_DMA
DMA_Handle gpdma;

/** DMA driver memory allocation handler. */
static void BSP_dmaMemoryAllocation (LPCLIB_Event event)
{
    LPCLIB_OSAL_SIZE size;

    switch (event.opcode) {
    case DMA_EVENT_MEMALLOC:
        size = *((uint32_t *)event.parameter);
        LPCLIB_OSALMemAlloc(size, event.parameter);
        break;

    case DMA_EVENT_MEMFREE:
        LPCLIB_OSALMemRelease(event.parameter);
        break;

    default:
        event.parameter = NULL;
        break;
    }
}

static const DMA_Config dmaInstallHandler = {
    .opcode = DMA_OPCODE_SET_SYSTEM_CALLBACK,
    {.callback = BSP_dmaMemoryAllocation, },
};
#endif


void BSP_init (void)
{
    GPIO_open();

#if BOARD_TYPE == BOARD_REVA2
    GPIO_setDirBit(SPI_CS1, ENABLE);
    GPIO_writeBit(SPI_CS1, 1);                          /* Deactivate LCD chip select */
    GPIO_setDirBit(LCD_A0, ENABLE);
    GPIO_setDirBit(LCD_RST, ENABLE);
    GPIO_writeBit(LCD_RST, 0);                          /* Activate LCD reset */
#endif
#if BOARD_TYPE == BOARD_REVA3
    GPIO_setDirBit(LCD_A0, ENABLE);
    GPIO_setDirBit(LCD_RST, ENABLE);
    GPIO_writeBit(LCD_RST, 0);                          /* Activate LCD reset */
    GPIO_setDirBit(SSEL_MUX_A, ENABLE);
    GPIO_setDirBit(SSEL_MUX_B, ENABLE);
    GPIO_setDirBit(TFT_CMD_DATA, ENABLE);
    GPIO_setDirBit(TFT_RESET, ENABLE);
    GPIO_writeBit(TFT_RESET, 0);                        /* Keep TFT in reset */
    GPIO_setDirBit(TOUCH_IRQ, DISABLE);
#endif
#if BOARD_TYPE == BOARD_REVA4
    GPIO_setDirBit(LCD_A0, ENABLE);
    GPIO_setDirBit(LCD_RST, ENABLE);
    GPIO_writeBit(LCD_RST, 0);                          /* Activate LCD reset */
    GPIO_setDirBit(SSEL_MUX_A, ENABLE);
    GPIO_setDirBit(SSEL_MUX_B, ENABLE);
    GPIO_setDirBit(TFT_CMD_DATA, ENABLE);
    GPIO_setDirBit(TFT_RESET, ENABLE);
    GPIO_writeBit(TFT_RESET, 0);                        /* Keep TFT in reset */
    GPIO_setDirBit(TOUCH_IRQ, DISABLE);

    GPIO_setDirBit(SDIO_LED, ENABLE);
    GPIO_writeBit(SDIO_LED, 0);                         /* SDIO LED off */

    GPIO_setDirBit(RGB_LED_RED, ENABLE);                /* RGB LED red color */
    GPIO_setDirBit(RGB_LED_GREEN, ENABLE);              /* RGB LED green color */
    GPIO_setDirBit(RGB_LED_BLUE, ENABLE);               /* RGB LED blue color */
    GPIO_writeBit(RGB_LED_RED, 1);                      /* RGB LED Red off */
    GPIO_writeBit(RGB_LED_GREEN, 1);                    /* RGB LED Green off */
    GPIO_writeBit(RGB_LED_BLUE, 1);                     /* RGB LED Blue off */

    GPIO_setDirBit(LED_D12, ENABLE);                    
    GPIO_writeBit(LED_D12, 1);                          /* Red LED D12 off     */
    GPIO_setDirBit(LED_D13, ENABLE);
    GPIO_writeBit(LED_D13, 1);                          /* Orange LED D13 off  */
    GPIO_setDirBit(LED_D14, ENABLE);
    GPIO_writeBit(LED_D14, 1);                          /* Orange LED D14 off  */
    GPIO_setDirBit(LED_D15, ENABLE);
    GPIO_writeBit(LED_D15, 1);                          /* Green LED D15 off   */
#endif

#if LPCLIB_I2C
    I2C_open(I2C0, &i2c_bus);
    I2C_ioctl(i2c_bus, &i2c_bitrate400k_config);
  #if BOARD_TYPE == BOARD_REVA2
    PCA9502_open(i2c_bus, PCA9502_ADDRESS, &portExpander);
    PCA9502_ioctl(portExpander, &portExpanderInit[0]);
    PCA9502_write8WithMask(portExpander, 0x0F, 0x0F);   /* All LED's off */
  #endif
  #if (BOARD_TYPE == BOARD_REVA3) || (BOARD_TYPE == BOARD_REVA4)
    PCA9673_open(i2c_bus, PCA9673_ADDRESS, &portExpander);
    PCA9673_ioctl(portExpander, &portExpanderInit[0]);
    PCA9673_write16WithMask(portExpander, 0x000F, 0x000F);   /* All LED's off */
  #endif
#endif

#if LPCLIB_SSP
    SSP_open(SSP0, &ssp0_bus);
    SSP_ioctl(ssp0_bus, &ssp0Config[0]);
    GPIO_writeBit(LCD_RST, 1);                          /* Deactivate LCD reset */
    ST7565R_open(ssp0_bus, &lcdSelect, &lcd);
    ST7565R_ioctl(lcd, &lcdInit[0]);
    GENERICSPI_open(ssp0_bus, &tftSelect, &tftController);
    TSC2046_open(ssp0_bus, &touchSelect, &touchController);
#endif

#if LPCLIB_DMA
    DMA_open(DMA0, &gpdma);
    DMA_ioctl(gpdma, &dmaInstallHandler);
#endif
}



/* Select pin multiplexers (and the electrical characteristics). */
void BSP_initPins (void)
{
    IOCON_open();

#define DATABUS_PIN(func) \
    IOCON_makeConfigD(PIN_FUNCTION_##func, PIN_PULL_NONE, PIN_SLEWRATE_FAST, PIN_INPUTBUFFER_ON, \
                                           PIN_INPUT_GLITCHFILTER_OFF)
#define ADDRESSBUS_PIN(func) \
    IOCON_makeConfigD(PIN_FUNCTION_##func, PIN_PULL_NONE, PIN_SLEWRATE_FAST, PIN_INPUTBUFFER_ON, \
                                           PIN_INPUT_GLITCHFILTER_OFF)
#define EMC_CONTROL_PIN(func) \
    IOCON_makeConfigD(PIN_FUNCTION_##func, PIN_PULL_NONE, PIN_SLEWRATE_FAST, PIN_INPUTBUFFER_ON, \
                                           PIN_INPUT_GLITCHFILTER_OFF)
#define CLOCK_PIN(func) \
    IOCON_makeConfigC(PIN_FUNCTION_##func, PIN_PULL_NONE, PIN_SLEWRATE_FAST, PIN_INPUTBUFFER_ON, \
                                           PIN_INPUT_GLITCHFILTER_OFF)
#define NORMAL_PIN(func) \
    IOCON_makeConfigD(PIN_FUNCTION_##func, PIN_PULL_NONE, PIN_SLEWRATE_SLOW, PIN_INPUTBUFFER_ON, \
                                           PIN_INPUT_GLITCHFILTER_OFF)
#define HIGHDRIVE_PIN(func) \
    IOCON_makeConfigH(PIN_FUNCTION_##func, PIN_PULL_NONE, PIN_SLEWRATE_SLOW, PIN_INPUTBUFFER_ON, \
                                           PIN_INPUT_GLITCHFILTER_OFF, PIN_DRIVESTRENGTH_4MA)
#define ENET_PIN(func) \
    IOCON_makeConfigD(PIN_FUNCTION_##func, PIN_PULL_NONE, PIN_SLEWRATE_SLOW, PIN_INPUTBUFFER_ON, \
                                           PIN_INPUT_GLITCHFILTER_OFF)
#define ENETH_PIN(func) \
    IOCON_makeConfigH(PIN_FUNCTION_##func, PIN_PULL_NONE, PIN_SLEWRATE_SLOW, PIN_INPUTBUFFER_ON, \
                                           PIN_INPUT_GLITCHFILTER_OFF, PIN_DRIVESTRENGTH_4MA)

#if BOARD_TYPE == BOARD_REVA2
    IOCON_configurePin(PIN_P1_7, DATABUS_PIN(3));               /* D0 */
    IOCON_configurePin(PIN_P1_8, DATABUS_PIN(3));               /* D1 */
    IOCON_configurePin(PIN_P1_9, DATABUS_PIN(3));               /* D2 */
    IOCON_configurePin(PIN_P1_10, DATABUS_PIN(3));              /* D3 */
    IOCON_configurePin(PIN_P1_11, DATABUS_PIN(3));              /* D4 */
    IOCON_configurePin(PIN_P1_12, DATABUS_PIN(3));              /* D5 */
    IOCON_configurePin(PIN_P1_13, DATABUS_PIN(3));              /* D6 */
    IOCON_configurePin(PIN_P1_14, DATABUS_PIN(3));              /* D7 */
    IOCON_configurePin(PIN_P5_4, DATABUS_PIN(2));               /* D8 */
    IOCON_configurePin(PIN_P5_5, DATABUS_PIN(2));               /* D9 */
    IOCON_configurePin(PIN_P5_6, DATABUS_PIN(2));               /* D10 */
    IOCON_configurePin(PIN_P5_7, DATABUS_PIN(2));               /* D11 */
    IOCON_configurePin(PIN_P5_0, DATABUS_PIN(2));               /* D12 */
    IOCON_configurePin(PIN_P5_1, DATABUS_PIN(2));               /* D13 */
    IOCON_configurePin(PIN_P5_2, DATABUS_PIN(2));               /* D14 */
    IOCON_configurePin(PIN_P5_3, DATABUS_PIN(2));               /* D15 */
    IOCON_configurePin(PIN_PD_2, DATABUS_PIN(2));               /* D16 */
    IOCON_configurePin(PIN_PD_3, DATABUS_PIN(2));               /* D17 */
    IOCON_configurePin(PIN_PD_4, DATABUS_PIN(2));               /* D18 */
    IOCON_configurePin(PIN_PD_5, DATABUS_PIN(2));               /* D19 */
    IOCON_configurePin(PIN_PD_6, DATABUS_PIN(2));               /* D20 */
    IOCON_configurePin(PIN_PD_7, DATABUS_PIN(2));               /* D21 */
    IOCON_configurePin(PIN_PD_8, DATABUS_PIN(2));               /* D22 */
    IOCON_configurePin(PIN_PD_9, DATABUS_PIN(2));               /* D23 */
    IOCON_configurePin(PIN_PE_5, DATABUS_PIN(3));               /* D24 */
    IOCON_configurePin(PIN_PE_6, DATABUS_PIN(3));               /* D25 */
    IOCON_configurePin(PIN_PE_7, DATABUS_PIN(3));               /* D26 */
    IOCON_configurePin(PIN_PE_8, DATABUS_PIN(3));               /* D27 */
    IOCON_configurePin(PIN_PE_9, DATABUS_PIN(3));               /* D28 */
    IOCON_configurePin(PIN_PE_10, DATABUS_PIN(3));              /* D29 */
    IOCON_configurePin(PIN_PE_11, DATABUS_PIN(3));              /* D30 */
    IOCON_configurePin(PIN_PE_12, DATABUS_PIN(3));              /* D31 */

    IOCON_configurePin(PIN_P2_9, ADDRESSBUS_PIN(3));            /* A0 */
    IOCON_configurePin(PIN_P2_10, ADDRESSBUS_PIN(3));           /* A1 */
    IOCON_configurePin(PIN_P2_11, ADDRESSBUS_PIN(3));           /* A2 */
    IOCON_configurePin(PIN_P2_12, ADDRESSBUS_PIN(3));           /* A3 */
    IOCON_configurePin(PIN_P2_13, ADDRESSBUS_PIN(3));           /* A4 */
    IOCON_configurePin(PIN_P1_0, ADDRESSBUS_PIN(2));            /* A5 */
    IOCON_configurePin(PIN_P1_1, ADDRESSBUS_PIN(2));            /* A6 */
    IOCON_configurePin(PIN_P1_2, ADDRESSBUS_PIN(2));            /* A7 */
    IOCON_configurePin(PIN_P2_8, ADDRESSBUS_PIN(3));            /* A8 */
    IOCON_configurePin(PIN_P2_7, ADDRESSBUS_PIN(3));            /* A9 */
    IOCON_configurePin(PIN_P2_6, ADDRESSBUS_PIN(2));            /* A10 */
    IOCON_configurePin(PIN_P2_2, ADDRESSBUS_PIN(2));            /* A11 */
    IOCON_configurePin(PIN_P2_1, ADDRESSBUS_PIN(2));            /* A12 */
    IOCON_configurePin(PIN_P2_0, ADDRESSBUS_PIN(2));            /* A13 */
    IOCON_configurePin(PIN_P6_8, ADDRESSBUS_PIN(1));            /* A14 */
    IOCON_configurePin(PIN_P6_7, ADDRESSBUS_PIN(1));            /* A15 */
    IOCON_configurePin(PIN_PD_16, ADDRESSBUS_PIN(2));           /* A16 */
    IOCON_configurePin(PIN_PD_15, ADDRESSBUS_PIN(2));           /* A17 */
    IOCON_configurePin(PIN_PE_0, ADDRESSBUS_PIN(3));            /* A18 */
    IOCON_configurePin(PIN_PE_1, ADDRESSBUS_PIN(3));            /* A19 */
    IOCON_configurePin(PIN_PE_2, ADDRESSBUS_PIN(3));            /* A20 */
    IOCON_configurePin(PIN_PE_3, ADDRESSBUS_PIN(3));            /* A21 */
    IOCON_configurePin(PIN_PE_4, ADDRESSBUS_PIN(3));            /* A22 */

    IOCON_configurePin(PIN_P1_5, EMC_CONTROL_PIN(3));           /* CS0 */
    IOCON_configurePin(PIN_PD_12, EMC_CONTROL_PIN(2));          /* CS2 */
    IOCON_configurePin(PIN_P1_6, EMC_CONTROL_PIN(3));           /* WE */
    IOCON_configurePin(PIN_P1_3, EMC_CONTROL_PIN(3));           /* OE */
    IOCON_configurePin(PIN_P1_4, EMC_CONTROL_PIN(3));           /* BLS0 */
    IOCON_configurePin(PIN_P6_6, EMC_CONTROL_PIN(1));           /* BLS1 */
    IOCON_configurePin(PIN_CLK0, CLOCK_PIN(0));                 /* EMC_CLK0 */
    IOCON_configurePin(PIN_CLK1, CLOCK_PIN(0));                 /* EMC_CLK1 */
    IOCON_configurePin(PIN_CLK2, CLOCK_PIN(0));                 /* EMC_CLK3 */
    IOCON_configurePin(PIN_CLK3, CLOCK_PIN(0));                 /* EMC_CLK2 */
    IOCON_configurePin(PIN_P6_4, EMC_CONTROL_PIN(3));           /* CAS */
    IOCON_configurePin(PIN_P6_5, EMC_CONTROL_PIN(3));           /* RAS */
    IOCON_configurePin(PIN_P6_9, EMC_CONTROL_PIN(3));           /* DYCS0 */
    IOCON_configurePin(PIN_P6_11, EMC_CONTROL_PIN(3));          /* CKE0 */
    IOCON_configurePin(PIN_P6_12, EMC_CONTROL_PIN(3));          /* DQM0 */
    IOCON_configurePin(PIN_P6_10, EMC_CONTROL_PIN(3));          /* DQM1 */

    IOCON_configurePin(PIN_I2C0, IOCON_makeConfigI(
        PIN_I2C0_GLITCHFILTER_50NS, PIN_I2C0_SPEED_FASTMODE));  /* I2C0_SDA, I2C0_SCL */

    IOCON_configurePin(PIN_P0_0, ENET_PIN(2));                  /* ENET_RXD1 */
    IOCON_configurePin(PIN_P0_1, ENET_PIN(2));                  /* ENET_COL */
    IOCON_configurePin(PIN_P1_15, ENET_PIN(3));                 /* ENET_RXD0 */
    IOCON_configurePin(PIN_P1_16, ENET_PIN(3));                 /* ENET_CRS */
    IOCON_configurePin(PIN_P1_17, ENETH_PIN(3));                /* ENET_MDIO */
    IOCON_configurePin(PIN_P1_18, ENET_PIN(3));                 /* ENET_TXD0 */
    IOCON_configurePin(PIN_P1_19, ENET_PIN(0));                 /* ENET_TX_CLK */
    IOCON_configurePin(PIN_P1_20, ENET_PIN(3));                 /* ENET_TXD1 */
    IOCON_configurePin(PIN_P2_3, HIGHDRIVE_PIN(2));             /* U3_TXD */
    IOCON_configurePin(PIN_P2_4, HIGHDRIVE_PIN(2));             /* U3_RXD */
    IOCON_configurePin(PIN_P2_5, HIGHDRIVE_PIN(2));             /* USB1_VBUS */
    IOCON_configurePin(PIN_P3_0, NORMAL_PIN(2));                /* I2S0_TX_SCK */
    IOCON_configurePin(PIN_P3_1, NORMAL_PIN(0));                /* I2S0_TX_WS */
    IOCON_configurePin(PIN_P3_2, NORMAL_PIN(0));                /* I2S0_TX_SDA */
    IOCON_configurePin(PIN_P3_3, NORMAL_PIN(3));                /* SPIFI_SCK */
    IOCON_configurePin(PIN_P3_4, NORMAL_PIN(3));                /* SPIFI_SIO3 */
    IOCON_configurePin(PIN_P3_5, NORMAL_PIN(3));                /* SPIFI_SIO2 */
    IOCON_configurePin(PIN_P3_6, NORMAL_PIN(3));                /* SPIFI_MISO */
    IOCON_configurePin(PIN_P3_7, NORMAL_PIN(3));                /* SPIFI_MOSI */
    IOCON_configurePin(PIN_P3_8, NORMAL_PIN(3));                /* SPIFI_CS */
//P4_0

    IOCON_configurePin(PIN_P6_0, NORMAL_PIN(4));                /* I2S0_RX_SCK */
    IOCON_configurePin(PIN_P6_1, NORMAL_PIN(3));                /* I2S0_RX_WS */
    IOCON_configurePin(PIN_P6_2, NORMAL_PIN(3));                /* I2S0_RX_SDA */



    IOCON_configurePin(PIN_PC_0, ENET_PIN(3));                  /* ENET_RX_CLK */
    IOCON_configurePin(PIN_PC_1, ENET_PIN(3));                  /* ENET_MDC */
//PC_2
//PC_3
//PC_4

//P4_8
    IOCON_configurePin(PIN_P7_4, NORMAL_PIN(0));                /* GPIO3.12 (SPI_CS1) */
//P8_0
//P8_1
//P8_2
//P8_8
    IOCON_configurePin(PIN_PA_1, IOCON_makeConfigH(             /* GPIO4.8 (BUTTON_IRQ_INPUT) */
        PIN_FUNCTION_0, PIN_PULL_UP, PIN_SLEWRATE_SLOW, PIN_INPUTBUFFER_ON,
        PIN_INPUT_GLITCHFILTER_OFF, PIN_DRIVESTRENGTH_4MA));
    IOCON_configurePin(PIN_PA_2, HIGHDRIVE_PIN(0));             /* GPIO4.9 (LCD_RST) */
    IOCON_configurePin(PIN_PA_3, HIGHDRIVE_PIN(0));             /* GPIO4.10 (LCD_A0) */
    IOCON_configurePin(PIN_PC_10, NORMAL_PIN(7));               /* SD_CMD */
    IOCON_configurePin(PIN_PC_13, NORMAL_PIN(2));               /* U1_TXD */
    IOCON_configurePin(PIN_PC_14, NORMAL_PIN(2));               /* U1_RXD */
    IOCON_configurePin(PIN_PF_0, NORMAL_PIN(0));                /* SSP0_SCK */
    IOCON_configurePin(PIN_PF_1, NORMAL_PIN(2));                /* SSP0_SSEL */
    IOCON_configurePin(PIN_PF_2, NORMAL_PIN(2));                /* SSP0_MISO */
    IOCON_configurePin(PIN_PF_3, NORMAL_PIN(2));                /* SSP0_MOSI */
    IOCON_configurePin(PIN_PF_10, NORMAL_PIN(6));               /* SD_WP */

  #if LPCLIB_LCD
//P4_1
    IOCON_configurePin(PIN_P4_2, NORMAL_PIN(2));                /* LCD_VD3 */
    IOCON_configurePin(PIN_P4_3, NORMAL_PIN(2));                /* LCD_VD2 */
//P4_4
    IOCON_configurePin(PIN_P4_5, NORMAL_PIN(2));                /* LCD_FP */
    IOCON_configurePin(PIN_P4_6, NORMAL_PIN(2));                /* LCD_ENAB */
    IOCON_configurePin(PIN_P4_7, NORMAL_PIN(0));                /* LCD_DCLK */
    IOCON_configurePin(PIN_P4_9, NORMAL_PIN(2));                /* LCD_VD11 */
    IOCON_configurePin(PIN_P4_10, NORMAL_PIN(2));               /* LCD_VD10 */
    IOCON_configurePin(PIN_P7_0, NORMAL_PIN(3));                /* LCD_LE */
    IOCON_configurePin(PIN_P7_1, NORMAL_PIN(3));                /* LCD_VD19 */
    IOCON_configurePin(PIN_P7_2, NORMAL_PIN(3));                /* LCD_VD18 */
    IOCON_configurePin(PIN_P7_6, NORMAL_PIN(3));                /* LCD_LP */
    IOCON_configurePin(PIN_P7_7, NORMAL_PIN(3));                /* LCD_PWR */
    IOCON_configurePin(PIN_P8_3, NORMAL_PIN(3));                /* LCD_VD12 */
    IOCON_configurePin(PIN_P8_4, NORMAL_PIN(3));                /* LCD_VD7 */
    IOCON_configurePin(PIN_P8_5, NORMAL_PIN(3));                /* LCD_VD6 */
    IOCON_configurePin(PIN_P8_6, NORMAL_PIN(3));                /* LCD_VD5 */
    IOCON_configurePin(PIN_P8_7, NORMAL_PIN(3));                /* LCD_VD4 */
    IOCON_configurePin(PIN_PB_0, NORMAL_PIN(2));                /* LCD_VD23 */
    IOCON_configurePin(PIN_PB_1, NORMAL_PIN(2));                /* LCD_VD22 */
    IOCON_configurePin(PIN_PB_2, NORMAL_PIN(2));                /* LCD_VD21 */
    IOCON_configurePin(PIN_PB_3, NORMAL_PIN(2));                /* LCD_VD20 */
    IOCON_configurePin(PIN_PB_4, NORMAL_PIN(2));                /* LCD_VD15 */
    IOCON_configurePin(PIN_PB_5, NORMAL_PIN(2));                /* LCD_VD14 */
    IOCON_configurePin(PIN_PB_6, NORMAL_PIN(2));                /* LCD_VD13 */
  #else
    /* Configure X11 pins as alternate functions (SCT, ...) */
    IOCON_configurePin(PIN_P4_2, NORMAL_PIN(1));                /* CTOUT_0 */
    IOCON_configurePin(PIN_P4_3, NORMAL_PIN(1));                /* CTOUT_3 */
    IOCON_configurePin(PIN_P4_5, NORMAL_PIN(1));                /* CTOUT_5 */
    IOCON_configurePin(PIN_P4_6, NORMAL_PIN(1));                /* CTOUT_4 */
    IOCON_configurePin(PIN_P4_7, NORMAL_PIN(1));                /* GP_CLKIN */
    IOCON_configurePin(PIN_P4_9, NORMAL_PIN(1));                /* CTIN_6 */
    IOCON_configurePin(PIN_P4_10, NORMAL_PIN(1));               /* CTIN_2 */
    IOCON_configurePin(PIN_P7_0, NORMAL_PIN(1));                /* CTOUT_14 */
    IOCON_configurePin(PIN_P7_1, NORMAL_PIN(1));                /* CTOUT_15 */
    IOCON_configurePin(PIN_P7_2, NORMAL_PIN(1));                /* CTIN_4 */
    IOCON_configurePin(PIN_P7_6, NORMAL_PIN(1));                /* CTOUT_11 */
    IOCON_configurePin(PIN_P7_7, NORMAL_PIN(1));                /* CTOUT_8 */
    IOCON_configurePin(PIN_PB_0, NORMAL_PIN(1));                /* CTOUT_10 */
    IOCON_configurePin(PIN_PB_1, NORMAL_PIN(5));                /* CTOUT_6 */
    IOCON_configurePin(PIN_PB_2, NORMAL_PIN(5));                /* CTOUT_7 */
    IOCON_configurePin(PIN_PB_4, NORMAL_PIN(5));                /* CTIN_5 */
    IOCON_configurePin(PIN_PB_5, NORMAL_PIN(5));                /* CTIN_7 */
  #endif
#endif /* #if BOARD_TYPE == BOARD_REVA2 */

#if (BOARD_TYPE == BOARD_REVA3) || (BOARD_TYPE == BOARD_REVA4)
    IOCON_configurePin(PIN_P1_7, DATABUS_PIN(3));               /* D0 */
    IOCON_configurePin(PIN_P1_8, DATABUS_PIN(3));               /* D1 */
    IOCON_configurePin(PIN_P1_9, DATABUS_PIN(3));               /* D2 */
    IOCON_configurePin(PIN_P1_10, DATABUS_PIN(3));              /* D3 */
    IOCON_configurePin(PIN_P1_11, DATABUS_PIN(3));              /* D4 */
    IOCON_configurePin(PIN_P1_12, DATABUS_PIN(3));              /* D5 */
    IOCON_configurePin(PIN_P1_13, DATABUS_PIN(3));              /* D6 */
    IOCON_configurePin(PIN_P1_14, DATABUS_PIN(3));              /* D7 */
    IOCON_configurePin(PIN_P5_4, DATABUS_PIN(2));               /* D8 */
    IOCON_configurePin(PIN_P5_5, DATABUS_PIN(2));               /* D9 */
    IOCON_configurePin(PIN_P5_6, DATABUS_PIN(2));               /* D10 */
    IOCON_configurePin(PIN_P5_7, DATABUS_PIN(2));               /* D11 */
    IOCON_configurePin(PIN_P5_0, DATABUS_PIN(2));               /* D12 */
    IOCON_configurePin(PIN_P5_1, DATABUS_PIN(2));               /* D13 */
    IOCON_configurePin(PIN_P5_2, DATABUS_PIN(2));               /* D14 */
    IOCON_configurePin(PIN_P5_3, DATABUS_PIN(2));               /* D15 */

    IOCON_configurePin(PIN_P2_9, ADDRESSBUS_PIN(3));            /* A0 */
    IOCON_configurePin(PIN_P2_10, ADDRESSBUS_PIN(3));           /* A1 */
    IOCON_configurePin(PIN_P2_11, ADDRESSBUS_PIN(3));           /* A2 */
    IOCON_configurePin(PIN_P2_12, ADDRESSBUS_PIN(3));           /* A3 */
    IOCON_configurePin(PIN_P2_13, ADDRESSBUS_PIN(3));           /* A4 */
    IOCON_configurePin(PIN_P1_0, ADDRESSBUS_PIN(2));            /* A5 */
    IOCON_configurePin(PIN_P1_1, ADDRESSBUS_PIN(2));            /* A6 */
    IOCON_configurePin(PIN_P1_2, ADDRESSBUS_PIN(2));            /* A7 */
    IOCON_configurePin(PIN_P2_8, ADDRESSBUS_PIN(3));            /* A8 */
    IOCON_configurePin(PIN_P2_7, ADDRESSBUS_PIN(3));            /* A9 */
    IOCON_configurePin(PIN_P2_6, ADDRESSBUS_PIN(2));            /* A10 */
    IOCON_configurePin(PIN_P2_2, ADDRESSBUS_PIN(2));            /* A11 */
    IOCON_configurePin(PIN_P2_1, ADDRESSBUS_PIN(2));            /* A12 */
    IOCON_configurePin(PIN_P2_0, ADDRESSBUS_PIN(2));            /* A13 */
    IOCON_configurePin(PIN_P6_8, ADDRESSBUS_PIN(1));            /* A14 */
    IOCON_configurePin(PIN_P6_7, ADDRESSBUS_PIN(1));            /* A15 */
    IOCON_configurePin(PIN_PD_16, ADDRESSBUS_PIN(2));           /* A16 */
    IOCON_configurePin(PIN_PD_15, ADDRESSBUS_PIN(2));           /* A17 */
    IOCON_configurePin(PIN_PE_0, ADDRESSBUS_PIN(3));            /* A18 */
    IOCON_configurePin(PIN_PE_1, ADDRESSBUS_PIN(3));            /* A19 */
    IOCON_configurePin(PIN_PE_2, ADDRESSBUS_PIN(3));            /* A20 */
    IOCON_configurePin(PIN_PE_3, ADDRESSBUS_PIN(3));            /* A21 */
    IOCON_configurePin(PIN_PE_4, ADDRESSBUS_PIN(3));            /* A22 */

	IOCON_configurePin(PIN_P1_3, EMC_CONTROL_PIN(3));           /* OE */
    IOCON_configurePin(PIN_P1_4, EMC_CONTROL_PIN(3));           /* BLS0 */
    IOCON_configurePin(PIN_P1_5, EMC_CONTROL_PIN(3));           /* CS0 */
    IOCON_configurePin(PIN_P1_6, EMC_CONTROL_PIN(3));           /* WE */    
    IOCON_configurePin(PIN_P6_4, EMC_CONTROL_PIN(3));           /* CAS */
    IOCON_configurePin(PIN_P6_5, EMC_CONTROL_PIN(3));           /* RAS */
    IOCON_configurePin(PIN_P6_6, EMC_CONTROL_PIN(1));           /* BLS1 */
    IOCON_configurePin(PIN_P6_9, EMC_CONTROL_PIN(3));           /* DYCS0 */
    IOCON_configurePin(PIN_P6_10, EMC_CONTROL_PIN(3));          /* DQM1 */
    IOCON_configurePin(PIN_P6_11, EMC_CONTROL_PIN(3));          /* CKE0 */
    IOCON_configurePin(PIN_P6_12, EMC_CONTROL_PIN(3));          /* DQM0 */
    IOCON_configurePin(PIN_PD_12, EMC_CONTROL_PIN(2));          /* CS2 */
    IOCON_configurePin(PIN_CLK0, CLOCK_PIN(0));                 /* EMC_CLK0 */
    IOCON_configurePin(PIN_CLK1, CLOCK_PIN(0));                 /* EMC_CLK1 */
    IOCON_configurePin(PIN_CLK2, CLOCK_PIN(0));                 /* EMC_CLK3 */
    IOCON_configurePin(PIN_CLK3, CLOCK_PIN(0));                 /* EMC_CLK2 */
    

	/* Ethernet RMII */
    if (1)
	{
        IOCON_configurePin(PIN_P0_0,  ENET_PIN(2));                 /* ENET_RXD1 */
        IOCON_configurePin(PIN_P0_1,  ENET_PIN(6));                 /* ENET_TX_EN */
        IOCON_configurePin(PIN_P1_15, ENET_PIN(3));                 /* ENET_RXD0 */
        IOCON_configurePin(PIN_P1_16, ENET_PIN(7));                 /* ENET_RXDV */
        IOCON_configurePin(PIN_P1_17, ENETH_PIN(3));                /* ENET_MDIO */
        IOCON_configurePin(PIN_P1_18, ENET_PIN(3));                 /* ENET_TXD0 */
	    IOCON_configurePin(PIN_P1_19, ENET_PIN(0));                 /* ENET_REF_CLK */
        IOCON_configurePin(PIN_P1_20, ENET_PIN(3));                 /* ENET_TXD1 */
        IOCON_configurePin(PIN_PC_1,  ENET_PIN(3));                 /* ENET_MDC */

        /* LEDs D12 ... D15 , can be used if Ethernet MII is not used */
        IOCON_configurePin(PIN_P9_0,  NORMAL_PIN(0));               /* GPIO4.12 (ENET_CRS) */
        IOCON_configurePin(PIN_P9_1,  NORMAL_PIN(0));               /* GPIO4.13 (ENET_RX_ERR) */
        IOCON_configurePin(PIN_P9_2,  NORMAL_PIN(0));               /* GPIO4.14 (ENET_RXD3) */
        IOCON_configurePin(PIN_P9_3,  NORMAL_PIN(0));               /* GPIO4.15 (ENET_RXD2) */
    }
	else
	{
    /* Ethernet MII */
	    IOCON_configurePin(PIN_P0_0,  ENET_PIN(2));                 /* ENET_RXD1 */
	    IOCON_configurePin(PIN_P0_1,  ENET_PIN(6));                 /* ENET_TX_EN */
        IOCON_configurePin(PIN_P1_15, ENET_PIN(3));                 /* ENET_RXD0 */
        IOCON_configurePin(PIN_P1_16, ENET_PIN(7));                 /* ENET_RXDV */
        IOCON_configurePin(PIN_P1_17, ENETH_PIN(3));                /* ENET_MDIO */
        IOCON_configurePin(PIN_P1_18, ENET_PIN(3));                 /* ENET_TXD0 */
        IOCON_configurePin(PIN_P1_19, ENET_PIN(0));                 /* ENET_TX_CLK */
        IOCON_configurePin(PIN_P1_20, ENET_PIN(3));                 /* ENET_TXD1 */
        IOCON_configurePin(PIN_P9_0,  ENET_PIN(5));                 /* ENET_CRS */
	    IOCON_configurePin(PIN_P9_1,  ENET_PIN(5));                 /* ENET_RX_ER */
        IOCON_configurePin(PIN_P9_2,  ENET_PIN(5));                 /* ENET_RXD3 */
	    IOCON_configurePin(PIN_P9_3,  ENET_PIN(5));                 /* ENET_RXD2 */
        IOCON_configurePin(PIN_P9_4,  ENET_PIN(5));                 /* ENET_TXD2 */
        IOCON_configurePin(PIN_P9_5,  ENET_PIN(5));                 /* ENET_TXD3 */
	    //IOCON_configurePin(PIN_P9_6,  ENET_PIN(5));                 /* ENET_COL , shared with MCOB1, to use it put a jumper on SV6 pins 9-10*/
        IOCON_configurePin(PIN_PC_0,  ENET_PIN(3));                 /* ENET_RX_CLK */
        IOCON_configurePin(PIN_PC_1,  ENET_PIN(3));                 /* ENET_MDC */
	}


    IOCON_configurePin(PIN_P2_3, HIGHDRIVE_PIN(7));             /* USB0_PWR_EN */
	IOCON_configurePin(PIN_P2_4, HIGHDRIVE_PIN(7));             /* USB0_PWR_FAULT */
    IOCON_configurePin(PIN_P2_5, HIGHDRIVE_PIN(4));             /* GPIO5.5 (LCD_A0) */
    IOCON_configurePin(PIN_P3_0, NORMAL_PIN(2));                /* I2S0_TX_SCK */
	IOCON_configurePin(PIN_P3_1, NORMAL_PIN(0));                /* I2S0_TX_WS */
    IOCON_configurePin(PIN_P3_2, NORMAL_PIN(0));                /* I2S0_TX_SDA */

	IOCON_configurePin(PIN_P3_3, NORMAL_PIN(3));                /* SPIFI_SCK */
    IOCON_configurePin(PIN_P3_4, NORMAL_PIN(3));                /* SPIFI_SIO3 */
    IOCON_configurePin(PIN_P3_5, NORMAL_PIN(3));                /* SPIFI_SIO2 */
    IOCON_configurePin(PIN_P3_6, NORMAL_PIN(3));                /* SPIFI_MISO */
    IOCON_configurePin(PIN_P3_7, NORMAL_PIN(3));                /* SPIFI_MOSI */
    IOCON_configurePin(PIN_P3_8, NORMAL_PIN(3));                /* SPIFI_CS */

	IOCON_configurePin(PIN_P4_0, NORMAL_PIN(0));                /* GPIO2.0 - also NMI on func 2 */

#if LPCLIB_LCD
    #if TFT_TYPE == LCD_32
	/* 3.2" LCD module from Embedded Artists */
	IOCON_configurePin(PIN_P4_1, NORMAL_PIN(2));                /* LCD_VD0 */
	IOCON_configurePin(PIN_P4_2, NORMAL_PIN(2));                /* LCD_VD3 */
    IOCON_configurePin(PIN_P4_3, NORMAL_PIN(2));                /* LCD_VD2 */
	IOCON_configurePin(PIN_P4_4, NORMAL_PIN(2));                /* LCD_VD1 */
	IOCON_configurePin(PIN_P4_5, NORMAL_PIN(2));                /* LCD_FP */
    IOCON_configurePin(PIN_P4_6, NORMAL_PIN(2));                /* LCD_ENAB */
	IOCON_configurePin(PIN_P4_7, NORMAL_PIN(0));                /* LCD_DCLK */
	IOCON_configurePin(PIN_P4_8, NORMAL_PIN(2));                /* LCD_VD9 */
	IOCON_configurePin(PIN_P4_9, NORMAL_PIN(2));                /* LCD_VD11 */
    IOCON_configurePin(PIN_P4_10, NORMAL_PIN(2));               /* LCD_VD10 */
	IOCON_configurePin(PIN_P7_0, NORMAL_PIN(3));                /* LCD_LE */
    IOCON_configurePin(PIN_P7_1, NORMAL_PIN(3));                /* LCD_VD19 */
    IOCON_configurePin(PIN_P7_2, NORMAL_PIN(3));                /* LCD_VD18 */
	IOCON_configurePin(PIN_P7_3, NORMAL_PIN(0));                /* GPIO3.11 (Func.0, #TFT_RESET), LCD_VD17 (Func.3) */
	IOCON_configurePin(PIN_P7_4, NORMAL_PIN(3));                /* LCD_VD16 */
    IOCON_configurePin(PIN_P7_5, NORMAL_PIN(3));                /* LCD_VD8 */
	IOCON_configurePin(PIN_P7_6, NORMAL_PIN(3));                /* LCD_LP */
    IOCON_configurePin(PIN_P7_7, NORMAL_PIN(3));                /* LCD_PWR */
	IOCON_configurePin(PIN_P8_3, NORMAL_PIN(3));                /* LCD_VD12 */
    IOCON_configurePin(PIN_P8_4, NORMAL_PIN(3));                /* LCD_VD7 */
    IOCON_configurePin(PIN_P8_5, NORMAL_PIN(3));                /* LCD_VD6 */
    IOCON_configurePin(PIN_P8_6, NORMAL_PIN(3));                /* LCD_VD5 */
    IOCON_configurePin(PIN_P8_7, NORMAL_PIN(3));                /* LCD_VD4 */
	IOCON_configurePin(PIN_PB_0, NORMAL_PIN(2));                /* LCD_VD23 */
    IOCON_configurePin(PIN_PB_1, NORMAL_PIN(2));                /* LCD_VD22 */
    IOCON_configurePin(PIN_PB_2, NORMAL_PIN(2));                /* LCD_VD21 */
    IOCON_configurePin(PIN_PB_3, NORMAL_PIN(2));                /* LCD_VD20 */
    IOCON_configurePin(PIN_PB_4, NORMAL_PIN(2));                /* LCD_VD15 */
    IOCON_configurePin(PIN_PB_5, NORMAL_PIN(2));                /* LCD_VD14 */
    IOCON_configurePin(PIN_PB_6, NORMAL_PIN(2));                /* LCD_VD13 */
    IOCON_configurePin(PIN_PE_7, NORMAL_PIN(4));                /* GPIO7.7 (TOUCH_IRQ) */
	IOCON_configurePin(PIN_PE_8, NORMAL_PIN(4));                /* GPIO7.8 (TFT_CMD_DATA) */
    #endif
    #if TFT_TYPE == TFT_7
    /* 7" LCD module from Embedded Artists */
    IOCON_configurePin(PIN_P4_3, NORMAL_PIN(2));                /* LCD_VD2 */
	IOCON_configurePin(PIN_P4_2, NORMAL_PIN(2));                /* LCD_VD3 */
    IOCON_configurePin(PIN_P8_7, NORMAL_PIN(3));                /* LCD_VD4 */
    IOCON_configurePin(PIN_P8_6, NORMAL_PIN(3));                /* LCD_VD5 */
    IOCON_configurePin(PIN_P8_5, NORMAL_PIN(3));                /* LCD_VD6 */
    IOCON_configurePin(PIN_P8_4, NORMAL_PIN(3));                /* LCD_VD7 */

    IOCON_configurePin(PIN_P4_10, NORMAL_PIN(2));               /* LCD_VD10 */
	IOCON_configurePin(PIN_P4_9, NORMAL_PIN(2));                /* LCD_VD11 */
	IOCON_configurePin(PIN_P8_3, NORMAL_PIN(3));                /* LCD_VD12 */
    IOCON_configurePin(PIN_PB_6, NORMAL_PIN(2));                /* LCD_VD13 */
    IOCON_configurePin(PIN_PB_5, NORMAL_PIN(2));                /* LCD_VD14 */
    IOCON_configurePin(PIN_PB_4, NORMAL_PIN(2));                /* LCD_VD15 */

	IOCON_configurePin(PIN_P7_3, NORMAL_PIN(0));                /* RESET for EA modules (GPIO3.11) */

    IOCON_configurePin(PIN_P7_2, NORMAL_PIN(3));                /* LCD_VD18 */
    IOCON_configurePin(PIN_P7_1, NORMAL_PIN(3));                /* LCD_VD19 */
    IOCON_configurePin(PIN_PB_3, NORMAL_PIN(2));                /* LCD_VD20 */
    IOCON_configurePin(PIN_PB_2, NORMAL_PIN(2));                /* LCD_VD21 */
    IOCON_configurePin(PIN_PB_1, NORMAL_PIN(2));                /* LCD_VD22 */
	IOCON_configurePin(PIN_PB_0, NORMAL_PIN(2));                /* LCD_VD23 */

	IOCON_configurePin(PIN_P4_5, NORMAL_PIN(2));                /* LCD_FP */
    IOCON_configurePin(PIN_P4_6, NORMAL_PIN(2));                /* LCD_ENAB */
	IOCON_configurePin(PIN_P7_6, NORMAL_PIN(3));                /* LCD_LP */
	IOCON_configurePin(PIN_P4_7, NORMAL_PIN(0));                /* LCD_DCLK */
    IOCON_configurePin(PIN_PF_4, NORMAL_PIN(1));                /* LCD_CLKIN --> GP_CLKIN */

    IOCON_configurePin(PIN_PE_7, NORMAL_PIN(4));                /* GPIO7.7 (TOUCH_IRQ) */
	IOCON_configurePin(PIN_PE_8, NORMAL_PIN(4));                /* GPIO7.8 (TFT_CMD_DATA) */

    #endif
#else
	/* alternative configurations if LCD is not used */
#endif



    IOCON_configurePin(PIN_P6_0, NORMAL_PIN(4));                /* I2S0_RX_SCK */
	IOCON_configurePin(PIN_P6_1, NORMAL_PIN(3));                /* I2S0_RX_WS */
	IOCON_configurePin(PIN_P6_2, NORMAL_PIN(3));                /* I2S0_RX_SDA */
	
	IOCON_configurePin(PIN_P6_3, NORMAL_PIN(0));                /* GPIO3.2 (JENNIC_RESET) */

	IOCON_configurePin(PIN_P8_0, HIGHDRIVE_PIN(3));             /* MCIN2 */
    IOCON_configurePin(PIN_P8_1, HIGHDRIVE_PIN(3));             /* MCIN1 */
    IOCON_configurePin(PIN_P8_2, HIGHDRIVE_PIN(3));             /* MCIN0 */

    IOCON_configurePin(PIN_P8_8, NORMAL_PIN(1));                /* USB1_ULPI_CLK */

    IOCON_configurePin(PIN_PA_0, NORMAL_PIN(6));                /* CGU_OUT1 */

	IOCON_configurePin(PIN_PA_1, IOCON_makeConfigH(             /* QEI_IDX */
        PIN_FUNCTION_1, PIN_PULL_UP, PIN_SLEWRATE_SLOW, PIN_INPUTBUFFER_ON,
        PIN_INPUT_GLITCHFILTER_OFF, PIN_DRIVESTRENGTH_4MA));
	IOCON_configurePin(PIN_PA_2, IOCON_makeConfigH(             /* QEI_PHB */
        PIN_FUNCTION_1, PIN_PULL_UP, PIN_SLEWRATE_SLOW, PIN_INPUTBUFFER_ON,
        PIN_INPUT_GLITCHFILTER_OFF, PIN_DRIVESTRENGTH_4MA));
	IOCON_configurePin(PIN_PA_3, IOCON_makeConfigH(             /* QEI_PHA */
        PIN_FUNCTION_1, PIN_PULL_UP, PIN_SLEWRATE_SLOW, PIN_INPUTBUFFER_ON,
        PIN_INPUT_GLITCHFILTER_OFF, PIN_DRIVESTRENGTH_4MA));

	IOCON_configurePin(PIN_PA_4, NORMAL_PIN(4));                /* GPIO5.19 (TOUCHGPIO0) */

    IOCON_configurePin(PIN_PC_2, NORMAL_PIN(4));                /* GPIO6.1 (SDIO_LED) */

    IOCON_configurePin(PIN_PC_3, NORMAL_PIN(4));                /* GPIO6.2 (SDIO_VOLT1) */
    IOCON_configurePin(PIN_PC_4, NORMAL_PIN(7));                /* SD_DAT0 */
    IOCON_configurePin(PIN_PC_5, NORMAL_PIN(7));                /* SD_DAT1 */
    IOCON_configurePin(PIN_PC_6, NORMAL_PIN(7));                /* SD_DAT1 */
    IOCON_configurePin(PIN_PC_7, NORMAL_PIN(7));                /* SD_DAT3 */
    IOCON_configurePin(PIN_PC_8, NORMAL_PIN(7));                /* SD_CD */

    IOCON_configurePin(PIN_PC_9, NORMAL_PIN(4));                /* GPIO6.8 (JENIC_MISO) */

    IOCON_configurePin(PIN_PC_10, NORMAL_PIN(7));               /* SD_CMD */

    IOCON_configurePin(PIN_PC_11, NORMAL_PIN(4));               /* GPIO6.10 (SSEL_MUX_A - SPI chip select signals) */
	IOCON_configurePin(PIN_PC_12, NORMAL_PIN(4));               /* GPIO6.11 (SSEL_MUX_B - SPI chip select signals) */

	IOCON_configurePin(PIN_PC_13, NORMAL_PIN(2));               /* U1_TXD */
    IOCON_configurePin(PIN_PC_14, NORMAL_PIN(2));               /* U1_RXD */

	IOCON_configurePin(PIN_PD_0, NORMAL_PIN(4));                /* GPIO6.14(LCD_RST) */

	IOCON_configurePin(PIN_PD_1, NORMAL_PIN(5));                /* SD_POW */

	IOCON_configurePin(PIN_PD_2, NORMAL_PIN(1));                /* CTOUT7 */
	IOCON_configurePin(PIN_PD_3, NORMAL_PIN(1));                /* CTOUT6 */
	IOCON_configurePin(PIN_PD_4, NORMAL_PIN(1));                /* CTOUT8 */
	IOCON_configurePin(PIN_PD_5, NORMAL_PIN(1));                /* CTOUT9 */
	IOCON_configurePin(PIN_PD_6, NORMAL_PIN(1));                /* CTOUT10 */

	IOCON_configurePin(PIN_PD_7, NORMAL_PIN(4));                /* GPIO6.21 (GPIOIRQ) */

	IOCON_configurePin(PIN_PD_8, NORMAL_PIN(1));                /* CTIN6 */

	IOCON_configurePin(PIN_PD_9, NORMAL_PIN(4));                /* GPIO6.23 (MC_SEL) */

	IOCON_configurePin(PIN_PD_10, NORMAL_PIN(1));               /* CTIN1 */

	IOCON_configurePin(PIN_PD_11, NORMAL_PIN(4));               /* GPIO6.25 (HDMI_STDBY) */

	IOCON_configurePin(PIN_PD_13, NORMAL_PIN(1));               /* CTIN0 */

#if BOARD_TYPE == BOARD_REVA3
	IOCON_configurePin(PIN_PE_5, NORMAL_PIN(1));                /* CTOUT3 */
	IOCON_configurePin(PIN_PE_6, NORMAL_PIN(1));                /* CTOUT2 */
#endif
#if BOARD_TYPE == BOARD_REVA4
    IOCON_configurePin(PIN_PE_5, NORMAL_PIN(4));                /* GPIO7.5 , RGB LED Red (CTOUT3 on func 1) */
	IOCON_configurePin(PIN_PE_6, NORMAL_PIN(4));                /* GPIO7.6 , RGB LED Green (CTOUT2 on func 1) */
#endif
	
	IOCON_configurePin(PIN_PE_9, NORMAL_PIN(4));                /* GPIO7.9 (UDA_RST) */

	IOCON_configurePin(PIN_PE_11, NORMAL_PIN(1));               /* CTOUT12 */
	IOCON_configurePin(PIN_PE_12, NORMAL_PIN(1));               /* CTOUT11 */

	IOCON_configurePin(PIN_PE_13, NORMAL_PIN(2));               /* I2C1_SDA */
	IOCON_configurePin(PIN_PE_15, NORMAL_PIN(2));               /* I2C1_SDL */

	IOCON_configurePin(PIN_PE_14, NORMAL_PIN(4));               /* GPIO7.14 (HDMI_INT) */
	IOCON_configurePin(PIN_PF_0, NORMAL_PIN(0));                /* SSP0_SCK */
	IOCON_configurePin(PIN_PF_1, NORMAL_PIN(2));                /* SSP0_SSEL */
	IOCON_configurePin(PIN_PF_2, NORMAL_PIN(2));                /* SSP0_MISO */
	IOCON_configurePin(PIN_PF_3, NORMAL_PIN(2));                /* SSP0_MOSI */

	//IOCON_configurePin(PIN_PF_4, NORMAL_PIN(2));                /* TRACE_CLK */
	//IOCON_configurePin(PIN_PF_5, NORMAL_PIN(3));                /* TRACE_DATA[0] */

	IOCON_configurePin(PIN_PF_6, NORMAL_PIN(4));                /* GPIO7.20 (touchscreen functionality) / TRACE_DATA[1] */
	IOCON_configurePin(PIN_PF_7, NORMAL_PIN(4));                /* GPIO7.21 (touchscreen functionality)/ TRACE_DATA[2] */
	IOCON_configurePin(PIN_PF_8, NORMAL_PIN(4));                /* GPIO7.22 (touchscreen functionality)/ TRACE_DATA[3] */

	IOCON_configurePin(PIN_PF_9, NORMAL_PIN(7));                /* ADC0_2 */
	IOCON_configurePin(PIN_PF_10, NORMAL_PIN(1));               /* U0_TXD */
	IOCON_configurePin(PIN_PF_11, NORMAL_PIN(1));               /* U0_RXD */


	IOCON_configurePin(PIN_I2C0, IOCON_makeConfigI(
        PIN_I2C0_GLITCHFILTER_50NS, PIN_I2C0_SPEED_FASTMODE));  /* I2C0_SDA, I2C0_SCL */
#endif /* #if BOARD_TYPE == BOARD_REVA3 */
   


    if (IOCON_checkErrors() != LPCLIB_SUCCESS) {
        while (1)
            ;
    }
}



#if LPCLIB_EMC
EMC_Handle theEmc;

static const EMC_Config emcConfig[] = {
    {.opcode = EMC_OPCODE_DYNAMIC_GLOBALS, .continued = ENABLE,
        {.dynTimings = {
            .refreshCycles = 500,
            .trp = 2,
            .tras = 4,
            .tsrex = 6,
            .tapr = 1,
            .tdal = 4,
            .twr = 2,
            .trc = 6,
            .trfc = 6,
            .txsr = 6,
            .trrd = 2,
            .tmrd = 2,}}},

    /* SDRAM: ISSI IS42S16400D-7IL 64 MBit */
    {.opcode = EMC_OPCODE_DYNAMIC_DEVICE, .continued = ENABLE,
        {.dynDevice = {
            .device = EMC_DYCS0,
            .lowPower = DISABLE,
            .ras = EMC_RAS_DELAY_3,
            .cas = EMC_CAS_DELAY_3,
            .busWidth = EMC_DYNAMIC_BUS_16,
            .org = EMC_DYNAMIC_ORG_1Mx16BITx4_12R_8C,}}},

    /* SRAM: ISSI IS62WV25616BLL-55TLI 256 KiB */
    {.opcode = EMC_OPCODE_STATIC_DEVICE, .continued = ENABLE,
        {.staticDevice = {
            .device = EMC_CS2,
            .busWidth = EMC_STATIC_BUS_16,
            .multi = EMC_STATIC_SINGLE_DEVICE,
            .writePulseWidth = 7,
            .writePulseDelay = 1,
            .readCycleWidth = 9,
            .readCycleWidthPage = 9,
            .readOeDelay = 0, }}},

    {.opcode = EMC_OPCODE_STATIC_DEVICE, .continued = DISABLE,
        {.staticDevice = {
            .device = EMC_CS0,
            .busWidth = EMC_STATIC_BUS_16,
            .multi = EMC_STATIC_SINGLE_DEVICE,
            .writePulseWidth = 10,
            .writePulseDelay = 1,
            .readCycleWidth = 10,
            .readCycleWidthPage = 10,
            .readOeDelay = 0, }}},
};
#endif


#if LPCLIB_EMC
/* Initialize external memory. */
void BSP_initEmc (void)
{
    EMC_open(&theEmc);
    EMC_ioctl(theEmc, &emcConfig[0]);

    // Something is wrong with the settings done before, it only work up to 108MHz
    // The function call below uses the settings from an older project and then it works on 120MHz and even on 132MHz
    EMC_Init();
}
#endif




/* Control a single LED. */
void BSP_writeLedBit (BSP_Led led, int value)
{
    if ((led & 0xFFFF0000 >> 16) == 0) {
    #if BOARD_TYPE == BOARD_REVA2
        PCA9502_writeBit(portExpander, (PCA9502_Pin)led, value);
    #endif
    #if (BOARD_TYPE == BOARD_REVA3) || (BOARD_TYPE == BOARD_REVA4)
        PCA9673_writeBit(portExpander, (PCA9673_Pin)led, value);
    #endif
    }

    if ((led & 0xFFFF0000 >> 16) == 1) {
        GPIO_writeBit((GPIO_Pin)(led & 0xFFFF), value);
    }
}



/********** Standard Library I/O **********/

#if (defined(__GNUC__))

#include <sys/stat.h>

extern uint32_t __heap_start;


int _fstat (int fd, struct stat *st);
int _fstat (int fd, struct stat *st)
{
    (void) fd;
    (void) st;

    return -1;
}


int _read (int fd, char *ptr, int len);
int _read (int fd, char *ptr, int len)
{
    (void) fd;

    int bytesUnRead = len;
    int32_t c;

    c = ITM_ReceiveChar();
    if (c >= 0) {
        *ptr = c;
        --bytesUnRead;
    }

    return len - bytesUnRead;
}


int _write (int fd, const char *ptr, int len);
int _write (int fd, const char *ptr, int len)
{
    (void) fd;

    int i;
    int bytesUnWritten = -1;
    char c;


    for (i = 0; i < len; i++) {
        if (*ptr == '\n') {
            c = '\r';
            ITM_SendChar(c);
        }
        ITM_SendChar(*ptr);
        ++ptr;
    }
    bytesUnWritten = len - i;

    return len - bytesUnWritten;
}


int _close (int fd);
int _close (int fd)
{
    (void) fd;

    return 0;
}


int _lseek (int fd, int ptr, int dir);
int _lseek (int fd, int ptr, int dir)
{
    (void)fd;
    (void)ptr;
    (void)dir;

    return 0;
}


int _isatty (int fd);
int _isatty (int fd)
{
    return (fd <= 2) ? 1 : 0;   /* one of stdin, stdout, stderr */
}


caddr_t _sbrk (int incr);
caddr_t _sbrk (int incr)
{
    static caddr_t heap = NULL;
    caddr_t prev_heap;

    if (heap == NULL) {
        heap = (caddr_t)&__heap_start;
    }

    prev_heap = heap;
    heap += incr;

    return prev_heap;
}


#else


FILE __stdout;
FILE __stdin;


int fputc (int ch, FILE *f)
{
    (void) f;

    if (ch == '\n') {
        ch = '\r';
    }
    ITM_SendChar(ch);

    return ch;
}


#if defined(__ICCARM__)
int fgetc (FILE *f)
#else
int getc (FILE *f)
#endif
{
    int c;

    c = ITM_ReceiveChar();
    if (c >= 0) {
        return c;
    }

    return EOF;
}

int ferror (FILE *f)
{
    return EOF;
}
#endif

