#ifndef __BSP_H
#define __BSP_H

#include "lpclib.h"


/* Board type */
#define BOARD_REVA2         2
#define BOARD_REVA3         3
#define BOARD_REVA4         4
#ifndef BOARD_TYPE
#warning "BOARD_TYPE defaults to BOARD_REVA4"
#define BOARD_TYPE          BOARD_REVA4
#endif


/* TFT type */
#define TFT_32  1
#define TFT_7   2
#ifndef BOARD_TYPE
#warning "TFT_TYPE defaults to TFT_32"
#define TFT_TYPE          TFT_32
#endif


/* GPIO functions */
#if BOARD_TYPE == BOARD_REVA2
#define BUTTON_IRQ_INPUT                    GPIO_4_8        /* PA_1 */
#define SPI_CS1                             GPIO_3_12
#define LCD_A0                              GPIO_4_10
#define LCD_RST                             GPIO_4_9
#endif

#if (BOARD_TYPE == BOARD_REVA3) || (BOARD_TYPE == BOARD_REVA4)
#define BUTTON_IRQ_INPUT                    GPIO_4_8        /* PA_1  */
#define SSEL_MUX_A                          GPIO_6_10       /* PC_11 */
#define SSEL_MUX_B                          GPIO_6_11       /* PC_12 */
#define LCD_A0                              GPIO_5_5        /* P2_5  */
#define LCD_RST                             GPIO_6_14       /* PD_0  */
#define JENIC_RST                           GPIO_3_2        /* P6_3  */
#define TFT_CMD_DATA                        GPIO_7_8        /* PE_8, used for EA display modules  */
#define TFT_RESET                           GPIO_3_11       /* P7_3  (LCDV17), used for reset of EA display modules  */
#define TOUCH_IRQ                           GPIO_7_7        /* used for EA display modules  */
#define UDA_RST                             GPIO_7_9
#define SDIO_LED							GPIO_6_1
#define RGB_LED_BLUE					    GPIO_7_8
#define RGB_LED_GREEN						GPIO_7_6
#define RGB_LED_RED							GPIO_7_5
#define LED_D12                             GPIO_4_14       /* P9_3 (ENET_RXD3)  */
#define LED_D13                             GPIO_4_15       /* P9_2 (ENET_RXD2/USB1_IND1) */
#define LED_D14                             GPIO_4_13       /* P9_1 (ENET_RX_ERR) */
#define LED_D15                             GPIO_4_12       /* P9_0 (ENET_CRS)    */
#endif


#if LPCLIB_I2C
extern I2C_Handle i2c_bus;

#if BOARD_TYPE == BOARD_REVA2
#include "pca9502.h"
#include "uda1380.h"

/* PCA9502 Port Expander (LED + Cap.Touch)
 * PCA9502 Port Expander (ADC)
 * SE97TP Temperature sensor
 * TFA9879HN Mono speaker amplifier
 */
#define PCA9502_ADDRESS                     0x4D
#define SE97_ADDRESS                        0x18
#define TFA9879_ADDRESS                     0x6C
#define UDA1380_ADDRESS                     0x1A

/* Pin assignments for the port expander */
#define LED0                                PCA9502_PIN0
#define LED1                                PCA9502_PIN1
#define LED2                                PCA9502_PIN2
#define LED3                                PCA9502_PIN3

extern PCA9502_Handle portExpander;
#endif

#if (BOARD_TYPE == BOARD_REVA3) || (BOARD_TYPE == BOARD_REVA4)
#include "pca9673.h"
#include "uda1380.h"

/* PCA9673 16-Port Expander (LED + Cap.Touch)
 * SE97TP Temperature sensor
 * TFA9879HN Mono speaker amplifier
 * UDA1380 stereo codec
 */
#define PCA9673_ADDRESS                     0x24
#define SE97_ADDRESS                        0x18
#define TFA9879_ADDRESS                     0x6C
#define UDA1380_ADDRESS                     0x1A

/* Pin assignments for the port expander */
#define LED0                                PCA9673_PIN00
#define LED1                                PCA9673_PIN01
#define LED2                                PCA9673_PIN02
#define LED3                                PCA9673_PIN03
#define JOYSTICK_RIGHT                      PCA9673_PIN10
#define JOYSTICK_UP                         PCA9673_PIN11
#define JOYSTICK_CENTER                     PCA9673_PIN12
#define JOYSTICK_LEFT                       PCA9673_PIN13
#define JOYSTICK_DOWN                       PCA9673_PIN14
#define BUTTON_SW3                          PCA9673_PIN15
#define BUTTON_SW5                          PCA9673_PIN16
#define BUTTON_SW6                          PCA9673_PIN17

extern PCA9673_Handle portExpander;
#endif
#endif


/* LED's */
typedef enum BSP_Led {
    BSP_LED0 = (0 << 16) | LED0,
    BSP_LED1 = (0 << 16) | LED1,
    BSP_LED2 = (0 << 16) | LED2,
    BSP_LED3 = (0 << 16) | LED3,

    /* LED's connected to GPIO pins */
#if (BOARD_TYPE == BOARD_REVA3) || (BOARD_TYPE == BOARD_REVA4)
    BSP_LED_SDIO = (1 << 16) | SDIO_LED,
#else
    BSP_LED_SDIO = -1,
#endif
} BSP_Led;



#if LPCLIB_SSP
#include "st7565r.h"
#include "generic-spi.h"
#include "tsc2046.h"

/* SPI devices on Hitex board (and EA QVGA display):
 *
 * LCD with controller ST7565R                      SSP0
 * TFT controller ILI9322                           SSP0
 * Touch controller TSC2046                         SSP0
 * High-speed ADC's                                 SSP0
 */

extern SSP_Handle ssp0_bus;

extern ST7565R_Handle lcd;
extern GENERICSPI_Handle tftController;
extern TSC2046_Handle touchController;
#endif


#if LPCLIB_DMA
extern DMA_Handle gpdma;
#endif


#if LPCLIB_EMC
/* External Memory Controller */
extern EMC_Handle theEmc;
#endif


/** Select pin multiplexers (and the electrical characteristics).
 *
 *  This must the first of three BSP function called during system boot. Typically called
 *  in SystemInit().
 */
void BSP_initPins (void);

/** Initialize external memory.
 *
 *  Called in SystemInit() after pins have been configured by BSP_initPins().
 */
void BSP_initEmc (void);

/** Setup of peripheral blocks and external devices.
 *
 *  Called in main() or the system initialization task. If using RTOS, the scheduler must be
 *  running when calling this function.
 */
void BSP_init (void);

/** Control all LED's.
 *
 *  \param[in] mask Affected LED's (0...31)
 *  \param[in] toggle If set, LED is toggled
 *  \param[in] value If not toggled, selected LED's are set/cleared according to value.
 */
uint32_t BSP_writeLed (uint32_t mask, uint32_t toggle, uint32_t value);


/** Control a single LED.
 *
 *  \param[in] led Affected LED (0...31)
 *  \param[in] value (0/1)
 */
void BSP_writeLedBit (BSP_Led led, int value);

#endif


