/***********************************************************************
 * $Id:   emc_LPC18xx_43xx.c
 *
 * Project: NXP LPC4300 LCD example
 *
 * Description:  Initialisation of the external memory interface and
 *               configuration for the specific memories connected to
 *               the LPC43xx
 *
 * Copyright(C) 2011, NXP Semiconductor
 * All rights reserved.
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * NXP Semiconductors assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. NXP Semiconductors
 * reserves the right to make changes in the software without
 * notification. NXP Semiconductors also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/

// Project includes
#include "lpc43xx.h"
//#include "system_LPC18xx_43xx.h"

#include "emc_LPC18xx_43xx.h"


// Macros and defines


/**********************************************************************
 ** Function prototypes
**********************************************************************/
#define DELAY_1usFreq  (1000000)        // 1MHz equivalent to 1usec
static uint32_t delayBase1us;           // calculated depending on M4/EMI frequency
static void vDelay(uint32_t u32Delay);  // delay function





/****************************************************************************************
* Call the required memory setup functions from here
*
*
****************************************************************************************/
void EMC_Init( void )
{
    // Initialise the control signals for static memories
    EMC_Config_Static();

    // The setup for dynamic memories (SDRAM)
    EMC_Init_SRDRAM(SDRAM_BASE, PART_WIDTH, PART_SIZE, EXT_WIDTH, COL_ADDR_BITS);

}




/****************************************************************************************
* Configure CS0 for 70ns 16-bit flash memory on the Hitex board
* Configure CS2 for 55ns 16-bit SRAM on the Hitex board
*
****************************************************************************************/
void EMC_Config_Static(void)
{

    // Configure CS0 for flash memory
    // @120MHz there should be 8 or 9 waitstates for the 70ns flash, apparently it works with 7
    LPC_EMC->S[0].STATICCONFIG |= 0x00000081;      // CS0: 16 bit = WE
    LPC_EMC->S[0].STATICWAITOEN = 0;              // CS0: WAITOEN = 0
    LPC_EMC->S[0].STATICWAITRD = 8;             // CS0: WAITRD = 7 

    // The Hitex board has external SRAM on CS2
    // @120MHz there should be 7 waitstates for the 55ns SRAM, it should work with 6
    LPC_EMC->S[2].STATICWAITOEN = 0;              // CS2: WAITOEN = 0
    LPC_EMC->S[2].STATICWAITRD = 7;               // CS2: WAITRD = 7	
}



  #define DOUT_DELAY      0
  #define CLK0_DELAY      6	  //
  #define CLKE0_DELAY     6	  //
  #define RAS_DELAY       0
  #define CAS_DELAY       0
  #define WE_DELAY        0
  #define DYCS0_DELAY     0
  #define DQM0_DELAY      0
  #define FBCLK0_DELAY    0
  #define CCLK_DELAY      0
  #define ADDR_DELAY      0
  #define DIN_DELAY       0
  #define DEN_DELAY       0


/****************************************************************************************
* Configure the delays for the SDRAM. These settings could be individual for each board.
* (see defines above)
*
* - on the Hitex board (IS42S16400D-7TL)
*
****************************************************************************************/
void initEmiDelays(void)
{
    // eventually configure delays, defaults are zero

    // CLK & CLKE0 delay
    *(uint32_t*)(LPC_SCU_BASE + 0xD00) = ((CLK0_DELAY << 16) | (CLKE0_DELAY << 0) );

    // EMCCTRLDELAY, address 0x4008 6D04
    *(uint32_t*)(LPC_SCU_BASE + 0xD04) = ((WE_DELAY << 12)| (CAS_DELAY << 4) | (RAS_DELAY << 0) );

    // DYCS0_DELAY, address 0x4008 6D08
    *(uint32_t*)(LPC_SCU_BASE + 0xD08) = ((DYCS0_DELAY << 0));

    // data out delay for D0 to D31   EMCDOUTDELAY
    *(uint32_t*)(LPC_SCU_BASE + 0xD0C) = ((DOUT_DELAY << 28) | (DOUT_DELAY << 24) | (DOUT_DELAY << 20) | (DOUT_DELAY << 16)|(DQM0_DELAY << 12) | (DQM0_DELAY << 8) | (DQM0_DELAY << 4) | (DQM0_DELAY << 0)) ;

    // EMCFBCLKDELAY, address 0x4008 6D10
    *(uint32_t*)(LPC_SCU_BASE + 0xD10) = ((CCLK_DELAY << 16)|(FBCLK0_DELAY << 12) | (FBCLK0_DELAY << 8) | (FBCLK0_DELAY << 4) | (FBCLK0_DELAY << 0)) ;

    // EMCADDRDELAY, address 0x4008 6D14, 0x4008 6D18, 0x4008 6D1C)
    *(uint32_t*)(LPC_SCU_BASE + 0xD14) = ((ADDR_DELAY << 28)|(ADDR_DELAY << 24)|(ADDR_DELAY << 20)|(ADDR_DELAY << 16)|(ADDR_DELAY << 12) | (ADDR_DELAY << 8) | (ADDR_DELAY << 4) | (ADDR_DELAY << 0)) ;
    *(uint32_t*)(LPC_SCU_BASE + 0xD18) = ((ADDR_DELAY << 28)|(ADDR_DELAY << 24)|(ADDR_DELAY << 20)|(ADDR_DELAY << 16)|(ADDR_DELAY << 12) | (ADDR_DELAY << 8) | (ADDR_DELAY << 4) | (ADDR_DELAY << 0)) ;
    *(uint32_t*)(LPC_SCU_BASE + 0xD1C) = ((ADDR_DELAY << 28)|(ADDR_DELAY << 24)|(ADDR_DELAY << 20)|(ADDR_DELAY << 16)|(ADDR_DELAY << 12) | (ADDR_DELAY << 8) | (ADDR_DELAY << 4) | (ADDR_DELAY << 0)) ;

    // data in delay for D0 to D31   EMCDINDELAY
    *(uint32_t*)(LPC_SCU_BASE + 0xD24) = ((DEN_DELAY << 28)|(DEN_DELAY << 24)|(DEN_DELAY << 20)|(DEN_DELAY << 16)|(DIN_DELAY << 12)|(DIN_DELAY << 8)|(DIN_DELAY << 4)|(DIN_DELAY << 0));

}


 
// Defines for SDRAM devices
#define EMC_B_ENABLE 					(1 << 19)
#define EMC_ENABLE 						(1 << 0)
#define EMC_CE_ENABLE 					(1 << 0)
#define EMC_CS_ENABLE 					(1 << 1)
#define EMC_CLOCK_DELAYED_STRATEGY 		(0 << 0)
#define EMC_COMMAND_DELAYED_STRATEGY 	(1 << 0)
#define EMC_COMMAND_DELAYED_STRATEGY2 	(2 << 0)
#define EMC_COMMAND_DELAYED_STRATEGY3 	(3 << 0)
#define EMC_INIT(i) 					((i) << 7)
#define EMC_NORMAL 						(0)
#define EMC_MODE 						(1)
#define EMC_PRECHARGE_ALL 				(2)
#define EMC_NOP 						(3)

/****************************************************************************************
* Configure the EMI for the SDRAM
*
* - on the Hitex board (IS42S16400D-7TL)
* - on the NXP evaluation board (MT48LC4M32B2)
* - on the NXP validation board (MT48LC4M32B2)
*
****************************************************************************************/
void EMC_Init_SRDRAM(uint32_t u32BaseAddr, uint32_t u32Width, uint32_t u32Size, uint32_t u32DataBus, uint32_t u32ColAddrBits)
{

	// calculate a 1 usec delay base 
    // TODO: change from hardcoded value to parameter (FreeRTOSConfig.h has the 120000000 settings as well)	
	delayBase1us = 120000000 / DELAY_1usFreq;

	// eventually adjust the CCU delays for EMI (default to zero)
	initEmiDelays();

	// Initialize EMC to interface with SDRAM. The EMC needs to run for this.
	LPC_EMC->CONTROL 			= 0x00000001;   // (Re-)enable the external memory controller	
	LPC_EMC->CONFIG 			= 0;

	LPC_EMC->D[0].DYNAMICCONFIG 	= ((u32Width << 7) | (u32Size << 9) | (u32DataBus << 14)); // Selects the configuration information for dynamic memory chip select 0.
	LPC_EMC->D[0].DYNAMICRASCAS 	= (2UL << 0) | (2UL << 8); // Selects the RAS and CAS latencies for dynamic memory chip select 0.	
	LPC_EMC->DYNAMICREADCONFIG	= EMC_COMMAND_DELAYED_STRATEGY;	 // Configures the dynamic memory read strategy.
	LPC_EMC->DYNAMICRP 		    = 1; // Selects the precharge command period
	LPC_EMC->DYNAMICRAS 		= 3; // Selects the active to precharge command period
	LPC_EMC->DYNAMICSREX 		= 5; // Selects the self-refresh exit time
	LPC_EMC->DYNAMICAPR 		= 0; // Selects the last-data-out to active command time
	LPC_EMC->DYNAMICDAL 		= 4; // Selects the data-in to active command time.
	LPC_EMC->DYNAMICWR 		    = 1; // Selects the write recovery time
	LPC_EMC->DYNAMICRC 	   	    = 5; // Selects the active to active command period
	LPC_EMC->DYNAMICRFC 		= 5; // Selects the auto-refresh period
	LPC_EMC->DYNAMICXSR 		= 5; // Selects the exit self-refresh to active command time
	LPC_EMC->DYNAMICRRD 		= 0; // Selects the active bank A to active bank B latency
	LPC_EMC->DYNAMICMRD 		= 0; // Selects the load mode register to active command time
	
	LPC_EMC->DYNAMICCONTROL 	= EMC_CE_ENABLE | EMC_CS_ENABLE | EMC_INIT(EMC_NOP);
	vDelay(100);
	
	LPC_EMC->DYNAMICCONTROL 	= EMC_CE_ENABLE | EMC_CS_ENABLE | EMC_INIT(EMC_PRECHARGE_ALL);

	LPC_EMC->DYNAMICREFRESH 	= 2; // Configures dynamic memory refresh operation
	vDelay(100);
	
	LPC_EMC->DYNAMICREFRESH 	= 83; // Configures dynamic memory refresh operation
	
	LPC_EMC->DYNAMICCONTROL 	= EMC_CE_ENABLE | EMC_CS_ENABLE | EMC_INIT(EMC_MODE);
	
	// Write configuration data to SDRAM device
	if(u32DataBus == 0)   // 16-bit data bus, the EMC enforces a burst size 8
	{
		*((volatile uint32_t *)(u32BaseAddr | ((3UL | (2UL << 4)) << (u32ColAddrBits + 2 + 1))));
	}
	else   // burst size 4 (which is not an option for 16-bit data bus anyway)
	{
		*((volatile uint32_t *)(u32BaseAddr | ((2UL | (2UL << 4)) << (u32ColAddrBits + 2 + 2))));
	}


	LPC_EMC->DYNAMICCONTROL   = 0;
	LPC_EMC->D[0].DYNAMICCONFIG |= EMC_B_ENABLE;   	// Enable the buffers

	// Buffers for the static memories can now be enabled as well. In a system with static and dynamic memory
	// this should only been done after the SDRAM initialisation --> here
	LPC_EMC->S[0].STATICCONFIG = 0x00080081;	

}


/**********************************************************************
 ** Function name:
 **
 ** Description:
 **
 ** Parameters:
 **
 ** Returned value:
 **********************************************************************/
static void vDelay(uint32_t u32Delay)
{
	volatile uint32_t i;
	
	for(i = 0; i < (u32Delay * delayBase1us); i++);
}

