/* Copyright (c) 2012, 
 * LPC1850/4350 Demo Software
 * All rights reserved.
 * no warranties
 */

#include <stdio.h>
#include <stdlib.h>
#include <inttypes.h>

#include "lpclib.h"

#include "FreeRTOS.h"
#include "task.h"

#include "task-gui.h"
#include "task-sys.h"
#include "task-led.h"

#include "bsp-hitex4300.h"


extern uint32_t SystemCoreClock;


/* Task priorities. */
#define mainGUI_TASK_PRIORITY         ((tskIDLE_PRIORITY + 0) | portPRIVILEGE_BIT)
#define mainLED_TASK_PRIORITY         ((tskIDLE_PRIORITY + 1) | portPRIVILEGE_BIT)
#define mainSYS_TASK_PRIORITY         ((tskIDLE_PRIORITY + 2) | portPRIVILEGE_BIT)


/** Task to initialize the system and all other tasks.
 *
 *  Initializes the system already under RTOS control. This is done because many drivers expect
 *  semaphores/mutexes to work. Sets up other tasks, then terminates itself after all initialization
 *  has been done.
 */
static void systemInit_task (void *pvParameters)
{
    (void) pvParameters;
    int n;

    /* Priority of peripheral interrupts depends on the application, and therefore the drivers
     * library does not set the priority.
     * After reset, all interrupts have highest priority, and this can lead to nasty failures
     * if your application uses an RTOS. Therefore it is a good idea to preset all interrupt
     * priorities to a safe default (the lowest priority). You may afterwards raise the
     * priority of individual interrupts as required by the application.
     */
    for (n = 0; n < 240; n++) {                         /* 240 is the maximum for any M4 implementation */
        NVIC_SetPriority((IRQn_Type)n, (1u << __NVIC_PRIO_BITS) - 1);
    }

    NVIC_SetPriority(I2C0_IRQn, 5);
    NVIC_SetPriority(SSP0_IRQn, 5);
    NVIC_SetPriority(PIN_INT0_IRQn, 5);

    /* Work on the clock tree. */
    CLKPWR_connectBaseClock(CLKPWR_BASECLOCK_SSP0, CLKPWR_SOURCE_PLL1);
    
    /* Prepare tasks */
    LED_initModule();
    GUI_initModule();
    SYS_initModule();


//TODO!!!!!!!!!!!!!!!!!!!!!!!!!!!
LPC_I2C0->CONCLR=0x60;
    BSP_init();


    /* Create tasks */
    LPCLIB_OSALTaskCreate("SYS", SYS_task, 2*configMINIMAL_STACK_SIZE, NULL, mainSYS_TASK_PRIORITY, NULL);
    LPCLIB_OSALTaskCreate("LED", LED_task, 1*configMINIMAL_STACK_SIZE, NULL, mainLED_TASK_PRIORITY, NULL);
    LPCLIB_OSALTaskCreate("GUI", GUI_task, 5*configMINIMAL_STACK_SIZE, NULL, mainGUI_TASK_PRIORITY, NULL);

    /* The systemInit task is no longer needed */
    LPCLIB_OSALTaskDelete(NULL);
}



int main (void)
{
    /* Create setup task */
    LPCLIB_OSALTaskCreate("i", systemInit_task, 3*configMINIMAL_STACK_SIZE, NULL, configMAX_PRIORITIES - 1, NULL);

    /* Start the scheduler. */
    LPCLIB_OSALStart();
    while(1)
        ;
}



/* Override system initialization (runs before main()!).
 * Set CPU clock to speed up the startup process.
 */
void SystemInit (void)
{
    /* Enable the FPU. Will be used already by scatter loading! */
    CLKPWR_enableFPU();

    SystemCoreClock = 0;
    CLKPWR_setCpuClock(configCPU_CLOCK_HZ);

    BSP_initPins();

    theEmc = LPCLIB_INVALID_HANDLE;
    BSP_initEmc();
}


