/* Copyright (c) 2012, 
 * LPC1850/4350 Demo Software
 * All rights reserved.
 * no warranties
 */

#include "bsp-hitex4300.h"

#include "lpclib.h"
#include "task-led.h"

#define LED_QUEUE_LENGTH                        4

typedef struct {
    uint8_t opcode;
} LED_Message;

/** Message opcodes for LED task. */
enum {
    LED_OPCODE_LED_TICK,                    /**< Time to toggle the LED */
};

struct {
    LPCLIB_OSAL_QUEUE queue;
    LPCLIB_OSAL_TIMER ledTick;
} ledContext;

enum {
    LED_TIMERMAGIC_LED,
};

/** Callback for OS timers.
 */
static void LED_osalCallback (LPCLIB_OSAL_TIMER handle)
{
    (void) handle;
    PHANDLE magic;

    LED_Message message;

    LPCLIB_OSALTimerGetMagic(handle, &magic);
    switch ((int)magic) {
    case LED_TIMERMAGIC_LED:
        message.opcode = LED_OPCODE_LED_TICK;
        LPCLIB_OSALQueuePost(ledContext.queue, &message, LPCLIB_OSAL_QUEUE_BACK, LPCLIB_OSAL_TIMEOUT_NONE, NULL, LPCLIB_OSAL_CALL_DEFAULT);
        break;
    }
}

void LED_task (void *pvParameters)
{
    (void) pvParameters;
    LED_Message message;
    int i = 0;

    /* Reminder for LED blinker */
    LPCLIB_OSALTimerCreate("", LPCLIB_OSAL_TIMER_PERIODIC, (PHANDLE)LED_TIMERMAGIC_LED, 0, 150, LED_osalCallback, &ledContext.ledTick);
    LPCLIB_OSALTimerStart(ledContext.ledTick);

    while (1) {
        /* Is there a new message? */
        if (LPCLIB_OSALQueuePend(ledContext.queue,
                                 &message,
                                 LPCLIB_OSAL_TIMEOUT_INFINITE,
                                 NULL,
                                 LPCLIB_OSAL_CALL_DEFAULT) == LPCLIB_OSAL_ERROR_NONE) {
            switch (message.opcode) {
            case LED_OPCODE_LED_TICK:

                GPIO_writeBit(SDIO_LED, GPIO_readBit(SDIO_LED) ^ 1);      // Toggle the SDIO LED

                ++i; if(i >= 4) i = 0;

                switch(i){
                case 0: 
						//GPIO_writeBit(RGB_LED_BLUE, 0);    // 0 switches RGB LED on
                        
                        PCA9673_writeBit(portExpander, PCA9673_PIN03, 1);
                        PCA9673_writeBit(portExpander, PCA9673_PIN00, 0);

                        if(( PCA9673_readBit(portExpander, BUTTON_SW3) == 0) | PCA9673_readBit(portExpander, JOYSTICK_UP) == 0 )  // button SW3 or joystick UP
                        {
                            GPIO_writeBit(LED_D12, 0);     // switch on red LED D12 
                        }
                        else
                        {
                            GPIO_writeBit(LED_D12, 1);     // switch off LED D12
                        }
                        break;
                case 1: 
						//GPIO_writeBit(RGB_LED_BLUE, 1);   // 1 switches RGB LED off                        
                        
                        PCA9673_writeBit(portExpander, PCA9673_PIN00, 1);
                        PCA9673_writeBit(portExpander, PCA9673_PIN01, 0);
                                                
                        if(( PCA9673_readBit(portExpander, BUTTON_SW5) == 0) | PCA9673_readBit(portExpander, JOYSTICK_RIGHT) == 0 )   // button SW5 or joystick RIGHT
                        {
                            GPIO_writeBit(LED_D13, 0);     // switch on yellow LED D13 
                        }
                        else
                        {
                            GPIO_writeBit(LED_D13, 1);     // switch off LED D13
                        }                   
 
                        break;
                case 2: 
						//GPIO_writeBit(RGB_LED_BLUE, 1);

                        PCA9673_writeBit(portExpander, PCA9673_PIN01, 1);
                        PCA9673_writeBit(portExpander, PCA9673_PIN02, 0);

                        if(( PCA9673_readBit(portExpander, BUTTON_SW6) == 0) | PCA9673_readBit(portExpander, JOYSTICK_DOWN) == 0 )   // button SW6 or joystick DOWN
                        {
                            GPIO_writeBit(LED_D14, 0);     // switch on yellow LED D14 
                        }
                        else
                        {
                            GPIO_writeBit(LED_D14, 1);     // switch off LED D14
                        }
                        break;
                case 3: 
						//GPIO_writeBit(RGB_LED_BLUE, 1); 

                        PCA9673_writeBit(portExpander, PCA9673_PIN02, 1);
                        PCA9673_writeBit(portExpander, PCA9673_PIN03, 0);

                        if(( PCA9673_readBit(portExpander, JOYSTICK_CENTER) == 0) | PCA9673_readBit(portExpander, JOYSTICK_LEFT) == 0 )   // joystick center or joystick LEFT
                        {
                            GPIO_writeBit(LED_D15, 0);     // switch on green LED D15 
                        }
                        else
                        {
                            GPIO_writeBit(LED_D15, 1);     // switch off LED D15
                        }
                        break;
                }
                break;
            }
        }
    }
}

void LED_initModule (void)
{
    /* Task communication */
    LPCLIB_OSALQueueCreate("", LED_QUEUE_LENGTH, sizeof(LED_Message), &ledContext.queue);
}
