
#ifndef __AT45DB081D_H
#define __AT45DB081D_H

#include <stdint.h>
#include <stdio.h>

#include "lpclib.h"


/** Handle for an AT45DB081 device. */
typedef struct AT45_Context *AT45_Handle;


/** Open the device.
 *
 *  Enable access to an AT45DB081 flash device.
 *
 *  \param[in] bus Handle of SSP bus to which the device is connected.
 *  \param[in] pDeviceSelect Method to handle SSEL demuxing
 *  \param[out] pHandle Device handle
 *  \retval LPCLIB_SUCCESS Success. \ref pHandle contains a valid handle.
 */
LPCLIB_Result AT45_open (SSP_Handle bus, const SSP_DeviceSelect *pDeviceSelect, AT45_Handle *pHandle);


/** Close access to flash device.
 *
 *  \param[in] pHandle Device handle
 *  \retval LPCLIB_SUCCESS Success.
 *  \retval LPCLIB_ILLEGAL_PARAMETER Invalid handle.
 */
LPCLIB_Result AT45_close (AT45_Handle *pHandle);


/** Read from flash.
 *
 *  \param[in] handle Device handle
 *  \param[out] rxBuffer Buffer that takes read data
 *  \param[in] numBytes Number of bytes to read.
 *  \retval LPCLIB_SUCCESS Success.
 *  \retval LPCLIB_ILLEGAL_PARAMETER Invalid handle.
 */
LPCLIB_Result AT45_read (AT45_Handle handle, void *rxBuffer, size_t numBytes);


/** Write to flash.
 *
 *  \param[in] handle Device handle
 *  \param[out] txBuffer Buffer that takes holds data
 *  \param[in] numBytes Number of bytes to write.
 *  \retval LPCLIB_SUCCESS Success.
 *  \retval LPCLIB_ILLEGAL_PARAMETER Invalid handle.
 */
LPCLIB_Result AT45_write (AT45_Handle handle, const void *txBuffer, size_t numBytes);


/** Position file pointer.
 */
LPCLIB_Result AT45_lseek (AT45_Handle handle, int32_t offset, LPCLIB_FilePos whence);

#endif
