
#ifndef __GENERIC_SPI_H
#define __GENERIC_SPI_H

//#include <stdint.h>

#include "lpclib.h"

/** Handle for a generic SPI device. */
typedef struct GENERICSPI_Context *GENERICSPI_Handle;


/** Open the device.
 *
 *  Enable access to a generic SPI device.
 *
 *  \param[in] bus Handle of SSP bus to which the device is connected.
 *  \param[in] pDeviceSelect Method to handle SSEL demuxing
 *  \param[out] pHandle Device handle
 *  \retval LPCLIB_SUCCESS Success. \ref pHandle contains a valid handle.
 */
LPCLIB_Result GENERICSPI_open (SSP_Handle bus, const SSP_DeviceSelect *pDeviceSelect, GENERICSPI_Handle *pHandle);



/** Configure the bit clock.
 *
 *  \param[in] handle
 *  \param[in] bitclock Bit clock frequency
 */
LPCLIB_Result GENERICSPI_setBitClock (GENERICSPI_Handle handle, uint32_t bitclock);



/** Send and receive.
 *
 *  \param[in] handle Device handle
 *  \param[in] pConfig Format and speed
 *  \param[in] pTx Pointer to TX data
 *  \param[out] pRx Pointer to RX data
 *  \param[in] numFrames Number of frames
 *  \retval LPCLIB_SUCCESS Success.
 *  \retval LPCLIB_ERROR Error.
 */
LPCLIB_Result GENERICSPI_sendAndReceive (GENERICSPI_Handle handle,
                                         const SSP_Config *pConfig,
                                         const void *pTx,
                                         void *pRx,
                                         int numFrames);

#endif
