
#ifndef __LED7_H
#define __LED7_H

#include <stdint.h>

#include "lpclib.h"

/** Handle for a LED7X device. */
typedef struct LED7X_Context *LED7X_Handle;


/** Open the device.
 *
 *  Enable access to the LED7 device on the Xpresso base board.
 *
 *  \param[in] bus Handle of SSP bus to which the device is connected.
 *  \param[in] pDeviceSelect Method to handle SSEL demuxing
 *  \param[out] pHandle Device handle
 *  \retval LPCLIB_SUCCESS Success. \ref pHandle contains a valid handle.
 */
LPCLIB_Result LED7X_open (SSP_Handle bus, const SSP_DeviceSelect *pDeviceSelect, LED7X_Handle *pHandle);


/** Control all segments including the decimal point with a pattern.
 *
 *  \param[in] handle Device handle
 *  \param[in] pattern Bit 0=d, 1=e, 2=g, 3=a, 4=b, 5=DP, 6=c, 7=f (Bit=1 --> segment off!)
 */
LPCLIB_Result LED7X_writePattern (LED7X_Handle handle, uint8_t pattern);


/** Show a digit on the 7-segment display.
 *
 *  \param[in] handle Device handle
 *  \param[in] digit Hex value (0...F) to be shown.
 */
void LED7X_writeDigit (LED7X_Handle handle, uint8_t digit);


#endif
