
#ifndef __24LC08_H
#define __24LC08_H

#include <stdint.h>

#include "lpclib.h"

/** \defgroup M24LC08_Public_Types M24LC08 Types, enums, macros
 *  @{
 */


/** Handle for a M24LC08 device. */
typedef struct M24LC08_Context *M24LC08_Handle;


/** @} M24LC08 Types, enums, macros */


/** \defgroup M24LC08_Public_Functions M24LC08 API Functions
 *  @{
 */


/** Open the device.
 *
 *  Enable access to a M24LC08 EEPROM connected to an already open I2C bus.
 *
 *  \param[in] bus Handle of I2C bus to which the device is connected.
 *  \param[in] address I2C device address
 *  \param[out] pHandle Device handle
 *  \retval LPCLIB_SUCCESS Success. \ref pHandle contains a valid handle.
 */
LPCLIB_Result M24LC08_open (I2C_Handle bus, const uint8_t address, M24LC08_Handle *pHandle);


/** Write a block of data to the EEPROM.
 *
 *  \param[in] handle Device handle as obtained by \ref M24LC08_open.
 *  \param[in] startAddress Address of first byte to be written.
 *  \param[in] numBytes Number of bytes to be written.
 *  \param[in] txBuffer Buffer that holds the data
 */
LPCLIB_Result M24LC08_write (M24LC08_Handle handle,
                             const uint32_t startAddress,
                             uint32_t numBytes,
                             const uint8_t *txBuffer);


/** Read a block of arbitrary length.
 *
 *  Read length can be from one byte up to the end of the EEPROM.
 *
 *  \param[in] handle Device handle as obtained by \ref M24LC08_open.
 *  \param[in] startAddress Address of first byte to be read
 *  \param[in] numBytes Number of bytes to be read
 *  \param[out] rxBuffer Buffer that takes the data
 *  \retval LPCLIB_SUCCESS Ok
 *  \retval LPCLIB_ILLEGAL_PARAMETER Illegal start address and/or length.
 *  \retval ...
 */
LPCLIB_Result M24LC08_read (M24LC08_Handle handle,
                            const uint32_t startAddress,
                            uint32_t numBytes,
                            uint8_t *rxBuffer);


/** @} M24LC08 API Functions */

#endif
