
#ifndef __PCA9500_H
#define __PCA9500_H

#include <stdint.h>

#include "lpclib.h"


/** \defgroup PCA9500_Public_Types PCA9500 Types, enums, macros
 *  @{
 */


/** Handle for a PCA9500 device. */
typedef struct PCA9500_Context *PCA9500_Handle;


/** @} PCA9500 Types, enums, macros */


/** \defgroup PCA9500_Public_Functions PCA9500 API Functions
 *  @{
 */


/** Open the device.
 *
 *  Enable access to a PCA9500 port expander / EEPROM connected to an already open I2C bus.
 *
 *  \param[in] bus Handle of I2C bus to which the device is connected.
 *  \param[in] address I2C device address
 *  \param[out] pHandle Device handle
 *  \retval LPCLIB_SUCCESS Success. \ref pHandle contains a valid handle.
 */
LPCLIB_Result PCA9500_open (I2C_Handle bus, const uint8_t address, PCA9500_Handle *pHandle);


/** Read all input pins.
 *
 *  Read as one (half-)word the state of all input pins.
 *
 *  \param[in] handle Device handle as obtained by \ref PCA9500_open.
 *  \param[in] pPortState Pointer to result
 */
LPCLIB_Result PCA9500_read8 (PCA9500_Handle handle, uint8_t *pPortState);


/** Write value (0/1) to a PCA9500 port pin.
 *
 *  \param[in] handle Port expander device handle
 *  \param[in] pin Pin selector
 *  \param[in] value Binary value to be written to port pin.
 */
LPCLIB_Result PCA9500_writeBit (PCA9500_Handle handle, uint8_t pin, uint8_t value);


/** Write value to the 8-bit port.
 *
 *  \param[in] handle Port expander device handle
 *  \param[in] value 8-bit value to be written to port pins.
 */
LPCLIB_Result PCA9500_write8 (PCA9500_Handle handle, uint8_t value);


/** @} PCA9532 API Functions */

#endif
