
#ifndef __PCF2129SPI_H
#define __PCF2129SPI_H

#include <stdint.h>

#include "lpclib.h"

/** Handle for a PCF2129 device. */
typedef struct PCF2129_Context *PCF2129_Handle;


typedef __PACKED(struct PCF2129_TimeDate {
    uint8_t seconds;
    uint8_t minutes;
    uint8_t hours;
    uint8_t days;
    uint8_t weekdays;
    uint8_t months;
    uint8_t years;
} ) PCF2129_TimeDate;


typedef struct PCF2129_TimeDateCtime {
    uint32_t ctime0;
    uint32_t ctime1;
} PCF2129_TimeDateCtime;


/** Open the device.
 *
 *  Enable access to a PCF2129 RTC device.
 *
 *  \param[in] bus Handle of SSP bus to which the device is connected.
 *  \param[in] pDeviceSelect Method to handle SSEL demuxing
 *  \param[out] pHandle Device handle
 *  \retval LPCLIB_SUCCESS Success. \ref pHandle contains a valid handle.
 */
LPCLIB_Result PCF2129_open (SSP_Handle bus, const SSP_DeviceSelect *pDeviceSelect, PCF2129_Handle *pHandle);


/** Read current time and date.
 */
LPCLIB_Result PCF2129_readTimeDateCtime (PCF2129_Handle handle, PCF2129_TimeDateCtime *timedate);
LPCLIB_Result PCF2129_readTimeDate      (PCF2129_Handle handle, PCF2129_TimeDate      *timedate);


/** Let the RTC run.
 */
LPCLIB_Result PCF2129_run (PCF2129_Handle handle);

#endif
