
#ifndef __SSD1305_H
#define __SSD1305_H

#include <stdint.h>

#include "lpclib.h"


/** \defgroup SSD1305_Public_Types SSD1305 Types, enums, macros
 *  @{
 */


/** Opcodes to specify the configuration command in a call to \ref SSD1305_ioctl. */
typedef enum SSD1305_Opcode {
    SSD1305_OPCODE_INIT,                    /**< Initialize the display controller */
    SSD1305_OPCODE_CLEAR_SCREEN,            /**< Clear screen content */
} SSD1305_Opcode;



/** Descriptor to specify the configuration in a call to \ref SSD1305_ioctl. */
typedef struct SSD1305_Config {
    SSD1305_Opcode opcode;                  /**< Config action opcode */
} SSD1305_Config;


/** Handle for an SSD1305 device. */
typedef struct SSD1305_Context *SSD1305_Handle;



/** @} SSD1305 Types, enums, macros */


/** \defgroup SSD1305_Public_Functions SSD1305 API Functions
 *  @{
 */


/** Open the device.
 *
 *  Enable access to an SSD1305 display controller.
 *
 *  \param[in] bus Handle of I2C bus to which the device is connected.
 *  \param[in] address I2C device address
 *  \param[out] pHandle Device handle
 *  \retval LPCLIB_SUCCESS Success. \ref pHandle contains a valid handle.
 */
LPCLIB_Result SSD1305_open (I2C_Handle bus, const uint8_t address, SSD1305_Handle *pHandle);


/** Configure the device.
 *
 *  Pass a configuration command to the SSD1305.
 *
 *  \param[in] handle Device handle.
 *  \param[in] pConfig Pointer to a configuration descriptor
 */
LPCLIB_Result SSD1305_ioctl (SSD1305_Handle handle, const SSD1305_Config *pConfig);


//TODO
LPCLIB_Result SSD1305_writeFullScreen (SSD1305_Handle handle, const uint8_t *frameBuffer);

/** @} SSD1305 API Functions */

#endif
