
#ifndef __TSC2046_H
#define __TSC2046_H

#include <stdint.h>

#include "lpclib.h"

/** Handle for a TSC2046 device. */
typedef struct TSC2046_Context *TSC2046_Handle;

/** Command code. */
typedef uint8_t TSC2046_Command;



/** Open the device.
 *
 *  Enable access to a TSC2046 Touch Screen Controller.
 *
 *  \param[in] bus Handle of SSP bus to which the device is connected.
 *  \param[in] pDeviceSelect Method to handle SSEL demuxing
 *  \param[out] pHandle Device handle
 *  \retval LPCLIB_SUCCESS Success. \ref pHandle contains a valid handle.
 */
LPCLIB_Result TSC2046_open (SSP_Handle bus, const SSP_DeviceSelect *pDeviceSelect, TSC2046_Handle *pHandle);


/** Send a command, and receice a response.
 *
 *  \param[in] handle Device handle.
 *  \param[in] command Command code
 *  \param[out] pResponse Pointer to 16-bit response
 *  \retval LPCLIB_SUCCESS Success
 *  \retval LPCLIB_ERROR Transfer error
 */
LPCLIB_Result TSC2046_sendCommand (TSC2046_Handle handle,
                                   TSC2046_Command command,
                                   uint16_t *pResponse);


/** Send a command sequence without a response.
 *
 *  \param[in] handle Device handle.
 *  \param[in] pCommand Command code(s)
 *  \param[in] numCommands Number of commands
 *  \retval LPCLIB_SUCCESS Success
 *  \retval LPCLIB_ERROR Transfer error
 */
LPCLIB_Result TSC2046_sendCommandSequence (TSC2046_Handle handle,
                                           const TSC2046_Command *pCommands,
                                           int numCommands);

#endif
