/* Copyright (c) 2012, 
 * LPC1850/4350 Demo Software
 * All rights reserved.
 * no warranties
 */
#include <stdint.h>
#include <string.h>

#include "lpclib.h"

#include "generic-spi.h"

/** Local device context. */
typedef struct GENERICSPI_Context {
    SSP_Handle bus;                      /**< Bus to which device is connected. */
    const SSP_DeviceSelect *pDeviceSelect;  /**< SSEL demuxing. */
    uint32_t bitclock;                   /**< Bit clock frequency */
} GENERICSPI_Context;

static GENERICSPI_Context globalContext; //TODO: Must allocate memory dynamically!!

/* Send and receive. */
LPCLIB_Result GENERICSPI_sendAndReceive (GENERICSPI_Handle handle,
                                         const SSP_Config *pConfig,
                                         const void *pTx,
                                         void *pRx,
                                         int numFrames)
{
    SSP_Job job;
    SSP_JobPhase phase;

    job.firstPhase = SSP_makePhase(&phase, pTx, pRx, numFrames);

    job.pDeviceSelect = handle->pDeviceSelect;
    job.pConfig = pConfig;

    if (SSP_submitJob(handle->bus, &job) != LPCLIB_SUCCESS) {
        return LPCLIB_ERROR;
    }

    return LPCLIB_SUCCESS;
}

LPCLIB_Result GENERICSPI_open (SSP_Handle bus, const SSP_DeviceSelect *pDeviceSelect, GENERICSPI_Handle *pHandle)
{
    GENERICSPI_Context *pContext = &globalContext; //TODO


    pContext->bus = bus;
    pContext->pDeviceSelect = pDeviceSelect;

    *pHandle = pContext;

    return LPCLIB_SUCCESS;
}

/* Configure the bit clock. */
LPCLIB_Result GENERICSPI_setBitClock (GENERICSPI_Handle handle, uint32_t bitclock)
{
    handle->bitclock = bitclock;

    return LPCLIB_SUCCESS;
}
