
#include <stdint.h>
#include <string.h>

#include "lpclib.h"
#include "i2cdev.h"

#include "isl29003.h"


/** ISL29003 Register Names. */
enum ISL29003_RegisterNames {
    ISL29003_REG_COMMAND = 0,
    ISL29003_REG_CONTROL = 1,
    ISL29003_REG_INT_THRESHOLD_HI = 2,
    ISL29003_REG_INT_THRESHOLD_LO = 3,
    ISL29003_REG_LSB_SENSOR = 4,
    ISL29003_REG_MSB_SENSOR = 5,
    ISL29003_REG_LSB_TIMER = 6,
    ISL29003_REG_MSB_TIMER = 7,
};


/** Local device context. */
typedef struct ISL29003_Context {
    I2C_Handle bus;                         /**< Bus to which device is connected */
    uint8_t slaveAddress;                   /**< Slave address on I2C bus */
    uint8_t regs[8];                        /**< Current register content */
} ISL29003_Context;


static ISL29003_Context globalContext; //TODO: Must allocate memory dynamically!!


static LPCLIB_Result ISL29003_writeReg (ISL29003_Handle handle, enum ISL29003_RegisterNames theReg)
{
    uint8_t buf[2];


    buf[0] = theReg;
    buf[1] = handle->regs[theReg];

    return I2CDEV_write(handle->bus, handle->slaveAddress, 2, buf);
}


static LPCLIB_Result ISL29003_readReg (ISL29003_Handle handle, enum ISL29003_RegisterNames theReg)
{
    LPCLIB_Result result;
    uint8_t buf;


    buf = theReg;
    result = I2CDEV_write(handle->bus, handle->slaveAddress, 1, &buf);
    if (result == LPCLIB_SUCCESS) {
        result = I2CDEV_read(handle->bus, handle->slaveAddress, 1, &handle->regs[theReg]);
    }

    return result;
}



LPCLIB_Result ISL29003_open (I2C_Handle bus, const uint8_t address, ISL29003_Handle *pHandle)
{
    ISL29003_Context *pContext = &globalContext; //TODO!
    LPCLIB_Result result;
    uint8_t reg;


    pContext->bus = bus;
    pContext->slaveAddress = address;

    *pHandle = pContext;

    /* Read current settings */
    for (reg = 0; reg < 8; reg++) {
        result = ISL29003_readReg(*pHandle, (enum ISL29003_RegisterNames)reg);
        if (result != LPCLIB_SUCCESS) {
            break;
        }
    }

    return result;
}


LPCLIB_Result ISL29003_ioctl (ISL29003_Handle handle, const ISL29003_Config *pConfig)
{
    if (handle == LPCLIB_INVALID_HANDLE) {
        return LPCLIB_ILLEGAL_PARAMETER;
    }

    switch (pConfig->opcode) {
    case ISL29003_OPCODE_POWER:
        if (pConfig->power == DISABLE) {
            handle->regs[ISL29003_REG_COMMAND] &= ~(1u << 7);
            ISL29003_writeReg(handle, ISL29003_REG_COMMAND);
            handle->regs[ISL29003_REG_COMMAND] |= (1u << 6);
            ISL29003_writeReg(handle, ISL29003_REG_COMMAND);
        }
        else {
            handle->regs[ISL29003_REG_COMMAND] &= ~(1u << 6);
            ISL29003_writeReg(handle, ISL29003_REG_COMMAND);
            handle->regs[ISL29003_REG_COMMAND] |= (1u << 7);
            ISL29003_writeReg(handle, ISL29003_REG_COMMAND);
//handle->regs[ISL29003_REG_CONTROL] |= (1u << 2);
//ISL29003_writeReg(handle, ISL29003_REG_CONTROL);
        }
        break;


    default:
        break;
    }

    return LPCLIB_SUCCESS;
}



LPCLIB_Result ISL29003_read (ISL29003_Handle handle, ISL29003_LuxValue *pLux)
{
    if (handle == LPCLIB_INVALID_HANDLE) {
        return LPCLIB_ILLEGAL_PARAMETER;
    }

    ISL29003_readReg(handle, ISL29003_REG_LSB_SENSOR);
    ISL29003_readReg(handle, ISL29003_REG_MSB_SENSOR);

//TODO...
    *pLux = (handle->regs[ISL29003_REG_MSB_SENSOR] * 256) + handle->regs[ISL29003_REG_LSB_SENSOR];

    return LPCLIB_SUCCESS;
}

