
#include <stdint.h>
#include <string.h>

#include "lpclib.h"

#include "led7x.h"



/** Local device context. */
typedef struct LED7X_Context {
    SSP_Handle bus;                         /**< Bus to which device is connected. */
    const SSP_DeviceSelect *pDeviceSelect;  /**< SSEL demuxing. */
} LED7X_Context;


static LED7X_Context globalContext; //TODO: Must allocate memory dynamically!!


/** Mapping of HEX digit to segment.
 *
 *  a=0, b=1, c=7, d=3, e=4, f=6, g=2, DP=5
 */
static const uint8_t led7_digit[16] = {
    0x24, 0x7D, 0xE0, 0x70,
    0x39, 0x32, 0x22, 0x7C,
    0x20, 0x30, 0x28, 0x23,
    0xA6, 0x61, 0xA2, 0xAA,
};



static const SSP_Config led7xDevice[] = {
    {.opcode = SSP_OPCODE_SET_FORMAT, .continued = ENABLE,
        {.format = {.bits = SSP_BITS_8,
                    .clockFormat = SSP_CLOCK_CPHA0_CPOL0,
                    .frameFormat = SSP_FRAMEFORMAT_SPI, }}},
    {.opcode = SSP_OPCODE_SET_BITRATE, .continued = DISABLE,
        {.bitrate = 1000000ul, }},
};



LPCLIB_Result LED7X_writePattern (LED7X_Handle handle, uint8_t pattern)
{
    SSP_Job job;
    SSP_JobPhase phase;


    SSP_makePhase(&phase, &pattern, NULL, 1);

    job.firstPhase = &phase;
    job.pDeviceSelect = handle->pDeviceSelect;
    job.pConfig = &led7xDevice[0];

    if (SSP_submitJob(handle->bus, &job) != LPCLIB_SUCCESS) {
        return LPCLIB_ERROR;
    }

    return LPCLIB_SUCCESS;
}



LPCLIB_Result LED7X_open (SSP_Handle bus, const SSP_DeviceSelect *pDeviceSelect, LED7X_Handle *pHandle)
{
    LED7X_Context *pContext = &globalContext; //TODO


    pContext->bus = bus;
    pContext->pDeviceSelect = pDeviceSelect;

    *pHandle = pContext;

    return LPCLIB_SUCCESS;
}



/* Show a digit on the 7-segment display */
void LED7X_writeDigit (LED7X_Handle handle, uint8_t digit)
{
    if (handle == LPCLIB_INVALID_HANDLE) {
        return;
    }

    LED7X_writePattern(handle, led7_digit[digit & 0x0F]);
}
