
#include <stdint.h>
#include <string.h>

#include "lpclib.h"
#include "i2cdev.h"

#include "lm75.h"


/** Local device context. */
typedef struct LM75_Context {
    I2C_Handle bus;                         /**< Bus to which device is connected. */
    uint8_t slaveAddress;                   /**< Slave address on I2C bus. */
} LM75_Context;


static LM75_Context globalContext; //TODO: Must allocate memory dynamically!!


LPCLIB_Result LM75_open (I2C_Handle bus, const uint8_t address, LM75_Handle *pHandle)
{
    LM75_Context *pContext = &globalContext; //TODO!


    pContext->bus = bus;
    pContext->slaveAddress = address;

    *pHandle = pContext;

    return LPCLIB_SUCCESS;
}



LPCLIB_Result LM75_ioctl (LM75_Handle handle, const LM75_Config *pConfig)
{
    LPCLIB_Result result = LPCLIB_SUCCESS;
    uint8_t buf[3];


    if (handle == LPCLIB_INVALID_HANDLE) {
        return LPCLIB_ILLEGAL_PARAMETER;
    }

    while (1) {
        switch (pConfig->opcode) {
        case LM75_OPCODE_SET_THYST:
        case LM75_OPCODE_SET_TOS:
            buf[0] = pConfig->opcode;                   /* Pointer byte! */
            buf[1] = pConfig->temperature8_8 >> 8;
            buf[2] = pConfig->temperature8_8 & 0x80;
            return I2CDEV_write(handle->bus, handle->slaveAddress, 3, &buf);
        }

        if (!pConfig->continued) {
            break;
        }

        ++pConfig;                                      /* More config's follow */
    }

    return result;
}



LPCLIB_Result LM75_read (LM75_Handle handle, uint16_t *pTemperature)
{
    uint8_t buf;
    uint8_t reg[2];
    LPCLIB_Result result;


    if (handle == LPCLIB_INVALID_HANDLE) {
        return LPCLIB_ILLEGAL_PARAMETER;
    }

    buf = LM75_REG_TEMP;

    result = I2CDEV_writeAndRead(handle->bus, handle->slaveAddress, 1, &buf, 2, &reg[0]);

    if (result == LPCLIB_SUCCESS) {
        *pTemperature = (reg[0] << 8) + (reg[1] & 0xE0);
    }

    return result;
}

