
#include <stdint.h>
#include <string.h>

#include "lpclib.h"
#include "i2cdev.h"

#include "m24lc08.h"


#define M24LC08_SIZE                        (1024)      /* Size in bytes */


/** Local device context. */
typedef struct M24LC08_Context {
    I2C_Handle bus;                         /**< Bus to which device is connected. */
    uint8_t baseAddress;                    /**< Base slave address. */
} M24LC08_Context;


static M24LC08_Context globalContext; //TODO: Must allocate memory dynamically!!


/* Open the device */
LPCLIB_Result M24LC08_open (I2C_Handle bus, const uint8_t address, M24LC08_Handle *pHandle)
{
    M24LC08_Context *pContext = &globalContext; //TODO!

    pContext->bus = bus;
    pContext->baseAddress = address;

    *pHandle = pContext;

    return LPCLIB_SUCCESS;
}



/* Write to EEPROM */
LPCLIB_Result M24LC08_write (M24LC08_Handle handle,
                             const uint32_t startAddress,
                             uint32_t numBytes,
                             const uint8_t *txBuffer)
{
    LPCLIB_Result result;
    uint8_t buf[17];


    /* Check if programming range is inside EEPROM */
    if (startAddress + numBytes > M24LC08_SIZE) {
        return LPCLIB_ILLEGAL_PARAMETER;
    }

//TODO only support 1 byte at the moment... :-(

    buf[0] = startAddress & 0xFF;
    buf[1] = *txBuffer;
    result = I2CDEV_write(handle->bus, handle->baseAddress + (startAddress >> 8), 2, &buf);

    return result;
}



/* Read from EEPROM. */
LPCLIB_Result M24LC08_read (M24LC08_Handle handle,
                            const uint32_t startAddress,
                            uint32_t numBytes,
                            uint8_t *rxBuffer)
{
    uint8_t buf;


    /* Check if programming range is inside EEPROM */
    if (startAddress + numBytes > M24LC08_SIZE) {
        return LPCLIB_ILLEGAL_PARAMETER;
    }

    buf = startAddress & 0xFF;

    return I2CDEV_writeAndRead(handle->bus, handle->baseAddress + (startAddress >> 8), 1, &buf, numBytes, rxBuffer);
}

