
#include <stdint.h>
#include <string.h>

#include "lpclib.h"
#include "i2cdev.h"

#include "pca9500.h"


/** Local device context. */
typedef struct PCA9500_Context {
    I2C_Handle bus;                         /**< Bus to which device is connected. */
    uint8_t slaveAddress;                   /**< Slave address on I2C bus. */
} PCA9500_Context;


static PCA9500_Context globalContext; //TODO: Must allocate memory dynamically!!



LPCLIB_Result PCA9500_open (I2C_Handle bus, const uint8_t address, PCA9500_Handle *pHandle)
{
    PCA9500_Context *pContext = &globalContext; //TODO!


    pContext->bus = bus;
    pContext->slaveAddress = address;

    *pHandle = pContext;

    return LPCLIB_SUCCESS;
}




LPCLIB_Result PCA9500_read8 (PCA9500_Handle handle, uint8_t *pPortState)
{
    if (handle == LPCLIB_INVALID_HANDLE) {
        return LPCLIB_ILLEGAL_PARAMETER;
    }

    return I2CDEV_read(handle->bus, handle->slaveAddress, 1, pPortState);
}


/* Write value to the 8-bit port. */
LPCLIB_Result PCA9500_write8 (PCA9500_Handle handle, uint8_t value)
{
    uint8_t control = value ^ 0xFF;

    return I2CDEV_write(handle->bus, handle->slaveAddress, 1, &control);
}


/* Write value (0/1) to a PCA9500 port pin. */
LPCLIB_Result PCA9500_writeBit (PCA9500_Handle handle, uint8_t pin, uint8_t value)
{
    uint8_t current;
    LPCLIB_Result result;

    result = PCA9500_read8(handle, &current);
    if (result == LPCLIB_SUCCESS) {
        current = ((current ^ 0xFF) & ~(1u << pin)) | ((value & 1) << pin);
        result = PCA9500_write8(handle, current);
    }

    return result;
}

