/* Copyright (c) 2012, 
 * LPC1850/4350 Demo Software
 * All rights reserved.
 * no warranties
 */
#include <stdint.h>
#include <string.h>

#include "lpclib.h"

#if LPCLIB_I2C

#include "i2cdev.h"
#include "pca9673.h"

/** Local device context. */
typedef struct PCA9673_Context {
    I2C_Handle bus;                        /**< Bus to which device is connected. */
    uint8_t slaveAddress;                   /**< Slave address on I2C bus. */
    uint16_t outputMask;                    /**< A "1" in a bit position indicates an output. */
} PCA9673_Context;

static PCA9673_Context globalContext; //TODO: Must allocate memory dynamically!!

LPCLIB_Result PCA9673_open (I2C_Handle bus, const uint8_t address, PCA9673_Handle *pHandle)
{
    PCA9673_Context *pContext = &globalContext; //TODO!


    pContext->bus = bus;
    pContext->slaveAddress = address;

    *pHandle = pContext;

    return LPCLIB_SUCCESS;
}

LPCLIB_Result PCA9673_ioctl (PCA9673_Handle handle, const PCA9673_Config *pConfig)
{
    LPCLIB_Result result = LPCLIB_SUCCESS;
    uint16_t oldMask;


    if (handle == LPCLIB_INVALID_HANDLE) {
        return LPCLIB_ILLEGAL_PARAMETER;
    }

    while (1) {
        switch (pConfig->opcode) {
        case PCA9673_OPCODE_SET_PIN_DIRECTION:
            oldMask = handle->outputMask;
            if (pConfig->pinFunc.func == PCA9673_PINFUNC_INPUT) {
                handle->outputMask &= ~(1u << pConfig->pinFunc.pin);
            }
            if (pConfig->pinFunc.func == PCA9673_PINFUNC_OUTPUT) {
                handle->outputMask |= (1u << pConfig->pinFunc.pin);
            }

            /* Update pins if some have changed from output to input. */
            if (oldMask & (oldMask ^ handle->outputMask)) {
                result = PCA9673_write16WithMask(handle,
                                             0xFFFF,
                                             oldMask & (oldMask ^ handle->outputMask));
            }
            break;
        }

        if (!pConfig->continued) {
            break;
        }

        ++pConfig;                                      /* More config's follow */
    }

    return result;
}

LPCLIB_Result PCA9673_read (PCA9673_Handle handle, uint16_t *pPortState)
{
    uint8_t regs[2];
    LPCLIB_Result result;


    if (handle == LPCLIB_INVALID_HANDLE) {
        return LPCLIB_ILLEGAL_PARAMETER;
    }

    result = I2CDEV_read(handle->bus, handle->slaveAddress, 2, &regs[0]);

    *pPortState = (regs[1] << 8) | regs[0];

    return result;
}

/* Write value to the 16-bit port through a mask. */
LPCLIB_Result PCA9673_write16WithMask (PCA9673_Handle handle, uint16_t value, uint16_t mask)
{
    LPCLIB_Result result;
    uint16_t current;

    
    result = PCA9673_read(handle, &current);
    if (result == LPCLIB_SUCCESS) {
        current = (current & ~mask) | (value & mask);
        current = current | (~(handle->outputMask));    /* Force inputs high */
        
        result = I2CDEV_write(handle->bus,
                              handle->slaveAddress,
							  2,
                              &current);
    }

    return result;
}

/* Write value (0/1) to a PCA9673 port pin. */
LPCLIB_Result PCA9673_writeBit (PCA9673_Handle handle, PCA9673_Pin pin, uint8_t value)
{
    return PCA9673_write16WithMask(handle, ((value & 1) << pin), (1u << pin));
}

/* Return current state of a single port pin. */
uint32_t PCA9673_readBit (PCA9673_Handle handle, PCA9673_Pin pin)
{
    uint16_t port;

    if (PCA9673_read(handle, &port) == LPCLIB_SUCCESS) {
        return (port >> pin) & 1;                       /* Extract requested pin */
    }

    return 0;                                           /* Error. Return something anyway */
}

#endif  /* #if LPCLIB_I2C */
