
#include <stdint.h>
#include <string.h>

#include "lpclib.h"
#include "i2cdev.h"

#include "ssd1305.h"


/** Local device context. */
typedef struct SSD1305_Context {
    I2C_Handle bus;                         /**< Bus to which device is connected. */
    uint8_t slaveAddress;                   /**< Slave address on I2C bus. */
} SSD1305_Context;


static SSD1305_Context globalContext; //TODO: Must allocate memory dynamically!!



LPCLIB_Result SSD1305_open (I2C_Handle bus, const uint8_t address, SSD1305_Handle *pHandle)
{
    SSD1305_Context *pContext = &globalContext; //TODO!


    pContext->bus = bus;
    pContext->slaveAddress = address;

    *pHandle = pContext;

    return LPCLIB_SUCCESS;
}



static const uint8_t ssd1305InitSequence[] = {
    0x80, 0x40,     /* Display start line = 0 */
    0x80, 0x2E,     /* deactivate scroll */
    0x80, 0x20,     /* horizontal addressing mode */
    0x80, 0x81,     /* set contrast: */ 
    0x80, 0x32,     /*   --> 50 */
    0x80, 0x82,     /* set brightness */
    0x80, 0x80,     /*   --> 128 */
    0x80, 0xA1,     /* Map column 131 to SEG0 */
    0x80, 0xA6,     /* Normal display */
//    0x80, 0xA7,     /* Reverse display */
    0x80, 0xA8,     /* set multiplex ratio: */
    0x80, 0x3F,     /*   --> 63+1 = 64 */
    0x80, 0xD3,     /* set display offset: */
    0x80, 0x00,     /*   --> 0 */
    0x80, 0xAD,     /* Master config: */
    0x80, 0x8E,     /* --> External VCC */
    0x80, 0xC8,     /* set com output scan direction: from COM[N-1]...COM[0] */
    0x80, 0xD5,     //(set display clock divide ratio/oscillator/frequency)
    0x80, 0xF0,
    0x80, 0xD8,     //(set area color mode on/off & low power display mode )
    0x80, 0x05,
    0x80, 0xD9,     //(set pre-charge period)
    0x80, 0xF1,
    0x80, 0xDA,     //(set com pins hardware configuration)
    0x80, 0x12,
    0x80, 0xDB,     /* set vcom deselect level */
    0x80, 0x34,     /*   --> 0.77 Vcc */
    0x80, 0x91,     //(set look up table for area color)
    0x80, 0x3F,
    0x80, 0x3F,
    0x80, 0x3F,
    0x80, 0x3F,
    0x80, 0xAF,     /* display on in normal mode */
    0x00, 0xA4,     /* display on */
};


static const uint8_t commandFullScreen[] = {
    0x80,
    0x20,   /* Addressing mode: */
    0x00,   /*  --> horizontal addressing mode */
    0x80,
    0x21,
    0x10,   /* Start at 16 (not 18!) */
    0x71,   /* End in 113 (--> width = 98, first two columns invisible) */
    0x00,
    0x22,
    0x00,
    0x07,
};


LPCLIB_Result SSD1305_writeFullScreen (SSD1305_Handle handle, const uint8_t *frameBuffer)
{
    uint8_t data;
    I2C_Job job;
    I2C_JobPhase phase1, phase2;
    LPCLIB_Result result;


    result = I2CDEV_write(handle->bus, handle->slaveAddress, sizeof(commandFullScreen), commandFullScreen);
    if (result == LPCLIB_SUCCESS) {
        data = 0x40;
        phase1.next = &phase2;
        phase1.option = I2C_PHASE_SEND;
        phase1.txstart = &data;
        phase1.length = 1;
    
        phase2.next = NULL;
        phase2.option = I2C_PHASE_SEND;
        phase2.txstart = frameBuffer;
        phase2.length = 8u*98u;
    
        job.firstPhase = &phase1;
        job.slaveAddress = handle->slaveAddress;

        result = I2C_submitJob(handle->bus, &job);
    }

    return result;
}



static LPCLIB_Result SSD1305_clearScreen (SSD1305_Handle handle)
{
    uint8_t data[2];
    int i;

    LPCLIB_Result result;


    result = I2CDEV_write(handle->bus, handle->slaveAddress, sizeof(commandFullScreen), commandFullScreen);

    if (result == LPCLIB_SUCCESS) {
        for (i = 0; i < (8*98); i++) {
            data[0] = 0x40;
            data[1] = 0;
            result = I2CDEV_write(handle->bus, handle->slaveAddress, 2, data);
            if (result != LPCLIB_SUCCESS) {
                break;
            }
        }
    }

    return result;
}



/* Configure the device */
LPCLIB_Result SSD1305_ioctl (SSD1305_Handle handle, const SSD1305_Config *pConfig)
{
    switch (pConfig->opcode) {
    case SSD1305_OPCODE_INIT:
        return I2CDEV_write(handle->bus,
                            handle->slaveAddress,
                            sizeof(ssd1305InitSequence),
                            ssd1305InitSequence);
    case SSD1305_OPCODE_CLEAR_SCREEN:
        return SSD1305_clearScreen(handle);
    }

    return LPCLIB_SUCCESS;
}


