/* Copyright (c) 2012, 
 * LPC1850/4350 Demo Software
 * All rights reserved.
 * no warranties
 */
#include <stdint.h>
#include <string.h>

#include "lpclib.h"

#include "tsc2046.h"



/** Local device context. */
typedef struct TSC2046_Context {
    SSP_Handle bus;                         /**< Bus to which device is connected. */
    const SSP_DeviceSelect *pDeviceSelect;  /**< SSEL demuxing. */
} TSC2046_Context;


static TSC2046_Context globalContext; 		//TODO: Must allocate memory dynamically!!




static const SSP_Config tsc2046Device[] = {
    {.opcode = SSP_OPCODE_SET_FORMAT, .continued = ENABLE,
        {.format = {.bits = SSP_BITS_8,
                    .clockFormat = SSP_CLOCK_CPHA0_CPOL0,
                    .frameFormat = SSP_FRAMEFORMAT_SPI, }}},

    {.opcode = SSP_OPCODE_SET_BITRATE, .continued = DISABLE,
        {.bitrate = 2000000ul }},
};

/** Send a command, and receive the response.
 */
LPCLIB_Result TSC2046_sendCommand (TSC2046_Handle handle,
                                   TSC2046_Command command,
                                   uint16_t *pResponse)
{
    uint16_t response;
    SSP_Job job;
    SSP_JobPhase phase[2];


    job.firstPhase = SSP_makePhase(&phase[0], &command, NULL, 1);
    phase[0].next = SSP_makePhase(&phase[1], NULL, (uint8_t *)&response, 2);

    job.pDeviceSelect = handle->pDeviceSelect;
    job.pConfig = &tsc2046Device[0];

    if (SSP_submitJob(handle->bus, &job) != LPCLIB_SUCCESS) {
        return LPCLIB_ERROR;
    }

    *pResponse = __REV16(response);
    
    return LPCLIB_SUCCESS;
}

/* Send a command sequence without a response. */
LPCLIB_Result TSC2046_sendCommandSequence (TSC2046_Handle handle,
                                           const TSC2046_Command *pCommands,
                                           int numCommands)
{
    SSP_Job job;
    SSP_JobPhase phase;


    job.firstPhase = SSP_makePhase(&phase, pCommands, NULL, numCommands * sizeof(TSC2046_Command));

    job.pDeviceSelect = handle->pDeviceSelect;
    job.pConfig = &tsc2046Device[0];

    if (SSP_submitJob(handle->bus, &job) != LPCLIB_SUCCESS) {
        return LPCLIB_ERROR;
    }

    return LPCLIB_SUCCESS;
}

LPCLIB_Result TSC2046_open (SSP_Handle bus, const SSP_DeviceSelect *pDeviceSelect, TSC2046_Handle *pHandle)
{
    TSC2046_Context *pContext = &globalContext; //TODO


    pContext->bus = bus;
    pContext->pDeviceSelect = pDeviceSelect;

    *pHandle = pContext;

    return LPCLIB_SUCCESS;
}
