
#include <stdint.h>
#include <string.h>

#include "lpclib.h"

#if LPCLIB_I2C

#include "i2cdev.h"

#include "uda1380.h"


/** Local device context. */
typedef struct UDA1380_Context {
    I2C_Handle bus;                         /**< Bus to which device is connected. */
    uint8_t slaveAddress;                   /**< Slave address on I2C bus. */
} UDA1380_Context;


static UDA1380_Context globalContext; //TODO: Must allocate memory dynamically!!


/** UDA1380 Registers. */
enum UDA1380_Register {
    UDA1380_REG_CLOCK = 0x00,               /* Clock settings */
    UDA1380_REG_MASTER_VOLUME = 0x10,       /* Master volume L+R */
    UDA1380_REG_DECIMATOR_VOLUME = 0x20,    /* Decimator (ADC) volume L+R */
};


LPCLIB_Result UDA1380_open (I2C_Handle bus, const uint8_t address, UDA1380_Handle *pHandle)
{
    UDA1380_Context *pContext = &globalContext; //TODO!


    pContext->bus = bus;
    pContext->slaveAddress = address;

    *pHandle = pContext;

    return LPCLIB_SUCCESS;
}



LPCLIB_Result UDA1380_ioctl (UDA1380_Handle handle, const UDA1380_Config *pConfig)
{
    LPCLIB_Result result = LPCLIB_SUCCESS;


    if (handle == LPCLIB_INVALID_HANDLE) {
        return LPCLIB_ILLEGAL_PARAMETER;
    }

    while (1) {
        switch (pConfig->opcode) {
        case UDA1380_OPCODE_CLOCKMODE:
            break;
        }

        if (!pConfig->continued) {
            break;
        }

        ++pConfig;                                      /* More config's follow */
    }

    return result;
}

#endif /* LPCLIB_I2C */
