/* $Id::                                                               $
 *
 * Copyright (c) 2011, NXP Semiconductors
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list
 * of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list
 * of conditions and the following disclaimer in the documentation and/or
 * other materials provided with the distribution.
 * Neither the name of the author nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/** \file
 *  \brief GPIO driver interface.
 *  This file defines all interface objects needed to use the GPIO driver.
 *
 *  \author NXP Semiconductors
 */

#ifndef __LPC43XX_GPIO_H__
#define __LPC43XX_GPIO_H__

/** \defgroup GPIO
 *  \ingroup API
 *  @{
 */


#include "lpc43xx_libconfig.h"
#include "lpclib_types.h"


/** \defgroup GPIO_Public_Types GPIO Types, enums, macros
 *  @{
 */

typedef enum GPIO_Port32 {
    GPIO_PORT0 = 0,                     /**< GPIO port 0, 32-bit access */
    GPIO_PORT1,                         /**< GPIO port 1, 32-bit access */
    GPIO_PORT2,                         /**< GPIO port 2, 32-bit access */
    GPIO_PORT3,                         /**< GPIO port 3, 32-bit access */
    GPIO_PORT4,                         /**< GPIO port 4, 32-bit access */
    GPIO_PORT5,                         /**< GPIO port 5, 32-bit access */
    GPIO_PORT6,                         /**< GPIO port 6, 32-bit access */
    GPIO_PORT7,                         /**< GPIO port 7, 32-bit access */
    __NUM_GPIO__,
} GPIO_Port32;


typedef enum GPIO_Port16 {
    GPIO_PORT0_L = (0 << 4) | (0 << 0), /**< GPIO port 0, 16-bit access, lower half-word */
    GPIO_PORT0_H = (1 << 4) | (0 << 0), /**< GPIO port 0, 16-bit access, higher half-word */
    GPIO_PORT1_L = (0 << 4) | (1 << 0), /**< GPIO port 1, 16-bit access, lower half-word */
    GPIO_PORT1_H = (1 << 4) | (1 << 0), /**< GPIO port 1, 16-bit access, higher half-word */
    GPIO_PORT2_L = (0 << 4) | (2 << 0), /**< GPIO port 2, 16-bit access, lower half-word */
    GPIO_PORT2_H = (1 << 4) | (2 << 0), /**< GPIO port 2, 16-bit access, higher half-word */
    GPIO_PORT3_L = (0 << 4) | (3 << 0), /**< GPIO port 3, 16-bit access, lower half-word */
    GPIO_PORT3_H = (1 << 4) | (3 << 0), /**< GPIO port 3, 16-bit access, higher half-word */
    GPIO_PORT4_L = (0 << 4) | (4 << 0), /**< GPIO port 4, 16-bit access, lower half-word */
    GPIO_PORT4_H = (1 << 4) | (4 << 0), /**< GPIO port 4, 16-bit access, higher half-word */
    GPIO_PORT5_L = (0 << 4) | (5 << 0), /**< GPIO port 5, 16-bit access, lower half-word */
    GPIO_PORT5_H = (1 << 4) | (5 << 0), /**< GPIO port 5, 16-bit access, higher half-word */
    GPIO_PORT6_L = (0 << 4) | (6 << 0), /**< GPIO port 6, 16-bit access, lower half-word */
    GPIO_PORT6_H = (1 << 4) | (6 << 0), /**< GPIO port 6, 16-bit access, higher half-word */
    GPIO_PORT7_L = (0 << 4) | (7 << 0), /**< GPIO port 7, 16-bit access, lower half-word */
    GPIO_PORT7_H = (1 << 4) | (7 << 0), /**< GPIO port 7, 16-bit access, higher half-word */
} GPIO_Port16;


typedef enum GPIO_Port8 {
    GPIO_PORT0_0 = (0 << 4) | (0 << 0), /**< GPIO port 0, 8-bit access, byte 0 */
    GPIO_PORT0_1 = (1 << 4) | (0 << 0), /**< GPIO port 0, 8-bit access, byte 1 */
    GPIO_PORT0_2 = (2 << 4) | (0 << 0), /**< GPIO port 0, 8-bit access, byte 2 */
    GPIO_PORT0_3 = (3 << 4) | (0 << 0), /**< GPIO port 0, 8-bit access, byte 3 */
    GPIO_PORT1_0 = (0 << 4) | (1 << 0), /**< GPIO port 1, 8-bit access, byte 0 */
    GPIO_PORT1_1 = (1 << 4) | (1 << 0), /**< GPIO port 1, 8-bit access, byte 1 */
    GPIO_PORT1_2 = (2 << 4) | (1 << 0), /**< GPIO port 1, 8-bit access, byte 2 */
    GPIO_PORT1_3 = (3 << 4) | (1 << 0), /**< GPIO port 1, 8-bit access, byte 3 */
    GPIO_PORT2_0 = (0 << 4) | (2 << 0), /**< GPIO port 2, 8-bit access, byte 0 */
    GPIO_PORT2_1 = (1 << 4) | (2 << 0), /**< GPIO port 2, 8-bit access, byte 1 */
    GPIO_PORT2_2 = (2 << 4) | (2 << 0), /**< GPIO port 2, 8-bit access, byte 2 */
    GPIO_PORT2_3 = (3 << 4) | (2 << 0), /**< GPIO port 2, 8-bit access, byte 3 */
    GPIO_PORT3_0 = (0 << 4) | (3 << 0), /**< GPIO port 3, 8-bit access, byte 0 */
    GPIO_PORT3_1 = (1 << 4) | (3 << 0), /**< GPIO port 3, 8-bit access, byte 1 */
    GPIO_PORT3_2 = (2 << 4) | (3 << 0), /**< GPIO port 3, 8-bit access, byte 2 */
    GPIO_PORT3_3 = (3 << 4) | (3 << 0), /**< GPIO port 3, 8-bit access, byte 3 */
    GPIO_PORT4_0 = (0 << 4) | (4 << 0), /**< GPIO port 4, 8-bit access, byte 0 */
    GPIO_PORT4_1 = (1 << 4) | (4 << 0), /**< GPIO port 4, 8-bit access, byte 1 */
    GPIO_PORT4_2 = (2 << 4) | (4 << 0), /**< GPIO port 4, 8-bit access, byte 2 */
    GPIO_PORT4_3 = (3 << 4) | (4 << 0), /**< GPIO port 4, 8-bit access, byte 3 */
    GPIO_PORT5_0 = (0 << 4) | (5 << 0), /**< GPIO port 5, 8-bit access, byte 0 */
    GPIO_PORT5_1 = (1 << 4) | (5 << 0), /**< GPIO port 5, 8-bit access, byte 1 */
    GPIO_PORT5_2 = (2 << 4) | (5 << 0), /**< GPIO port 5, 8-bit access, byte 2 */
    GPIO_PORT5_3 = (3 << 4) | (5 << 0), /**< GPIO port 5, 8-bit access, byte 3 */
    GPIO_PORT6_0 = (0 << 4) | (6 << 0), /**< GPIO port 6, 8-bit access, byte 0 */
    GPIO_PORT6_1 = (1 << 4) | (6 << 0), /**< GPIO port 6, 8-bit access, byte 1 */
    GPIO_PORT6_2 = (2 << 4) | (6 << 0), /**< GPIO port 6, 8-bit access, byte 2 */
    GPIO_PORT6_3 = (3 << 4) | (6 << 0), /**< GPIO port 6, 8-bit access, byte 3 */
    GPIO_PORT7_0 = (0 << 4) | (7 << 0), /**< GPIO port 7, 8-bit access, byte 0 */
    GPIO_PORT7_1 = (1 << 4) | (7 << 0), /**< GPIO port 7, 8-bit access, byte 1 */
    GPIO_PORT7_2 = (2 << 4) | (7 << 0), /**< GPIO port 7, 8-bit access, byte 2 */
    GPIO_PORT7_3 = (3 << 4) | (7 << 0), /**< GPIO port 7, 8-bit access, byte 3 */
} GPIO_Port8;


typedef enum GPIO_Pin {
    GPIO_0_0 = 0,
    GPIO_0_1,
    GPIO_0_2,
    GPIO_0_3,
    GPIO_0_4,
    GPIO_0_5,
    GPIO_0_6,
    GPIO_0_7,
    GPIO_0_8,
    GPIO_0_9,
    GPIO_0_10,
    GPIO_0_11,
    GPIO_0_12,
    GPIO_0_13,
    GPIO_0_14,
    GPIO_0_15,

    GPIO_1_0 = 32,
    GPIO_1_1,
    GPIO_1_2,
    GPIO_1_3,
    GPIO_1_4,
    GPIO_1_5,
    GPIO_1_6,
    GPIO_1_7,
    GPIO_1_8,
    GPIO_1_9,
    GPIO_1_10,
    GPIO_1_11,
    GPIO_1_12,
    GPIO_1_13,
    GPIO_1_14,
    GPIO_1_15,

    GPIO_2_0 = 64,
    GPIO_2_1,
    GPIO_2_2,
    GPIO_2_3,
    GPIO_2_4,
    GPIO_2_5,
    GPIO_2_6,
    GPIO_2_7,
    GPIO_2_8,
    GPIO_2_9,
    GPIO_2_10,
    GPIO_2_11,
    GPIO_2_12,
    GPIO_2_13,
    GPIO_2_14,
    GPIO_2_15,

    GPIO_3_0 = 96,
    GPIO_3_1,
    GPIO_3_2,
    GPIO_3_3,
    GPIO_3_4,
    GPIO_3_5,
    GPIO_3_6,
    GPIO_3_7,
    GPIO_3_8,
    GPIO_3_9,
    GPIO_3_10,
    GPIO_3_11,
    GPIO_3_12,
    GPIO_3_13,
    GPIO_3_14,
    GPIO_3_15,
    
    GPIO_4_0 = 128,
    GPIO_4_1,
    GPIO_4_2,
    GPIO_4_3,
    GPIO_4_4,
    GPIO_4_5,
    GPIO_4_6,
    GPIO_4_7,
    GPIO_4_8,
    GPIO_4_9,
    GPIO_4_10,
    GPIO_4_11,
    GPIO_4_12,
    GPIO_4_13,
    GPIO_4_14,
    GPIO_4_15,

    GPIO_5_0 = 160,
    GPIO_5_1,
    GPIO_5_2,
    GPIO_5_3,
    GPIO_5_4,
    GPIO_5_5,
    GPIO_5_6,
    GPIO_5_7,
    GPIO_5_8,
    GPIO_5_9,
    GPIO_5_10,
    GPIO_5_11,
    GPIO_5_12,
    GPIO_5_13,
    GPIO_5_14,
    GPIO_5_15,
    GPIO_5_16,
    GPIO_5_17,
    GPIO_5_18,
    GPIO_5_19,
    GPIO_5_20,
    GPIO_5_21,
    GPIO_5_22,
    GPIO_5_23,
    GPIO_5_24,
    GPIO_5_25,
    GPIO_5_26,
    
    GPIO_6_0 = 192,
    GPIO_6_1,
    GPIO_6_2,
    GPIO_6_3,
    GPIO_6_4,
    GPIO_6_5,
    GPIO_6_6,
    GPIO_6_7,
    GPIO_6_8,
    GPIO_6_9,
    GPIO_6_10,
    GPIO_6_11,
    GPIO_6_12,
    GPIO_6_13,
    GPIO_6_14,
    GPIO_6_15,
    GPIO_6_16,
    GPIO_6_17,
    GPIO_6_18,
    GPIO_6_19,
    GPIO_6_20,
    GPIO_6_21,
    GPIO_6_22,
    GPIO_6_23,
    GPIO_6_24,
    GPIO_6_25,
    GPIO_6_26,
    GPIO_6_27,
    GPIO_6_28,
    GPIO_6_29,
    GPIO_6_30,

    GPIO_7_0 = 224,
    GPIO_7_1,
    GPIO_7_2,
    GPIO_7_3,
    GPIO_7_4,
    GPIO_7_5,
    GPIO_7_6,
    GPIO_7_7,
    GPIO_7_8,
    GPIO_7_9,
    GPIO_7_10,
    GPIO_7_11,
    GPIO_7_12,
    GPIO_7_13,
    GPIO_7_14,
    GPIO_7_15,
    GPIO_7_16,
    GPIO_7_17,
    GPIO_7_18,
    GPIO_7_19,
    GPIO_7_20,
    GPIO_7_21,
    GPIO_7_22,
    GPIO_7_23,
    GPIO_7_24,
    GPIO_7_25,
} GPIO_Pin;


/** Opcodes to specify the configuration command in a call to \ref GPIO_ioctl. */
typedef enum GPIO_Opcode {
#if LPCLIB_GPIO_INTERRUPTS
    GPIO_OPCODE_SET_CALLBACK,               /**< Config action: Install callback function */
    GPIO_OPCODE_CONFIGURE_INTERRUPT,        /**< Config action: Interrupt configuration */
#endif
    __GPIO_OPCODE_INVALID__,
} GPIO_Opcode;


#if LPCLIB_GPIO_INTERRUPTS
/** GPIO interrupt mode (edge vs. level) */
typedef enum GPIO_InterruptMode {
    GPIO_INT_FALLING_EDGE = 0,              /**< GPIO interrupt on falling edge */
    GPIO_INT_RISING_EDGE,                   /**< GPIO interrupt on rising edge */
    GPIO_INT_BOTH_EDGES,                    /**< GPIO interrupt on both edges */
    GPIO_INT_LOW_LEVEL,                     /**< GPIO interrupt on low level */
    GPIO_INT_HIGH_LEVEL,                    /**< GPIO interrupt on high level */
} GPIO_InterruptMode;


/** Individual GPIO interrupt line (has its own vector). */
typedef enum GPIO_IndividualInterrupt {
    GPIO_INDIVIDUAL_INT_0 = 0,
    GPIO_INDIVIDUAL_INT_1 = 1,
    GPIO_INDIVIDUAL_INT_2 = 2,
    GPIO_INDIVIDUAL_INT_3 = 3,
    GPIO_INDIVIDUAL_INT_4 = 4,
    GPIO_INDIVIDUAL_INT_5 = 5,
    GPIO_INDIVIDUAL_INT_6 = 6,
    GPIO_INDIVIDUAL_INT_7 = 7,
} GPIO_IndividualInterrupt;


/** Configuration parameters for GPIO interrupt. */
struct GPIO_ConfigInterrupt {
    GPIO_Pin pin;                           /**< The pin to be configured */
    LPCLIB_Switch enable;                   /**< Flag: Enable interrupt */
    GPIO_InterruptMode mode;                /**< Level (vs edge) sensitive */
    GPIO_IndividualInterrupt interruptLine; /**< Selects one of eight individual interrupts */
};
#endif


/** Descriptor to specify the configuration in a call to \ref I2C_Ioctl. */
typedef struct GPIO_Config {
    GPIO_Opcode opcode;                     /**< Config action opcode */

    union {
    #if LPCLIB_GPIO_INTERRUPTS
        struct GPIO_ConfigInterrupt interrupt;          /**< Interrupts */
        LPCLIB_Callback callbackFromISR;                /**< Callback function */
    #endif
        uint8_t __dummy__;
    };
} GPIO_Config;


/** @} GPIO Types, enums, macros */

/** \defgroup GPIO_Public_Functions GPIO API Functions
 *  @{
 */


/** Prepare the use of the GPIO block.
 */
void GPIO_open (void);


/** Set GPIO pin direction for whole 32-bit port.
 *
 *  \param[in] port Port number
 *  \param[in] value 32-bit value where each bit corresponds to the pin in the
 *                   same bit position of the port. (0=input, 1=output).
 *  \param[in] mask 32-bit value with a mask for parameter \a value. Only those
 *                  bits of \a value that have a 1 in the corresponding position
 *                  of \a mask take effect.
 */
void GPIO_setDir32 (GPIO_Port32 port, uint32_t value, uint32_t mask);


/** Set GPIO pin direction for whole 32-bit port.
 *
 *  \param[in] port Pin selector
 *  \param[in] outputEnable Boolean value to enable the output.
 */
void GPIO_setDirBit (GPIO_Pin pin, LPCLIB_Switch outputEnable);


/** Set GPIO options.
 *
 *  \param[in] pConfig Config descriptor
 */
void GPIO_ioctl (const GPIO_Config *pConfig);


/** Write value to a 32-bit GPIO port.
 *
 *  \param[in] port Port selector
 *  \param[in] value 32-bit value to be written to port pins.
 */
static void GPIO_write32 (GPIO_Port32 port, uint32_t value);


/** Write value to a 16-bit GPIO port.
 *
 *  \param[in] port Port selector
 *  \param[in] value 16-bit value to be written to port pins.
 */
static void GPIO_write16 (GPIO_Port16 port, uint16_t value);


/** Write value to an 8-bit GPIO port.
 *
 *  \param[in] port Port selector
 *  \param[in] value 8-bit value to be written to port pins.
 */
static void GPIO_write8  (GPIO_Port8  port, uint8_t  value);


/** Write value (0/1) to a GPIO port pin.
 *
 *  \param[in] port Pin selector
 *  \param[in] value Binary value to be written to port pin.
 */
static void GPIO_writeBit(GPIO_Pin pin, uint8_t value);


/** Write value to a 32-bit GPIO port through a mask.
 *
 *  \param[in] port Port selector
 *  \param[in] value 32-bit value to be written to port pins.
 *  \param[in] mask 32-bit value with a mask for parameter \a value. Only those
 *                  bits of \a value that have a 1 in the corresponding position
 *                  of \a mask take effect.
 */
static void GPIO_write32WithMask (GPIO_Port32 port, uint32_t value, uint32_t mask);


/** Write value to a 16-bit GPIO port through a mask.
 *
 *  \param[in] port Port selector
 *  \param[in] value 16-bit value to be written to port pins.
 *  \param[in] mask 16-bit value with a mask for parameter \a value. Only those
 *                  bits of \a value that have a 1 in the corresponding position
 *                  of \a mask take effect.
 */
void GPIO_write16WithMask (GPIO_Port16 port, uint16_t value, uint16_t mask);


/** Write value to an 8-bit GPIO port through a mask.
 *
 *  \param[in] port Port selector
 *  \param[in] value 8-bit value to be written to port pins.
 *  \param[in] mask 8-bit value with a mask for parameter \a value. Only those
 *                  bits of \a value that have a 1 in the corresponding position
 *                  of \a mask take effect.
 */
void GPIO_write8WithMask (GPIO_Port8 port, uint8_t value, uint8_t mask);


/** Return current state of the pins of a 32-bit GPIO port.
 *
 *  \param port Port selector
 *  \return 32-bit value with state of all port pins.
 */
static uint32_t GPIO_read32 (GPIO_Port32 port);


/** Return current state of the pins of a 16-bit GPIO port.
 *
 *  \param port Port selector
 *  \return 16-bit value with state of all port pins.
 */
static uint16_t GPIO_read16 (GPIO_Port16 port);


/** Return current state of the pins of an 8-bit GPIO port.
 *
 *  \param port Port selector
 *  \return 8-bit value with state of all port pins.
 */
static uint8_t GPIO_read8 (GPIO_Port8 port);


/** Return current state of a single GPIO port pin.
 *
 *  \param port Pin selector
 *  \return (32-bit) value with state (0/1) of the port pin.
 */
static uint32_t GPIO_readBit (GPIO_Pin pin);


/** Set pins of a 32-bit GPIO port.
 *
 *  \param[in] port Port selector
 *  \param[in] value 32-bit value where a 1 in bit position n sets pin n
 *                   of the port. A zero has no effect.
 */
static void GPIO_set32 (GPIO_Port32 port, uint32_t value);


/** Set pins of a 16-bit GPIO port.
 *
 *  \param[in] port Port selector
 *  \param[in] value 16-bit value where a 1 in bit position n sets pin n
 *                   of the port. A zero has no effect.
 */
static void GPIO_set16 (GPIO_Port16 port, uint16_t value);


/** Set pins of an 8-bit GPIO port.
 *
 *  \param[in] port Port selector
 *  \param[in] value 8-bit value where a 1 in bit position n sets pin n
 *                   of the port. A zero has no effect.
 */
static void GPIO_set8 (GPIO_Port8 port, uint8_t value);


/** Clear pins of a 32-bit GPIO port.
 *
 *  \param[in] port Port selector
 *  \param[in] value 32-bit value where a 1 in bit position n clears pin n
 *                   of the port. A zero has no effect.
 */
static void GPIO_clr32 (GPIO_Port32 port, uint32_t value);


/** Clear pins of a 16-bit GPIO port.
 *
 *  \param[in] port Port selector
 *  \param[in] value 16-bit value where a 1 in bit position n clears pin n
 *                   of the port. A zero has no effect.
 */
static void GPIO_clr16 (GPIO_Port16 port, uint16_t value);


/** Clear pins of an 8-bit GPIO port.
 *
 *  \param[in] port Port selector
 *  \param[in] value 8-bit value where a 1 in bit position n clears pin n
 *                   of the port. A zero has no effect.
 */
static void GPIO_clr8 (GPIO_Port8 port, uint8_t value);


/** Clear individual GPIO line interrupt.
 *
 *  Must be called by the interrupt handler in order to acknowledge the individual
 *  GPIO line interrupt. If the interrupt line is configured for a level sensitive
 *  interrupt, the active level is toggled.
 *
 *  \param[in] interruptLine Specifies the interrupt line
 */
static void GPIO_acknowledgeIndividualInterrupt (GPIO_IndividualInterrupt interruptLine);



__FORCEINLINE(void GPIO_write32(GPIO_Port32 port, uint32_t value))
{
    LPC_GPIO_PORT->PIN[port] = value;
}

__FORCEINLINE(void GPIO_write32WithMask(GPIO_Port32 port, uint32_t value, uint32_t mask))
{
    /* NOTE: This is a thread-safe operation with respect to the affected pins, assuming only one
     *       task controls the affected pins. Other tasks may control other pins of the same port,
     *       and can do so even when scheduled to run after reading PIN, and before writing NOT.
     */
    LPC_GPIO_PORT->NOT[port & 0x0F] = (LPC_GPIO_PORT->PIN[port & 0x0F] ^ value) & mask;
}

__FORCEINLINE(void GPIO_write16(GPIO_Port16 port, uint16_t value))
{
    /* NOTE: See description of \ref GPIO_write32WithMask. */
    if ((port >> 4) == 0) {
        LPC_GPIO_PORT->NOT[port & 0x0F] = (LPC_GPIO_PORT->PIN[port & 0x0F] ^ (value << 0)) & (0xFFFFu << 0);
    }
    else {
        LPC_GPIO_PORT->NOT[port & 0x0F] = (LPC_GPIO_PORT->PIN[port & 0x0F] ^ (value << 16)) & (0xFFFFu << 16);
    }
}

__FORCEINLINE(void GPIO_write8(GPIO_Port8 port, uint8_t value))
{
    /* NOTE: See description of \ref GPIO_write32WithMask. */
    if ((port >> 4) == 0) {
        LPC_GPIO_PORT->NOT[port & 0x0F] = (LPC_GPIO_PORT->PIN[port & 0x0F] ^ (value << 0)) & (0xFF << 0);
    }
    if ((port >> 4) == 1) {
        LPC_GPIO_PORT->NOT[port & 0x0F] = (LPC_GPIO_PORT->PIN[port & 0x0F] ^ (value << 8)) & (0xFF << 8);
    }
    if ((port >> 4) == 2) {
        LPC_GPIO_PORT->NOT[port & 0x0F] = (LPC_GPIO_PORT->PIN[port & 0x0F] ^ (value << 16)) & (0xFF << 16);
    }
    if ((port >> 4) == 3) {
        LPC_GPIO_PORT->NOT[port & 0x0F] = (LPC_GPIO_PORT->PIN[port & 0x0F] ^ (value << 24)) & (0xFFu << 24);
    }
}

__FORCEINLINE(void GPIO_writeBit(GPIO_Pin pin, uint8_t value))
{
    LPC_GPIO_PORT->B[pin] = value;
}



__FORCEINLINE(uint32_t GPIO_read32(GPIO_Port32 port))
{
    return LPC_GPIO_PORT->PIN[port];
}

__FORCEINLINE(uint16_t GPIO_read16(GPIO_Port16 port))
{
    if ((port >> 4) == 0) {
        return LPC_GPIO_PORT->PIN[port & 0x0F] & 0xFFFF;
    }
    else {
        return LPC_GPIO_PORT->PIN[port & 0x0F] >> 16;
    }
}

__FORCEINLINE(uint8_t GPIO_read8(GPIO_Port8 port))
{
    if ((port >> 4) == 0) {
        return (LPC_GPIO_PORT->PIN[port & 0x0F] >> 0) & 0xFF;
    }
    else if ((port >> 4) == 1) {
        return (LPC_GPIO_PORT->PIN[port & 0x0F] >> 8) & 0xFF;
    }
    else if ((port >> 4) == 2) {
        return (LPC_GPIO_PORT->PIN[port & 0x0F] >> 16) & 0xFF;
    }
    else {  /* NOTE: We can be sure this is byte 3 because of the typed parameter 'port'! */
        return (LPC_GPIO_PORT->PIN[port & 0x0F] >> 24) & 0xFF;
    }
}

__FORCEINLINE(uint32_t GPIO_readBit(GPIO_Pin pin))
{
    return LPC_GPIO_PORT->B[pin];
}



__FORCEINLINE(void GPIO_set32 (GPIO_Port32 port, uint32_t value))
{
    LPC_GPIO_PORT->SET[port] = value;
}

__FORCEINLINE(void GPIO_set16 (GPIO_Port16 port, uint16_t value))
{
    if ((port >> 4) == 0) {
        LPC_GPIO_PORT->SET[port & 0x0F] = value;
    }
    else {
        LPC_GPIO_PORT->SET[port & 0x0F] = value << 16;
    }
}

__FORCEINLINE(void GPIO_set8 (GPIO_Port8 port, uint8_t value))
{
    if ((port >> 4) == 0) {
        LPC_GPIO_PORT->SET[port & 0x0F] = value << 0;
    }
    if ((port >> 4) == 1) {
        LPC_GPIO_PORT->SET[port & 0x0F] = value << 8;
    }
    if ((port >> 4) == 2) {
        LPC_GPIO_PORT->SET[port & 0x0F] = value << 16;
    }
    if ((port >> 4) == 3) {
        LPC_GPIO_PORT->SET[port & 0x0F] = value << 24;
    }
}

__FORCEINLINE(void GPIO_clr32 (GPIO_Port32 port, uint32_t value))
{
    LPC_GPIO_PORT->CLR[port] = value;
}

__FORCEINLINE(void GPIO_clr16 (GPIO_Port16 port, uint16_t value))
{
    if ((port >> 4) == 0) {
        LPC_GPIO_PORT->CLR[port & 0x0F] = value;
    }
    else {
        LPC_GPIO_PORT->CLR[port & 0x0F] = value << 16;
    }
}

__FORCEINLINE(void GPIO_clr8 (GPIO_Port8 port, uint8_t value))
{
    if ((port >> 4) == 0) {
        LPC_GPIO_PORT->CLR[port & 0x0F] = value << 0;
    }
    if ((port >> 4) == 1) {
        LPC_GPIO_PORT->CLR[port & 0x0F] = value << 8;
    }
    if ((port >> 4) == 2) {
        LPC_GPIO_PORT->CLR[port & 0x0F] = value << 16;
    }
    if ((port >> 4) == 3) {
        LPC_GPIO_PORT->CLR[port & 0x0F] = value << 24;
    }
}


__FORCEINLINE(void GPIO_acknowledgeIndividualInterrupt (GPIO_IndividualInterrupt interruptLine))
{
    LPC_GPIO_PIN_INT->IST = (1u << interruptLine);
}


/** @} GPIO API Functions */

/** @} GPIO */

#endif /* #ifndef __LPC43XX_GPIO_H__ */

