/* $Id::                                                               $
 *
 * Copyright (c) 2011, NXP Semiconductors
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list
 * of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list
 * of conditions and the following disclaimer in the documentation and/or
 * other materials provided with the distribution.
 * Neither the name of the author nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** \file
 *  LPC11xx Library Configuration.
 *  This file defines all compile time options that enable (disable) and/or
 *  parameterize features of the peripheral driver library.
 */


/** \defgroup LPC_LibConfig Library Configuration
 *  \ingroup LPC4300_PDL
 *  \{
 */


#ifndef LPCLIB_CONFIG_H
#define LPCLIB_CONFIG_H


/* Application config options */
#include "lpclib_config.h"



/** Device family.
 *  LPCLIB_FAMILY_LPC43XX:
 */
#define LPCLIB_FAMILY_LPC43XX   1
#ifndef LPCLIB_FAMILY
#define LPCLIB_FAMILY                                   LPCLIB_FAMILY_LPC43XX
#endif


/** Support GPIO interrupts.
 */
#ifndef LPCLIB_GPIO_INTERRUPTS
#define LPCLIB_GPIO_INTERRUPTS                          (0)
#endif



/** Crystal frequency (in Hz).
 *  Set to 0 if no crystal is used. In this case no attempt
 *  is made to start the main oscillator.
 */
#ifndef LPCLIB_CLKPWR_XTAL_FREQUENCY
#define LPCLIB_CLKPWR_XTAL_FREQUENCY                    (12000000ul)
#endif

/** PLL input clock source.
 *  This must be one of CLKPWR_OSCILLATOR_IRC or CLKPWR_OSCILLATOR_SYSTEM.
 *  (See enum type \ref CLKPWR_OSCILLATOR_t).
 */
#ifndef LPCLIB_CLKPWR_PLL_INPUT_CLOCK
#define LPCLIB_CLKPWR_PLL_INPUT_CLOCK                   CLKPWR_OSCILLATOR_SYSTEM
#endif

/** Desired CPU clock after startup (in Hz).
 *  The library attempts to set this clock frequency using the Power Profiles
 *  ROM library. If for any reason this should fail, the library will select
 *  the undivided PLL input clock as the CPU clock instead.
 */
#ifndef LPCLIB_CLKPWR_CPU_STARTUP_FREQUENCY
#define LPCLIB_CLKPWR_CPU_STARTUP_FREQUENCY             (48000000ul)
#endif



/** Include TIMER driver. */
#ifndef LPCLIB_TIMER
#define LPCLIB_TIMER                                    (1)
#endif



/** Include UART driver. */
#ifndef LPCLIB_UART
#define LPCLIB_UART                                     (1)
#endif

/** Support modem control signals. */
#ifndef LPCLIB_UART_MODEM_CONTROL
#define LPCLIB_UART_MODEM_CONTROL                       (1)
#endif

/** Support RS485 mode. */
#ifndef LPCLIB_UART_RS485
#define LPCLIB_UART_RS485                               (0)
#endif

/** Support for auto-bauding. */
#ifndef LPCLIB_UART_AUTOBAUDING
#define LPCLIB_UART_AUTOBAUDING                         (0)
#endif

/** Support handling of overflow, framing and parity errors.
 *  These error conditions are ignored if this option is disabled.
 */
#ifndef LPCLIB_UART_ERROR_HANDLING
#define LPCLIB_UART_ERROR_HANDLING                      (0)
#endif

/** Support static setting of fractional divider
 *  (when divider ratio is known a priori)
 */
#ifndef LPCLIB_UART_FRACTIONAL_STATIC
#define LPCLIB_UART_FRACTIONAL_STATIC                   (0)
#endif

/** Support dynamic setting of fractional divider
 *  (when divider ratio must be calculated by target)
 */
#ifndef LPCLIB_UART_FRACTIONAL_DYNAMIC
#define LPCLIB_UART_FRACTIONAL_DYNAMIC                  (0)
#endif

/** Size of RX buffer(s) (in bytes) */
#ifndef LPCLIB_UART_RX_BUFFER_SIZE
#define LPCLIB_UART_RX_BUFFER_SIZE                      (10)
#endif

/** Size of TX buffer(s) (in bytes) */
#ifndef LPCLIB_UART_TX_BUFFER_SIZE
#define LPCLIB_UART_TX_BUFFER_SIZE                      (10)
#endif

/** Blocking UART_Write() in case buffer is full */
#ifndef LPCLIB_UART_WRITE_BLOCKING
#define LPCLIB_UART_WRITE_BLOCKING                      (1)
#endif



/** Include I2C driver */
#ifndef LPCLIB_I2C
#define LPCLIB_I2C                                      (1)
#endif

/** Default bus clock speed (in Hz) */
#ifndef LPCLIB_I2C_DEFAULT_BUSCLOCK
#define LPCLIB_I2C_DEFAULT_BUSCLOCK                     (100000ul)
#endif

/** Support master mode */
#ifndef LPCLIB_I2C_MASTER
#define LPCLIB_I2C_MASTER                               (1)
#endif

/** I2C async mode (default is sync mode) */
#ifndef LPCLIB_I2C_ASYNC
#define LPCLIB_I2C_ASYNC                                (0)
#endif

/** Support I2C slave mode */
#ifndef LPCLIB_I2C_SLAVE
#define LPCLIB_I2C_SLAVE                                (0)
#endif

/** Number of supported slave addresses (set to either 1 or 4) */
#ifndef LPCLIB_I2C_NUM_SLAVE_ADDRESSES
#define LPCLIB_I2C_NUM_SLAVE_ADDRESSES                  (4)
#endif

/** Default slave address (2*0x01...2*0x78, 0x00=off) */
#ifndef LPCLIB_I2C_SLAVE_DEFAULT_ADDRESS
#define LPCLIB_I2C_SLAVE_DEFAULT_ADDRESS                (0x00)
#endif
/** Mask for default slave address (0xFE=all address bits relevant) */
#ifndef LPCLIB_I2C_SLAVE_DEFAULT_MASK
#define LPCLIB_I2C_SLAVE_DEFAULT_MASK                   (0x00)
#endif



/** Include SSP driver */
#ifndef LPCLIB_SSP
#define LPCLIB_SSP                                      (1)
#endif

/** Support SSP master mode. */
#ifndef LPCLIB_SSP_MASTER
#define LPCLIB_SSP_MASTER                               (1)
#endif

/** Support SSP slave mode. */
#ifndef LPCLIB_SSP_SLAVE
#define LPCLIB_SSP_SLAVE                                (0)
#endif

/** SSP async mode (default is sync mode) */
#ifndef LPCLIB_SSP_ASYNC
#define LPCLIB_SSP_ASYNC                                (0)
#endif



/** Include ADC driver */
#ifndef LPCLIB_ADC
#define LPCLIB_ADC                                      1
#endif



/** Include CAN driver */
#ifndef LPCLIB_CAN
#define LPCLIB_CAN                                      0
#endif

/** Default CAN bus bitrate (bit/s) */
#ifndef LPCLIB_CAN_DEFAULT_BITRATE
#define LPCLIB_CAN_DEFAULT_BITRATE                      (100000ul)
#endif



/** Include USB discrete driver */
#ifndef LPCLIB_USB
#define LPCLIB_USB                                      0
#endif

/** Number of 64-byte buffers available to non-control endpoints.
 *  There must be at least the number of buffers sufficient to realize all
 *  endpoints in a configuration with single-buffering.
 *  Maximum possible number of buffers for the LPC11U14/201 is 27.
 */
#ifndef LPCLIB_USB_NUM_BUFFERS
#define LPCLIB_USB_NUM_BUFFERS                          (27)
#endif

/** Include USB ROM driver */
#ifndef LPCLIB_USBROM
#define LPCLIB_USBROM                                   0
#endif



/** Include LCD (TFT) driver */
#ifndef LPCLIB_LCD
#define LPCLIB_LCD                                      0
#endif



/** Use an RTOS.
 *  If this is set, you must provide the OSAL layer (and the selected OS).
 */
#ifndef LPCLIB_RTOS
#define LPCLIB_RTOS                                     (0)
#endif


/*******************  Automatic settings  *****************/

/* Include the correct CMSIS-compatible device header.
 */
#if LPCLIB_FAMILY == LPCLIB_FAMILY_LPC43XX
    #include "LPC43xx.h"
#endif


/*******************  Check the config parameters  *****************/


/*******************  IRQ handler prototypes  *****************/

void DAC_IRQHandler (void);
void M0_IRQHandler (void);
void DMA_IRQHandler (void);
void FLASH_EEPROM_IRQHandler (void);
void ETH_IRQHandler (void);
void SDIO_IRQHandler (void);
void LCD_IRQHandler (void);
void USB0_IRQHandler (void);
void USB1_IRQHandler (void);
void SCT_IRQHandler (void);
void RIT_IRQHandler (void);
void TIMER0_IRQHandler (void);
void TIMER1_IRQHandler (void);
void TIMER2_IRQHandler (void);
void TIMER3_IRQHandler (void);
void MCPWM_IRQHandler (void);
void ADC0_IRQHandler (void);
void I2C0_IRQHandler (void);
void I2C1_IRQHandler (void);
void SPI_IRQHandler (void);
void ADC1_IRQHandler (void);
void SSP0_IRQHandler (void);
void SSP1_IRQHandler (void);
void UART0_IRQHandler (void);
void UART1_IRQHandler (void);
void UART2_IRQHandler (void);
void UART3_IRQHandler (void);
void I2S0_IRQHandler (void);
void I2S1_IRQHandler (void);
void SPIFI_IRQHandler (void);
void SGPIO_IRQHandler (void);
void GPIO0_IRQHandler (void);
void GPIO1_IRQHandler (void);
void GPIO2_IRQHandler (void);
void GPIO3_IRQHandler (void);
void GPIO4_IRQHandler (void);
void GPIO5_IRQHandler (void);
void GPIO6_IRQHandler (void);
void GPIO7_IRQHandler (void);
void GINT0_IRQHandler (void);
void GINT1_IRQHandler (void);
void EVRT_IRQHandler (void);
void CAN1_IRQHandler (void);
void VADC_IRQHandler (void);
void ATIMER_IRQHandler (void);
void RTC_IRQHandler (void);
void WDT_IRQHandler (void);
void CAN0_IRQHandler (void);
void QEI_IRQHandler (void);


#endif /* LPCLIB_CONFIG_H */

/** @} */

