/* $Id::                                                               $
 *
 * Copyright (c) 2011, NXP Semiconductors
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list
 * of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list
 * of conditions and the following disclaimer in the documentation and/or
 * other materials provided with the distribution.
 * Neither the name of the author nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** \file
 *  \brief OS interface.
 *
 *  \author NXP Semiconductors
 */

#ifndef __LPCLIB_OSAL_H
#define __LPCLIB_OSAL_H

#include "lpc43xx_libconfig.h"

#include "lpclib_types.h"


#define LPCLIB_OSAL_TIMEOUT_NONE        (0)
#define LPCLIB_OSAL_TIMEOUT_INFINITE    (0xFFFFFFFFul)



#if LPCLIB_RTOS

#include "OSAL.h"

#define LPCLIB_OSAL_ERROR                   OSAL_ERROR
#define LPCLIB_OSAL_ERROR_NONE              OSAL_ERROR_NONE
#define LPCLIB_OSAL_ERROR_NOT_SUPPORTED     OSAL_ERROR_NOT_SUPPORTED
#define LPCLIB_OSAL_ERROR_QUEUE_FULL        OSAL_ERROR_QUEUE_FULL

#define LPCLIB_OSAL_NAME                    OSAL_NAME
#define LPCLIB_OSAL_MESSAGE                 OSAL_MESSAGE
#define LPCLIB_OSAL_COUNT                   OSAL_COUNT
#define LPCLIB_OSAL_TIMEOUT                 OSAL_TIMEOUT
#define LPCLIB_OSAL_ARG                     OSAL_ARG

#define LPCLIB_OSAL_QUEUE                   OSAL_QUEUE

#define LPCLIB_OSAL_CALL_DEFAULT            OSAL_CALL_DEFAULT
#define LPCLIB_OSAL_CALL_ISR                OSAL_CALL_ISR

#define LPCLIB_DECLARE_SEMAPHORE(sema)      OSAL_SEMAPHORE sema
#define LPCLIB_DECLARE_MUTEX(mutex)         OSAL_MUTEX mutex

#define LPCLIB_OSALInit                     OSALInit
#define LPCLIB_OSALStart                    OSALStart
#define LPCLIB_OSALEnterCritical            OSALEnterCritical
#define LPCLIB_OSALExitCritical             OSALExitCritical
#define LPCLIB_OSALEnterISR(switch)         OSALEnterISR(switch)
#define LPCLIB_OSALExitISR(switch)          OSALExitISR(switch)

#define LPCLIB_OSALTaskCreate               OSALTaskCreate
#define LPCLIB_OSALTaskDelete               OSALTaskDelete

#define LPCLIB_OSALTaskDelay                OSALTaskDelay
#define LPCLIB_OSALGetTickCount             OSALGetTickCount

#define LPCLIB_OSAL_TIMER                   OSAL_TIMER
#define LPCLIB_OSAL_TIMER_ONE_SHOT          OSAL_TIMER_ONE_SHOT
#define LPCLIB_OSAL_TIMER_PERIODIC          OSAL_TIMER_PERIODIC

#define LPCLIB_OSALTimerCreate              OSALTimerCreate
#define LPCLIB_OSALTimerStart               OSALTimerStart
#define LPCLIB_OSALTimerGetMagic            OSALTimerGetMagic

#define LPCLIB_OSAL_MEMORY                  OSAL_MEMORY
#define LPCLIB_OSALMemAlloc                 OSALMemAlloc
#define LPCLIB_OSALMemRelease               OSALMemRelease

#define LPCLIB_OSALSemaphoreCreate          OSALSemaphoreCreate
#define LPCLIB_OSALSemaphoreDelete          OSALSemaphoreDelete
#define LPCLIB_OSALSemaphoreAcquire         OSALSemaphoreAcquire
#define LPCLIB_OSALSemaphoreRelease         OSALSemaphoreRelease

#define LPCLIB_OSALMutexCreate              OSALMutexCreate
#define LPCLIB_OSALMutexDelete              OSALMutexDelete

#define LPCLIB_OSALQueueCreate              OSALQueueCreate
#define LPCLIB_OSALQueuePost                OSALQueuePost
#define LPCLIB_OSALQueuePeek                OSALQueuePeek
#define LPCLIB_OSALQueuePend                OSALQueuePend

#define LPCLIB_OSAL_QUEUE_FRONT             OSAL_QUEUE_FRONT
#define LPCLIB_OSAL_QUEUE_BACK              OSAL_QUEUE_BACK

#define LPCLIB_OSAL_FALSE                   OSAL_FALSE
#define LPCLIB_OSAL_TRUE                    OSAL_TRUE

#else

typedef enum {
    LPCLIB_OSAL_ERROR_NONE = 0,
    LPCLIB_OSAL_ERROR_QUEUE_EMPTY,
    LPCLIB_OSAL_ERROR_QUEUE_FULL,
} LPCLIB_OSAL_ERROR;

#define LPCLIB_OSAL_FALSE                   (0)
#define LPCLIB_OSAL_TRUE                    (~LPCLIB_OSAL_FALSE)

typedef void* LPCLIB_OSAL_PHANDLE;
typedef LPCLIB_OSAL_PHANDLE LPCLIB_OSAL_TASK;
typedef LPCLIB_OSAL_PHANDLE LPCLIB_OSAL_SEMAPHORE;
typedef LPCLIB_OSAL_PHANDLE LPCLIB_OSAL_MUTEX;
typedef void LPCLIB_OSAL_MESSAGE;
typedef uint32_t LPCLIB_OSAL_COUNT;
typedef int32_t LPCLIB_OSAL_ARG;
typedef const int8_t LPCLIB_OSAL_NAME;
typedef uint32_t LPCLIB_OSAL_MESSAGE_SIZE;
typedef uint32_t LPCLIB_OSAL_TIMEOUT;


struct __LPCLIB_OSAL_QUEUE_IMPL {
    volatile int wrptr;
    volatile int rdptr;
    int queueSize;
    int itemSize;
    uint8_t *data;
};

typedef struct __LPCLIB_OSAL_QUEUE_IMPL * LPCLIB_OSAL_QUEUE;

#define LPCLIB_OSAL_DECLARE_QUEUE(handle,theQueueSize,theItemSize)  \
    uint8_t handle##_data[theQueueSize][theItemSize];               \
    struct __LPCLIB_OSAL_QUEUE_IMPL handle##_impl


typedef enum {
    LPCLIB_OSAL_CALL_DEFAULT,
    LPCLIB_OSAL_CALL_ISR,
} LPCLIB_OSAL_CALL_OPTION;

#define LPCLIB_DECLARE_SEMAPHORE(sema) \
    /*extern*/ LPCLIB_OSAL_SEMAPHORE sema; uint8_t sema##_value
#define LPCLIB_DECLARE_MUTEX(mutex) \
    /*extern*/ LPCLIB_OSAL_MUTEX mutex; uint8_t mutex##_value


__FORCEINLINE(void LPCLIB_OSALEnterISR (LPCLIB_OSAL_ARG *pHighPrioTaskSwitch))
{
    (void) pHighPrioTaskSwitch;
}


__FORCEINLINE(void LPCLIB_OSALExitISR (LPCLIB_OSAL_ARG *pHighPrioTaskSwitch))
{
    (void) pHighPrioTaskSwitch;
}


__FORCEINLINE(LPCLIB_OSAL_ERROR LPCLIB_OSALSemaphoreCreate(LPCLIB_OSAL_NAME *pName,
                                                           LPCLIB_OSAL_COUNT maxCount,
                                                           LPCLIB_OSAL_COUNT initCount,
                                                           LPCLIB_OSAL_SEMAPHORE *pHandle))
{
    (void) pName;
    (void) maxCount;
    (void) initCount;

    *(uint32_t *)pHandle = (uint32_t)pName; //TODO: Name is used as pointer to sema##_value...

    return LPCLIB_OSAL_ERROR_NONE;
}

__FORCEINLINE(LPCLIB_OSAL_ERROR LPCLIB_OSALSemaphoreDelete(LPCLIB_OSAL_SEMAPHORE handle))
{
    (void) handle;

    return LPCLIB_OSAL_ERROR_NONE;
}

__FORCEINLINE(LPCLIB_OSAL_ERROR LPCLIB_OSALSemaphoreAcquire(LPCLIB_OSAL_SEMAPHORE handle,
                                                            LPCLIB_OSAL_TIMEOUT timeout,
                                                            LPCLIB_OSAL_CALL_OPTION callFromISR))
{
    (void) timeout;
    (void) callFromISR;

    if (timeout != LPCLIB_OSAL_TIMEOUT_NONE) {
        while ((*(volatile uint8_t *)handle) != 0)
            ;
    }
    *(uint8_t *)handle = 1;

    return LPCLIB_OSAL_ERROR_NONE;
}

__FORCEINLINE(LPCLIB_OSAL_ERROR LPCLIB_OSALSemaphoreRelease(LPCLIB_OSAL_SEMAPHORE handle,
                                                            LPCLIB_OSAL_ARG *HighPrioTaskSwitch,
                                                            LPCLIB_OSAL_CALL_OPTION callFromISR))
{
    (void) HighPrioTaskSwitch;
    (void) callFromISR;

    *(uint8_t *)handle = 0;

    return LPCLIB_OSAL_ERROR_NONE;
}


#define LPCLIB_OSALMutexCreate(pName,pHandle) \
    LPCLIB_OSALSemaphoreCreate(pName,1,1,pHandle)


#define LPCLIB_OSALMutexDelete(handle) \
    LPCLIB_OSALSemaphoreDelete(handle)


/* NOTE: This macro only works if parameter 'pHandle' starts with a '&' character! */
#define LPCLIB_OSALQueueCreate(pName,theQueueSize,theItemSize,pHandle)  \
    do {                                                                \
        *((LPCLIB_OSAL_QUEUE *)pHandle) = pHandle##_impl;               \
        (*((LPCLIB_OSAL_QUEUE *)pHandle))->wrptr = 0;                   \
        (*((LPCLIB_OSAL_QUEUE *)pHandle))->rdptr = 0;                   \
        (*((LPCLIB_OSAL_QUEUE *)pHandle))->queueSize = theQueueSize;    \
        (*((LPCLIB_OSAL_QUEUE *)pHandle))->itemSize = theItemSize;      \
        (*((LPCLIB_OSAL_QUEUE *)pHandle))->data = pHandle##_data[0][0]; \
    } while(0)


LPCLIB_OSAL_ERROR __LPCLIB_OSAL_QueueSend (LPCLIB_OSAL_MESSAGE *pMsg,
                                           struct __LPCLIB_OSAL_QUEUE_IMPL *impl);

#define LPCLIB_OSALQueuePost(handle,pMsg,location,timeout,pHighPrioTaskSwitch,callFromISR)  \
    __LPCLIB_OSAL_QueueSend(pMsg,handle)


LPCLIB_OSAL_ERROR __LPCLIB_OSAL_QueueReceive (LPCLIB_OSAL_MESSAGE *pMsg,
                                              struct __LPCLIB_OSAL_QUEUE_IMPL *impl);

#define LPCLIB_OSALQueuePend(handle,pMsg,timeout,pHighPrioTaskSwitch,callFromISR)  \
    __LPCLIB_OSAL_QueueReceive(pMsg,handle)


#endif  /* #if LPCLIB_RTOS */


#endif /* #ifndef __LPCLIB_OSAL_H */

