/* Copyright (c) 2010, DF9DQ
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions
 * and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 * Neither the name of the author nor the names of its contributors may be used to endorse
 * or promote products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __USBD_AUDIO_H
#define __USBD_AUDIO_H

#include "usbd.h"


/* USB Class Codes and applicable sub classes and protocols */
#define USBC_AUDIO                                      (0x01)
#define   USBCS_AUDIO_SUBCLASS_UNDEFINED                    (0x00)
#define   USBCS_AUDIO_AUDIOCONTROL                          (0x01)
#define   USBCS_AUDIO_AUDIOSTREAMING                        (0x02)
#define   USBCS_AUDIO_MIDISTREAMING                         (0x03)
#define     USBCP_AUDIO_PR_PROTOCOL_UNDEFINED                   (0x00)


/* Class-specific Descriptors (and their length) */
#define USBD_AUDIO_CS_UNDEFINED                         (0x20)
#define USBD_SIZE_AUDIO_CS_UNDEFINED                    (5)
#define USBD_AUDIO_CS_DEVICE                            (0x21)
#define USBD_AUDIO_CS_CONFIGURATION                     (0x22)
#define USBD_AUDIO_CS_STRING                            (0x23)
#define USBD_AUDIO_CS_INTERFACE                         (0x24)
#define USBD_AUDIO_CS_ENDPOINT                          (0x25)


#if 0
/** Callback for audio controls. */
typedef void (*usbaudio_ControlCallback)(uint8_t classInstance, uint8_t controlId, uint16_t currentValue);

/** Callback for streaming interface control. */
typedef void (*usbaudio_StreamingInterfaceCallback)(uint8_t classInstance,
                                                    uint8_t endpoint,
                                                    uint8_t selector,
                                                    uint32_t currentValue);
#endif
                                                    

/** USB audio endpoint descriptor. */
typedef struct __attribute__((packed)) {
    uint8_t     bLength;
    uint8_t     bDescriptorType;
    uint8_t     bEndpointAddress;
    uint8_t     bmAttributes;
    uint16_t    wMaxPacketSize;
    uint8_t     bInterval;
    uint8_t     bRefresh;
    uint8_t     bSyncAddress;
} usb_EndpointDescriptorAudio;

/** Minimum, maximum and resolution for a control in a unit. */
typedef struct usbaudio_control_range_tag {
    uint8_t     unit_id;
    uint8_t     control_id;
    int16_t     min;
    int16_t     max;
    int16_t     current;
    uint16_t    resolution;
    uint8_t     length;
    LPCLIB_Callback callback;
} usbaudio_control_range;


/** USB Class-Specific AC Interface descriptor. */
typedef struct __attribute__((packed)) {
    uint8_t     bLength;
    uint8_t     bDescriptorType;
    uint8_t     bDescriptorSubtype;
    uint16_t    bcdADC;
    uint16_t    wTotalLength;
    uint8_t     bInCollection;
    /*uint8_t     baInterfaceNr[];*/ /* Must be declared separately! */
} usbaudio_ControlInterfaceDescriptor;


/** USB Audio Input Terminal Descriptor. */
typedef struct __attribute__((packed)) {
    uint8_t     bLength;
    uint8_t     bDescriptorType;
    uint8_t     bDescriptorSubtype;
    uint8_t     bTerminalID;
    uint16_t    wTerminalType;
    uint8_t     bAssocTerminal;
    uint8_t     bNrChannels;
    uint16_t    wChannelConfig;
    uint8_t     iChannelNames;
    uint8_t     iTerminal;
} usbaudio_InputTerminalDescriptor;


/** USB Audio Output Terminal Descriptor. */
typedef struct __attribute__((packed)) {
    uint8_t     bLength;
    uint8_t     bDescriptorType;
    uint8_t     bDescriptorSubtype;
    uint8_t     bTerminalID;
    uint16_t    wTerminalType;
    uint8_t     bAssocTerminal;
    uint8_t     bSourceID;
    uint8_t     iTerminal;
} usbaudio_OutputTerminalDescriptor;


/** USB Feature Unit Descriptor. */
typedef struct __attribute__((packed)) {
    uint8_t     bLength;
    uint8_t     bDescriptorType;
    uint8_t     bDescriptorSubtype;
    uint8_t     bUnitID;
    uint8_t     bSourceID;
    uint8_t     bControlSize;
    /*uint8_t       bmaControls[];*/ /* Must be declared separately! */
    /*uint8_t       iFeature;*/ /* Must be declared separately! */
} usbaudio_FeatureUnitDescriptor;


/** USB Selector Unit Descriptor. */
typedef struct __attribute__((packed)) {
    uint8_t     bLength;
    uint8_t     bDescriptorType;
    uint8_t     bDescriptorSubtype;
    uint8_t     bUnitID;
    uint8_t     bNrInPins;
    /*uint8_t       baSourceID[];*/ /* Must be declared separately! */
    /*uint8_t       iSelector;*/ /* Must be declared separately! */
} usbaudio_SelectorUnitDescriptor;


/** USB Microphone Type I Format Type Descriptor. */
typedef struct __attribute__((packed)) {
    uint8_t     bLength;
    uint8_t     bDescriptorType;
    uint8_t     bDescriptorSubtype;
    uint8_t     bFormatType;
    uint8_t     bNrChannels;
    uint8_t     bSubFrameSize;
    uint8_t     bBitResolution;
    uint8_t     bSamFreqType;
    /*uint8_t       tSamFreq[3][];*/ /* Must be declared separately! */
} usbaudio_MicrophoneType1FormatTypeDescriptor;


/** USB Audio Streaming Interface Descriptor. */
typedef struct __attribute__((packed)) {
    uint8_t     bLength;
    uint8_t     bDescriptorType;
    uint8_t     bDescriptorSubtype;
    uint8_t     bTerminalLink;
    uint8_t     bDelay;
    uint16_t    wFormatTag;
} usbaudio_StreamingInterfaceDescriptor;


/** USB Audio Data Endpoint Descriptor. */
typedef struct __attribute__((packed)) {
    uint8_t     bLength;
    uint8_t     bDescriptorType;
    uint8_t     bDescriptorSubtype;
    uint8_t     bmAttributes;
    uint8_t     bLockDelayUnits;
    uint16_t    wLockDelay;
} usbaudio_IsoAudioDataEndpointDescriptor;


/**********************************************************************************************/


/** Audio control interface */
typedef struct {
    uint8_t     interfaceNumber;
    bool        usesInterrupt;
    uint8_t     endpointNumberInterrupt;
    uint8_t     numControls;
    usbaudio_control_range *controls;
} usbaudio_ControlInterface;

/** One alternative for an audio streaming interface. */
typedef struct {
    uint8_t     numEndpoints;
    uint8_t     linkedTerminal;
    uint8_t     endpointNumber;
    uint16_t    packetSize;
    uint8_t     syncMode;
    uint8_t     endpointNumberSync;
    uint16_t    supportedControls;
    uint8_t     numSamplingFrequencies;
    const uint32_t  *samplingFrequencies;
} usbaudio_StreamingInterfaceParams;

/** Complete audio streaming interface. */
typedef struct {
    uint8_t     interfaceNumber;
    uint8_t     numAltSettings;
    uint8_t     activeSetting;
    uint32_t    currentSamplerate;
    bool        isMuted;
    const usbaudio_StreamingInterfaceParams *params;
    LPCLIB_Callback streamingCallback;
} usbaudio_StreamingInterface;


/** Complete description of an Audio Class function. */
typedef const struct {
    const usbaudio_ControlInterface *controlInterface;
    uint8_t     numStreamingInterfaces;
    usbaudio_StreamingInterface *streamingInterfaces;
} usbaudio_FunctionDeclaration;


/**********************************************************************************************/


void usbaudio_attachControls (uint8_t instance,
                              usbaudio_control_range *controls,
                              uint8_t num_controls);

/** Called when the alternate setting of an interface changes. */
void usbaudio_interfaceChange (uint8_t classInstance,
                               uint8_t interfaceNumber,
                               uint8_t newAlternateSetting);

/** The frame handler is meant to be implemented in the application. */
void usbaudio_frameHandler (uint8_t classInstance, uint16_t frameNumber);

/** Handle endpoint data transfers. */
//LPCLIB_Result usbaudio_handleEndpointData (USBDEV_Handle handle, uint8_t classInstance, uint8_t logicalEndpoint, USB_Buffer *buffer);

/** Handler for new buffer request. */
//void usbaudio_handleBufferRequest (USBDEV_Handle handle, uint8_t classInstance, uint8_t logicalEndpoint);

//void usbaudio_init (uint8_t classInstance, bool enable);


/** Initialize the module. Call before any other function. */
//bool usbaudio_initModule (uint8_t numFunctions, const usbaudio_FunctionDeclaration *functions);

#endif
