/**
 ******************************************************************************
 * @file	OSAL.h
 * @brief	Enter brief Description of this module.
 * @version	1.0
 * @date	02.06.2010
 * @author	nxp16889
 *
 * @details Enter detailed description of this module.
 *
 * ----------------------------------------------------------------------------\n
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * NXP Semiconductors assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. NXP Semiconductors
 * reserves the right to make changes in the software without
 * notification. NXP Semiconductors also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 * ----------------------------------------------------------------------------\n
 ******************************************************************************
 */
#ifndef __OSAL_H__
#define __OSAL_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "typedefs.h"

typedef	void OSAL_TASK_ARGS;
typedef U32 OSAL_PRIO;
typedef U32 OSAL_STACK_SIZE;
typedef const S8 OSAL_NAME;
typedef U32 OSAL_TICK;
typedef void OSAL_MESSAGE;
typedef U32 OSAL_COUNT;
typedef U32 OSAL_SIZE;
typedef U32 OSAL_MESSAGE_SIZE;
typedef U32 OSAL_TIMEOUT;
typedef U32 OSAL_MEM_SIZE;
typedef U8 OSAL_BUFFER;
typedef U32 OSAL_FLAG;
typedef S32 OSAL_ARG;

typedef void* PHANDLE;
typedef PHANDLE OSAL_TASK;
typedef PHANDLE OSAL_SEMAPHORE;
typedef PHANDLE	OSAL_MUTEX;
typedef PHANDLE OSAL_MAILBOX;
typedef PHANDLE	OSAL_QUEUE;
typedef PHANDLE OSAL_TIMER;
typedef PHANDLE OSAL_EVENT;
typedef PHANDLE	OSAL_PARTITION;
typedef PHANDLE OSAL_MEMORY;


#define OSAL_IDLE_TASK_PRIORITY 	(0)

#define OSAL_TIMEOUT_NONE 			(0)
#define OSAL_TIMEOUT_INFINITE 		(0xFFFFFFFF)

/**
 *******************************************************************************
 * @enum	OSAL_QUEUE_LOCATION
 * @brief
 * Enumerates the possible location when writing to a queue. FRONT means that
 * the message will be put in front of all other messages in a queue. BACK means
 * messages will be added as last message. BACK is the normal mode.
 *******************************************************************************
 */
typedef enum
{
	OSAL_QUEUE_FRONT, 	/*!< Message will be sent to front of queue.*/
	OSAL_QUEUE_BACK 	/*!< Message will be sent to back of queue.*/
} OSAL_QUEUE_LOCATION;

/**
 *******************************************************************************
 * @enum	OSAL_CALL_OPTION
 * @brief
 * Enter brief description of this enumeration here.
 *******************************************************************************
 */
typedef enum
{
	OSAL_CALL_DEFAULT, /*!< Enumeration 1*/
	OSAL_CALL_ISR /*!< Enumeration 2*/
} OSAL_CALL_OPTION;

/**
 *******************************************************************************
 * @enum	OSAL_TIMER_TYPE
 * @brief
 * Enter brief description of this enumeration here.
 *******************************************************************************
 */
typedef enum
{
	OSAL_TIMER_ONE_SHOT, 	/*!< Enumeration 1*/
	OSAL_TIMER_PERIODIC		/*!< Enumeration 2*/
} OSAL_TIMER_TYPE;

/**
 *******************************************************************************
 * @enum	OSAL_EVENT_TYPE
 * @brief
 * Enter brief description of this enumeration here.
 *******************************************************************************
 */
typedef enum
{
	OSAL_EVENT_TYPE_FIFO = 1,	/*!< Event type is type FIFO.*/
	OSAL_EVENT_TYPE_PRIO		/*!< Event type is preemptive priority.*/
} OSAL_EVENT_TYPE;

/**
 *******************************************************************************
 * @enum	OSAL_MAILBOX_TYPE
 * @brief
 * Defines the type of a mailbox which can be from type FIFO or PRIOTITY.
 *******************************************************************************
 */
typedef enum
{
	OSAL_MAILBOX_FIFO = 1,	/*!< Mailbox is from type FIFO - First in - first out*/
	OSAL_MAILBOX_PRIO		/*!< Mailbox support preemptive priority order.*/
} OSAL_MAILBOX_TYPE;

/**
 *******************************************************************************
 * @enum	OSAL_EVENT_MASK
 * @brief
 * Defines the event type, this can be single, all or any. When using ALL, a logical
 * AND operation will be performed. When using ANY, a logical OR operation
 * will be performed.
 *******************************************************************************
 */
typedef enum
{
	OSAL_EVENT_ALL, 	/*!< Wait for all events - AND operation*/
	OSAL_EVENT_ANY, 	/*!< Wait for any event - OR operation*/
	OSAL_EVENT_SINGLE	/*!< Wait for a single event*/
} OSAL_EVENT_MASK;

/**
 *******************************************************************************
 * @enum	OSAL_EVENT_RESET_TYPE
 * @brief
 * Defines the event reset type, automatically or manually.
 *******************************************************************************
 */
typedef enum
{
	OSAL_EVENT_RESET_MANUAL, 	/*!< Event must be reseted manually.*/
	OSAL_EVENT_RESET_AUTO 		/*!< Event will be reseted automatically*/
} OSAL_EVENT_RESET_TYPE;

/**
 *******************************************************************************
 * @enum	OSAL_EVENT_STATE
 * @brief
 * Enter brief description of this enumeration here.
 *******************************************************************************
 */
typedef enum
{
	OSAL_EVENT_STATE_CLEARED, 	/*!< Event is cleared, means no event occurred.*/
	OSAL_EVENT_STATE_SET		/*!< Event is set, means event occurred.*/
} OSAL_EVENT_STATE;

/**
 *******************************************************************************
 * @typedef (*OSAL_TASK_FUNCTION)(void *pArgs)
 * @brief
 * Defines the function type used for any tasks.
 *******************************************************************************
 */
typedef void (*OSAL_TASK_FUNCTION)(void *pArgs);
/**
 *******************************************************************************
 * @typedef (*OSAL_CALLBACK)(void)
 * @brief
 * Defines a callback function used i.e. for timer callback's.
 *******************************************************************************
 */
typedef void (*OSAL_CALLBACK)(OSAL_TIMER handle);


#ifndef  OSAL_FALSE
#define  OSAL_FALSE			(0)
#endif

#ifndef  OSAL_TRUE
#define  OSAL_TRUE			(~OSAL_FALSE)
#endif


/**
 *******************************************************************************
 * @enum	OSAL_ERROR
 * @brief
 * Used OS abstraction layer error codes.
 *******************************************************************************
 */
typedef enum
{
	OSAL_ERROR_NONE, 				/*!< Enumeration 1*/
	OSAL_ERROR_INVALID_HANDLE,
	OSAL_ERROR_OUT_OF_HANDLES,
	OSAL_ERROR_TIMEOUT,
	OSAL_ERROR_OUT_OF_MEMORY,
	OSAL_ERROR_NOT_ENOUGH_DATA,
	OSAL_ERROR_OUT_OF_RANGE,
	OSAL_ERROR_NOT_SUPPORTED,
	OSAL_ERROR_ILLEGAL_OPERATION,
	OSAL_ERROR_NOT_FOUND,
	OSAL_ERROR_NOT_AVAILABLE,
	OSAL_ERROR_INTERNAL_ERROR,
	OSAL_ERROR_NAK,
	OSAL_ERROR_FAILED,
	OSAL_ERROR_NOT_READY,
	OSAL_ERROR_INVALID_PARAMETER,
	OSAL_ERROR_ALREADY_EXISTS,
	OSAL_ERROR_READ_WRITE_ERROR,
	OSAL_ERROR_PROTECTED,
	OSAL_ERROR_NOT_ACTIVE,
	OSAL_ERROR_CANCELLED,
	OSAL_ERROR_OVERFLOW,
	OSAL_ERROR_STALL,
	OSAL_ERROR_BAD_CHECKSUM,
	OSAL_ERROR_INVALID,
	OSAL_ERROR_QUEUE_EMPTY,
	OSAL_ERROR_QUEUE_FULL,
	OSAL_ERROR_STACK_OUT_OF_SPACE	=	1000,
	OSAL_ERROR_TOO_MANY_TASKS,
	OSAL_ERROR_DEVICE_NOT_FOUND		=	2000,
	OSAL_ERROR_UNKNOWN_DEVICE_CMD,
	OSAL_ERROR_ILLEGAL_PARAMETER,
	OSAL_ERROR_UNKNOWN				=	-1
} OSAL_ERROR;

#endif /* __OSAL_H__ */

/*
 * author nxp16889
 *
 * OSAL_NO_GLOBALS must be set for modules which will overwrite the WEAK function prototypes!
 */
#ifdef OSAL_GLOBALS
#define __EXTERN__
#else
#define __EXTERN__ extern
#endif
/*
 ****************************************************************************
 * System Management
 ****************************************************************************
 */
__EXTERN__ OSAL_ERROR OSALInit(void);
__EXTERN__ OSAL_ERROR OSALStart(void);
__EXTERN__ void OSALEnterCritical(void);
__EXTERN__ void OSALExitCritical(void);
__EXTERN__ void OSALEnterISR(OSAL_ARG *pHighPrioTaskSwitch);
__EXTERN__ void OSALExitISR(OSAL_ARG *pHighPrioTaskSwitch);
__EXTERN__ void OSALDisableInterrupts(void);
__EXTERN__ void OSALEnableInterrupts(void);
/*
 ****************************************************************************
 * Task Management
 ****************************************************************************
 */
__EXTERN__ OSAL_ERROR OSALTaskCreate(OSAL_NAME *pName, OSAL_TASK_FUNCTION func, OSAL_STACK_SIZE stackSize, OSAL_TASK_ARGS *pArgs, OSAL_PRIO prio, OSAL_TASK *pHandle);
__EXTERN__ OSAL_ERROR OSALTaskDelete(OSAL_TASK handle);
__EXTERN__ OSAL_ERROR OSALTaskSuspend(OSAL_TASK handle);
__EXTERN__ OSAL_ERROR OSALTaskResume(OSAL_TASK handle);
__EXTERN__ OSAL_ERROR OSALTaskPrioSet(OSAL_TASK handle, OSAL_PRIO prio);
__EXTERN__ OSAL_ERROR OSALTaskPrioGet(OSAL_TASK handle, OSAL_PRIO *prio);
/*
 ****************************************************************************
 * Time Management
 ****************************************************************************
 */
__EXTERN__ OSAL_ERROR OSALTaskDelay(OSAL_TASK handle, OSAL_TICK *pStartTime, OSAL_TICK ticks);
__EXTERN__ OSAL_TICK OSALGetTickCount(void);
/*
 ****************************************************************************
 * Timer Management
 ****************************************************************************
 */
__EXTERN__ OSAL_ERROR OSALTimerCreate(OSAL_NAME *pName, OSAL_TIMER_TYPE type, PHANDLE magic, U32 initValue, U32 reloadValue, OSAL_CALLBACK callBack, OSAL_TIMER *pHandle);
__EXTERN__ OSAL_ERROR OSALTimerDelete(OSAL_TIMER handle);
__EXTERN__ OSAL_ERROR OSALTimerStart(OSAL_TIMER handle);
__EXTERN__ OSAL_ERROR OSALTimerStop(OSAL_TIMER handle);
__EXTERN__ OSAL_ERROR OSALTimerGetTime(OSAL_TIMER handle, U32 *pCount);
__EXTERN__ OSAL_ERROR OSALTimerSetTime(OSAL_TIMER handle, U32 initValue, U32 reloadValue);
__EXTERN__ OSAL_ERROR OSALTimerGetMagic(OSAL_TIMER handle, PHANDLE *pMagic);
/*
 ****************************************************************************
 * Memory Management
 ****************************************************************************
 */
__EXTERN__ OSAL_ERROR OSALMemAlloc(OSAL_SIZE size, OSAL_MEMORY *pHandle);
__EXTERN__ OSAL_ERROR OSALMemRelease(OSAL_MEMORY handle);
__EXTERN__ OSAL_ERROR OSALMemPartitionCreate(OSAL_NAME *pName, OSAL_BUFFER *pBuffer, OSAL_SIZE blockSize, OSAL_COUNT blockNumber, OSAL_PARTITION *pPartition);
__EXTERN__ OSAL_ERROR OSALMemPartitionDelete(OSAL_PARTITION handle);
__EXTERN__ OSAL_ERROR OSALMemPartitionAlloc(OSAL_PARTITION handle, OSAL_MEMORY *pBlock);
__EXTERN__ OSAL_ERROR OSALMemPartitionRelease(OSAL_PARTITION handle);
__EXTERN__ OSAL_ERROR OSALMemPartitionGetFreeBlockNum(OSAL_PARTITION handle, OSAL_COUNT *pFreeBlocks);
/*
 ****************************************************************************
 * Semaphores
 ****************************************************************************
 */
__EXTERN__ OSAL_ERROR OSALSemaphoreCreate(OSAL_NAME *pName, OSAL_COUNT maxCount, OSAL_COUNT initCount, OSAL_SEMAPHORE *pHandle);
__EXTERN__ OSAL_ERROR OSALSemaphoreDelete(OSAL_SEMAPHORE handle);
__EXTERN__ OSAL_ERROR OSALSemaphoreAcquire(OSAL_SEMAPHORE handle, OSAL_TIMEOUT timeout, OSAL_CALL_OPTION callFromISR);
__EXTERN__ OSAL_ERROR OSALSemaphoreRelease(OSAL_SEMAPHORE handle, OSAL_ARG *HighPrioTaskSwitch, OSAL_CALL_OPTION callFromISR);
/*
 ****************************************************************************
 * Mutexes
 ****************************************************************************
 */
__EXTERN__ OSAL_ERROR OSALMutexCreate(OSAL_NAME *pName, OSAL_MUTEX *pHandle);
__EXTERN__ OSAL_ERROR OSALMutexDelete(OSAL_MUTEX handle);
/*
 ****************************************************************************
 * Mailboxes
 ****************************************************************************
 */
__EXTERN__ OSAL_ERROR OSALMailboxCreate(OSAL_NAME *pName, OSAL_MAILBOX_TYPE type, OSAL_MAILBOX *pHandle);
__EXTERN__ OSAL_ERROR OSALMailboxDelete(OSAL_MAILBOX handle);
__EXTERN__ OSAL_ERROR OSALMailboxPost(OSAL_MAILBOX handle, OSAL_MESSAGE *pMsg, OSAL_CALL_OPTION callFromISR);
__EXTERN__ OSAL_ERROR OSALMailboxPend(OSAL_MAILBOX handle, OSAL_MESSAGE *pMsg, OSAL_TIMEOUT timeout);
/*
 ****************************************************************************
 * Message Queues
 ****************************************************************************
 */
__EXTERN__ OSAL_ERROR OSALQueueCreate(OSAL_NAME *pName, OSAL_COUNT queueSize, OSAL_MESSAGE_SIZE intemSize, OSAL_QUEUE *pHandle );
__EXTERN__ OSAL_ERROR OSALQueueDelete(OSAL_QUEUE handle);
__EXTERN__ OSAL_ERROR OSALQueuePost(OSAL_QUEUE handle, OSAL_MESSAGE *pMsg, OSAL_QUEUE_LOCATION location, OSAL_TIMEOUT timeout, OSAL_ARG *pHighPrioTaskSwitch, OSAL_CALL_OPTION callFromISR);
__EXTERN__ OSAL_ERROR OSALQueuePend(OSAL_QUEUE handle, OSAL_MESSAGE *pMsg, OSAL_TIMEOUT timeout,  OSAL_ARG *pHighPrioTaskSwitch, OSAL_CALL_OPTION callFromISR);
__EXTERN__ OSAL_ERROR OSALQueuePeek(OSAL_QUEUE handle, OSAL_MESSAGE *pMsg, OSAL_TIMEOUT timeout);
__EXTERN__ OSAL_ERROR OSALQueueCount(OSAL_QUEUE handle, OSAL_COUNT *pCount);
/*
 ****************************************************************************
 * Flags and Events
 ****************************************************************************
 */
__EXTERN__ OSAL_ERROR OSALEventCreate(OSAL_NAME *pName, OSAL_EVENT_RESET_TYPE resetType, OSAL_EVENT_STATE initState, OSAL_EVENT *Handle);
__EXTERN__ OSAL_ERROR OSALEventDelete(OSAL_EVENT handle);
__EXTERN__ OSAL_ERROR OSALEventPend(OSAL_EVENT handle, OSAL_EVENT_MASK eventMask, OSAL_TIMEOUT timeout, OSAL_FLAG *pEvent);
__EXTERN__ OSAL_ERROR OSALEventSet(OSAL_EVENT handle, OSAL_CALL_OPTION callFromISR);
__EXTERN__ OSAL_ERROR OSALEventClear(OSAL_EVENT handle);

/*
 ****************************************************************************
 * System Utilities
 ****************************************************************************
 */
__EXTERN__ OSAL_ERROR OSALQueueAddToRegistry(OSAL_QUEUE handle, OSAL_NAME *pName);
__EXTERN__ void OSALIdleHook(void);
__EXTERN__ void OSALStackOverflowHook(OSAL_TASK *pHandle, OSAL_NAME *pTaskName);
__EXTERN__ void OSALSysTickHook(void);

#ifdef __cplusplus
}
#endif

