/**
 ******************************************************************************
 * @file	OSAL.c
 * @brief	Enter brief Description of this module.
 * @version	1.0
 * @date	02.06.2010
 * @author	nxp16889
 *
 * @details Enter detailed description of this module.
 *
 * ----------------------------------------------------------------------------\n
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * NXP Semiconductors assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. NXP Semiconductors
 * reserves the right to make changes in the software without
 * notification. NXP Semiconductors also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 * ----------------------------------------------------------------------------\n
 ******************************************************************************
 */
#define OSAL_GLOBALS
#include "OSAL.h"
#undef	OSAL_GLOBALS

/*
 ****************************************************************************
 * author nxp16889
 *
 * Declare OSAL function which can be overwritten by the application code.
 * This functions have a empty body or have a default implementation. Be sure
 * to declare this before including a certain OS wrapper file due to references.
 ****************************************************************************
 */
WEAK void OSALIdleHook (void);
WEAK void OSALSysTickHook (void);
WEAK void OSALStackOverflowHook(OSAL_TASK *pHandle, OSAL_NAME *pTaskName);



/*
 ****************************************************************************
 * author nxp16889
 *
 * Include the corresponding OS wrapper file.
 ****************************************************************************
 */
void OSW_EnterCritical (void);
void OSW_ExitCritical (void);
void OSW_EnterISR (OSAL_ARG *pHighPrioTaskSwitch);
void OSW_ExitISR (OSAL_ARG *pHighPrioTaskSwitch);
void OSW_DisableInterrupts (void);
void OSW_EnableInterrupts (void);
OSAL_ERROR OSW_Start (void);
OSAL_TICK OSW_GetTickCount (void);
OSAL_ERROR OSW_TaskCreate (OSAL_NAME *pName,
                           OSAL_TASK_FUNCTION func,
                           OSAL_STACK_SIZE stackSize,
                           OSAL_TASK_ARGS *pArgs,
                           OSAL_PRIO prio,
                           OSAL_TASK *pHandle);
OSAL_ERROR OSW_TaskDelete (OSAL_TASK handle);
OSAL_ERROR OSW_TaskSuspend (OSAL_TASK handle);
OSAL_ERROR OSW_TaskResume (OSAL_TASK handle);
OSAL_ERROR OSW_TaskPrioSet (OSAL_TASK handle, OSAL_PRIO prio);
OSAL_ERROR OSW_TaskPrioGet (OSAL_TASK handle, OSAL_PRIO *pPrio);
OSAL_ERROR OSW_TaskDelay (OSAL_TASK handle, OSAL_TICK *pStartTime, OSAL_TICK ticks);
OSAL_ERROR OSW_QueueCreate (OSAL_NAME *pName,
                            OSAL_COUNT queueSize,
                            OSAL_MESSAGE_SIZE itemSize,
                            OSAL_QUEUE *pHandle);
OSAL_ERROR OSW_QueueDelete (OSAL_QUEUE handle);
OSAL_ERROR OSW_QueuePost (OSAL_QUEUE handle,
                          OSAL_MESSAGE *pMsg,
                          OSAL_QUEUE_LOCATION location,
                          OSAL_TIMEOUT timeout,
                          OSAL_ARG *pHighPrioTaskSwitch,
                          OSAL_CALL_OPTION callFromISR);
OSAL_ERROR OSW_QueuePend (OSAL_QUEUE handle,
                          OSAL_MESSAGE *pMsg,
                          OSAL_TIMEOUT timeout,
                          OSAL_ARG *pHighPrioTaskSwitch,
                          OSAL_CALL_OPTION callFromISR);
OSAL_ERROR OSW_QueuePeek (OSAL_QUEUE handle, OSAL_MESSAGE *pMsg, OSAL_TIMEOUT timeout);
OSAL_ERROR OSW_QueueCount (OSAL_QUEUE handle, OSAL_COUNT *pCount);
OSAL_ERROR OSW_SemaphoreCreate (OSAL_NAME *pName,
                                OSAL_COUNT maxCount,
                                OSAL_COUNT initCount,
                                OSAL_SEMAPHORE *pHandle);
OSAL_ERROR OSW_TimerCreate(OSAL_NAME *pName,
                           OSAL_TIMER_TYPE type,
                           PHANDLE magic,
                           U32 initValue,
                           U32 reloadValue,
                           OSAL_CALLBACK callBack,
                           OSAL_TIMER *pHandle);
OSAL_ERROR OSW_TimerDelete(OSAL_TIMER handle);
OSAL_ERROR OSW_TimerStart(OSAL_TIMER handle);
OSAL_ERROR OSW_TimerStop(OSAL_TIMER handle);
OSAL_ERROR OSW_TimerGetTime(OSAL_TIMER handle, U32 *pCount);
OSAL_ERROR OSW_TimerSetTime(OSAL_TIMER handle, U32 initValue, U32 reloadValue);
OSAL_ERROR OSW_TimerGetMagic(OSAL_TIMER handle, PHANDLE *pMagic);
OSAL_ERROR OSW_MemAlloc(OSAL_SIZE size, OSAL_MEMORY *pHandle);
OSAL_ERROR OSW_MemRelease(OSAL_MEMORY handle);
void OSW_SemaphoreDelete (OSAL_SEMAPHORE handle);
OSAL_ERROR OSW_SemaphoreAcquire (OSAL_SEMAPHORE handle,
                                 OSAL_TIMEOUT timeout,
                                 OSAL_CALL_OPTION callFromISR);
OSAL_ERROR OSW_SemaphoreRelease (OSAL_SEMAPHORE handle,
                                 OSAL_ARG *pHighPrioTaskSwitch,
                                 OSAL_CALL_OPTION callFromISR);
OSAL_ERROR OSW_MutexCreate (OSAL_NAME *pName, OSAL_MUTEX *pHandle);
void OSW_MutexDelete (OSAL_MUTEX handle);
OSAL_ERROR OSW_QueueAddToRegistry (OSAL_QUEUE hQueue, OSAL_NAME *pName);

#if (RTOS == FreeRTOS)
#include "OSAL_FreeRTOS.c"
#elif (RTOS == CoOS)
#include "OSAL_CoOS.c"
#elif (RTOS == MICROCOSII)
#include "OSAL_µCOSII.c"
#else
#error "No RTOS selected!"
#endif

/*
 ****************************************************************************
 * System Management Functions
 ****************************************************************************
 */
OSAL_ERROR OSALInit (void)
{
	return OSAL_ERROR_NONE;
}

OSAL_ERROR OSALStart (void)
{
	return OSW_Start();
}

void OSALEnterCritical (void)
{
	OSW_EnterCritical();
}

void OSALExitCritical (void)
{
	OSW_ExitCritical();
}

void OSALEnterISR(OSAL_ARG *pHighPrioTaskWoken)
{
	OSW_EnterISR(pHighPrioTaskWoken);
}

void OSALExitISR(OSAL_ARG *pHighPrioTaskWoken)
{
	OSW_ExitISR(pHighPrioTaskWoken);
}

void OSALDisableInterrupts()
{
	OSW_DisableInterrupts();
}

void OSALEnableInterrupts()
{
	OSW_EnableInterrupts();
}

/*
 ****************************************************************************
 * Task Management Functions
 ****************************************************************************
 */
OSAL_ERROR OSALTaskCreate(OSAL_NAME *pName,
                          OSAL_TASK_FUNCTION func,
                          OSAL_STACK_SIZE stackSize,
                          OSAL_TASK_ARGS *pArgs,
                          OSAL_PRIO prio,
                          OSAL_TASK *pHandle)
{
	return OSW_TaskCreate(pName, func, stackSize, pArgs, prio, pHandle);
}

OSAL_ERROR OSALTaskDelete(OSAL_TASK handle)
{
	return OSW_TaskDelete(handle);
}

OSAL_ERROR	OSALTaskSuspend(OSAL_TASK handle)
{
	return OSW_TaskSuspend(handle);
}

OSAL_ERROR OSALTaskResume(OSAL_TASK handle)
{
	return OSW_TaskResume(handle);
}

OSAL_ERROR OSALTaskPrioSet(OSAL_TASK handle, OSAL_PRIO prio)
{
	return OSW_TaskPrioSet(handle, prio);
}

OSAL_ERROR OSALTaskPrioGet(OSAL_TASK handle, OSAL_PRIO *pPrio)
{
	return OSW_TaskPrioGet(handle, pPrio);
}

/*
 ****************************************************************************
 * Time Management
 ****************************************************************************
 */
OSAL_ERROR OSALTaskDelay(OSAL_TASK handle, OSAL_TICK *pStartTime, OSAL_TICK ticks)
{
	return OSW_TaskDelay(handle, pStartTime, ticks);
}

OSAL_TICK OSALGetTickCount()
{
	return OSW_GetTickCount();
}

/*
 ****************************************************************************
 * Timer Management Functions
 ****************************************************************************
 */
OSAL_ERROR OSALTimerCreate(OSAL_NAME *pName, OSAL_TIMER_TYPE type, PHANDLE magic, U32 initValue, U32 reloadValue, OSAL_CALLBACK callBack, OSAL_TIMER *pHandle)
{
    return OSW_TimerCreate(pName, type, magic, initValue, reloadValue, callBack, pHandle);
}

OSAL_ERROR OSALTimerDelete(OSAL_TIMER handle)
{
    return OSW_TimerDelete(handle);
}

OSAL_ERROR OSALTimerStart(OSAL_TIMER handle)
{
    return OSW_TimerStart(handle);
}

OSAL_ERROR OSALTimerStop(OSAL_TIMER handle)
{
    return OSW_TimerStop(handle);
}

OSAL_ERROR OSALTimerGetTime(OSAL_TIMER handle, U32 *pCount)
{
    return OSW_TimerGetTime(handle, pCount);
}

OSAL_ERROR OSALTimerSetTime(OSAL_TIMER handle, U32 initValue, U32 reloadValue)
{
    return OSW_TimerSetTime(handle, initValue, reloadValue);
}

OSAL_ERROR OSALTimerGetMagic(OSAL_TIMER handle, PHANDLE *pMagic)
{
    return OSW_TimerGetMagic(handle, pMagic);
}


/*
 ****************************************************************************
 * Memory Management Functions
 ****************************************************************************
 */
OSAL_ERROR OSALMemAlloc(OSAL_SIZE size, OSAL_MEMORY *pHandle)
{
    return OSW_MemAlloc(size, pHandle);
}

OSAL_ERROR OSALMemRelease(OSAL_MEMORY handle)
{
    return OSW_MemRelease(handle);
}


/*
 ****************************************************************************
 * Semaphores
 ****************************************************************************
 */
OSAL_ERROR OSALSemaphoreCreate(OSAL_NAME *pName, OSAL_COUNT maxCount, OSAL_COUNT initCount, OSAL_SEMAPHORE *pHandle)
{
	return OSW_SemaphoreCreate(pName, maxCount, initCount, pHandle);
}

OSAL_ERROR OSALSemaphoreDelete(OSAL_SEMAPHORE handle)
{
	OSALEnterCritical();
	OSW_SemaphoreDelete(handle);
	OSALExitCritical();

	return OSAL_ERROR_NONE;
}

OSAL_ERROR OSALSemaphoreAcquire(OSAL_SEMAPHORE handle, OSAL_TIMEOUT timeout, OSAL_CALL_OPTION callFromISR)
{
	return OSW_SemaphoreAcquire(handle, timeout, callFromISR);
}

OSAL_ERROR OSALSemaphoreRelease(OSAL_SEMAPHORE handle, OSAL_ARG *pHighPrioTaskSwitch, OSAL_CALL_OPTION callFromISR)
{
	return OSW_SemaphoreRelease(handle, pHighPrioTaskSwitch, callFromISR);
}

/*
 ****************************************************************************
 * Mutexes
 ****************************************************************************
 */
OSAL_ERROR OSALMutexCreate(OSAL_NAME *pName, OSAL_MUTEX *pHandle)
{
	return OSW_MutexCreate(pName, pHandle);
}

OSAL_ERROR OSALMutexDelete(OSAL_MUTEX handle)
{
	OSALEnterCritical();
	OSW_MutexDelete(handle);
	OSALExitCritical();

	return OSAL_ERROR_NONE;
}

/*
 ****************************************************************************
 * Mailboxes
 ****************************************************************************
 */

/*
 ****************************************************************************
 * Message Queues
 ****************************************************************************
 */
OSAL_ERROR	OSALQueueCreate(OSAL_NAME *pName,
          	                OSAL_COUNT queueSize,
          	                OSAL_MESSAGE_SIZE itemSize,
          	                OSAL_QUEUE *pHandle )
{
	return OSW_QueueCreate(pName, queueSize, itemSize, pHandle);
}

OSAL_ERROR OSALQueueDelete(OSAL_QUEUE handle)
{
	return OSW_QueueDelete(handle);
}

OSAL_ERROR OSALQueuePost(OSAL_QUEUE handle,
                         OSAL_MESSAGE *pMsg,
                         OSAL_QUEUE_LOCATION location,
                         OSAL_TIMEOUT timeout,
                         OSAL_ARG *pHighPrioTaskSwitch,
                         OSAL_CALL_OPTION callFromISR)
{
	return OSW_QueuePost(handle, pMsg, location, timeout, pHighPrioTaskSwitch, callFromISR);
}

OSAL_ERROR OSALQueuePend(OSAL_QUEUE handle,
                            OSAL_MESSAGE *pMsg,
                            OSAL_TIMEOUT timeout,
                            OSAL_ARG *pHighPrioTaskSwitch,
                            OSAL_CALL_OPTION callFromISR)
{
	return OSW_QueuePend(handle, pMsg, timeout, pHighPrioTaskSwitch, callFromISR);
}

OSAL_ERROR OSALQueuePeek(OSAL_QUEUE handle, OSAL_MESSAGE *pMsg, OSAL_TIMEOUT timeout)
{
	return OSW_QueuePeek(handle, pMsg, timeout);
}

OSAL_ERROR OSALQueueCount(OSAL_QUEUE handle, OSAL_COUNT *pCount)
{
	return OSW_QueueCount(handle, pCount);
}

/*
 ****************************************************************************
 * Event Management Functions
 ****************************************************************************
 */




/*
 ****************************************************************************
 * System Utilities
 ****************************************************************************
 */
OSAL_ERROR OSALQueueAddToRegistry(OSAL_QUEUE handle, OSAL_NAME *pName)
{
	return OSW_QueueAddToRegistry(handle, pName);
}

/**
 **********************************************************************
 * @fn			void OSALIdleHook()
 *
 * @brief		OSAL's IdleHook function which can be overwritten by the
 * 				application code. Doing this just implement this function
 * 				with exactly the same name.
 *
 * @return 		void
 *********************************************************************
 */
void OSALIdleHook()
{
}

/**
 **********************************************************************
 * @fn			void OSALStackOverflowHook(OSAL_TASK *pHandle, OSAL_NAME *pTaskName)
 *
 * @brief		OSAL's StackOverflowHook function with default implementation.
 * 				This function can be overwritten by the application layer.
 * 				Doing this implement the function with exactly the same
 * 				name.
 *
 * @param[in]	*pHandle 	Task handle.
 * @param[in]	pTaskName 	Pointer to the Task Name.
 *
 * @return 		void
 *********************************************************************
 */
void OSALStackOverflowHook(OSAL_TASK *pHandle, OSAL_NAME *pTaskName)
{
	( void ) pHandle;
	( void ) pTaskName;

	for( ;; );
}

/**
 **********************************************************************
 * @fn			void OSALSysTickHook()
 *
 * @brief		OSAL's SysTick timer hook function as default implementation.
 * 				Application layer can overwrite this function and
 * 				implement the needed action. For overwriting this
 * 				function, just implement this with exactly the same name
 * 				and build project.
 *
 * @return 		void
 *********************************************************************
 */
void OSALSysTickHook()
{
}
