;/*
;    FreeRTOS V7.0.2 - Copyright (C) 2011 Real Time Engineers Ltd.
;	
;
;    ***************************************************************************
;     *                                                                       *
;     *    FreeRTOS tutorial books are available in pdf and paperback.        *
;     *    Complete, revised, and edited pdf reference manuals are also       *
;     *    available.                                                         *
;     *                                                                       *
;     *    Purchasing FreeRTOS documentation will not only help you, by       *
;     *    ensuring you get running as quickly as possible and with an        *
;     *    in-depth knowledge of how to use FreeRTOS, it will also help       *
;     *    the FreeRTOS project to continue with its mission of providing     *
;     *    professional grade, cross platform, de facto standard solutions    *
;     *    for microcontrollers - completely free of charge!                  *
;     *                                                                       *
;     *    >>> See http://www.FreeRTOS.org/Documentation for details. <<<     *
;     *                                                                       *
;     *    Thank you for using FreeRTOS, and thank you for your support!      *
;     *                                                                       *
;    ***************************************************************************
;
;
;    This file is part of the FreeRTOS distribution.
;
;    FreeRTOS is free software; you can redistribute it and/or modify it under
;    the terms of the GNU General Public License (version 2) as published by the
;    Free Software Foundation AND MODIFIED BY the FreeRTOS exception.
;    >>>NOTE<<< The modification to the GPL is included to allow you to
;    distribute a combined work that includes FreeRTOS without being obliged to
;    provide the source code for proprietary components outside of the FreeRTOS
;    kernel.  FreeRTOS is distributed in the hope that it will be useful, but
;    WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;    or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;    more details. You should have received a copy of the GNU General Public
;    License and the FreeRTOS license exception along with FreeRTOS; if not it
;    can be viewed here: http://www.freertos.org/a00114.html and also obtained
;    by writing to Richard Barry, contact details for whom are available on the
;    FreeRTOS WEB site.
;
;    http://www.FreeRTOS.org - Documentation, latest information, license and
;    contact details.
;
;    http://www.SafeRTOS.com - A version that is certified for use in safety
;    critical systems.
;
;    http://www.OpenRTOS.com - Commercial support, development, porting,
;    licensing and training services.
;*/


        EXPORT  vPortSetInterruptMask
        EXPORT  vPortClearInterruptMask

NVIC                        EQU     0xE000E100
NVIC_ISER_Offset            EQU     0x0000
NVIC_ICER_Offset            EQU     0x0080

SysTick                     EQU     0xE000E010
SysTick_CTRL_Offset         EQU     0x0000
SysTick_CTRL_TICKINT_Pos    EQU     1
SysTick_CTRL_TICKINT_Msk    EQU     (1 << SysTick_CTRL_TICKINT_Pos)


        AREA    |.bss|, DATA

savedNVIC_ISER              DCD     1


        THUMB
        AREA    PORT_ASM, CODE


; Mask all interrupts that mustn't be active in a critical section.
;
;  This function expects a bitmask in configBITMASK_SYSCALL_IRQ that must have
;  a bit set in every position of an ISR which uses the FreeRTOS API.
;  In addition to these external interrupts, the SysTick interrupt must be
;  disabled as well.
;  Disabling external interrupts in the NVIC with a mask is not an atomic operation.
;  If the application is expected to dynamically enable/disable external interrupts,
;  part of this function must be a hard critical section, where all interrupts 
;  are disabled globally.
;  Another reason for global interupt disable is this:
;  Assume SysTick would be enabled first (external interrupts still enabled),
;  and an external interrupt arrived immediately afterwards, causing a task switch.
;  The software might then end up waiting for some semaphore in a high-priority task
;  with no chance of ever being preempted again.
;
vPortSetInterruptMask

        PRESERVE8

;        push    {r1-r3}

        ; Disable selected external interrupts
        ldr     r1, =(NVIC + NVIC_ICER_Offset)
        ldr     r3, =SysTick

        cpsid   i                                       ;1

        ldr     r2, [r1]                                ;2
        str     r0, [r1]                                ;2

        ; Block SysTick interrupts
        ldr     r0, [r3, #SysTick_CTRL_Offset]          ;2
        movs    r1, #SysTick_CTRL_TICKINT_Msk           ;1
        bics    r0, r0, r1                              ;1
        str     r0, [r3, #SysTick_CTRL_Offset]          ;2

        cpsie   i                                       ;1

        ldr     r1, =savedNVIC_ISER
        str     r2, [r1]

;        pop     {r1-r3}

        bx      lr



vPortClearInterruptMask

        PRESERVE8

 ;       push    {r1-r3}

        ; Re-enable SysTick interrupts
        ldr     r1, =SysTick
        ldr     r3, [r1, #SysTick_CTRL_Offset]
        movs    r2, #SysTick_CTRL_TICKINT_Msk
        orrs    r3, r3, r2
        str     r3, [r1, #SysTick_CTRL_Offset]

        ; Re-enable selected external interrupts
        ldr     r1, =savedNVIC_ISER
        ldr     r2, [r1]
        ands    r0, r0, r2
        ldr     r1, =NVIC
        str     r0, [r1, #NVIC_ISER_Offset]

 ;       pop     {r1-r3}

        bx      lr

        ALIGN

        END

